@@ -10,11 +10,11 @@ vendor="389 Project"
 # PACKAGE_VERSION is constructed from these
 VERSION_MAJOR=1
 VERSION_MINOR=3
-VERSION_MAINT=1.9
+VERSION_MAINT=2.13
 # if this is a PRERELEASE, set VERSION_PREREL
 # otherwise, comment it out
 # be sure to include the dot prefix in the prerel
-# VERSION_PREREL=.pre.a1
+# VERSION_PREREL=.a1
 # NOTES on VERSION_PREREL
 # use aN for an alpha release e.g. a1, a2, etc.
 # use rcN for a release candidate e.g. rc1, rc2, etc.
@@ -22,6 +22,13 @@ VERSION_MAINT=1.9
 # RC1.`date +%Y%m%d`
 # a git commit may also be used
 
+# Set the version and release numbers for local developer RPM builds. We
+# set these here because we do not want the git commit hash in the RPM
+# version since it can make RPM upgrades difficult.  If we have a git
+# commit hash, we add it into the release number below.
+RPM_RELEASE=`date -u +%Y%m%d%H%M%S`
+RPM_VERSION=$VERSION_MAJOR.$VERSION_MINOR.${VERSION_MAINT}$VERSION_PREREL
+
 if test -n "$VERSION_PREREL"; then
 # if the source is from a git repo, put the last commit
 # in the version
@@ -32,6 +39,7 @@ if test -n "$VERSION_PREREL"; then
     COMMIT=`cd $srcdir ; git log -1 --pretty=format:%h 2> /dev/null`
     if test -n "$COMMIT" ; then
         VERSION_PREREL=$VERSION_PREREL.git$COMMIT
+        RPM_RELEASE=$RPM_RELEASE.git$COMMIT
     fi
 fi
 
