@@ -145,6 +145,7 @@ static char* const access_str_proxy 	= "proxy";
 #define DS_LAS_GROUP 		"group"
 #define DS_LAS_USERDN		"userdn"
 #define DS_LAS_GROUPDN		"groupdn"
+#define	DS_LAS_SELFDNATTR	"selfdnattr"
 #define	DS_LAS_USERDNATTR	"userdnattr"
 #define	DS_LAS_AUTHMETHOD	"authmethod"
 #define	DS_LAS_GROUPDNATTR	"groupdnattr"
@@ -209,6 +210,7 @@ typedef enum
 #define DS_ATTR_AUTHTYPE	"authtype"
 #define DS_ATTR_CERT		"clientcert"
 #define DS_ATTR_SSF		"ssf"
+#define DS_ATTR_LDAPI		"ldapi"
 
 #define ACL_ANOM_MAX_ACL 40
 struct scoped_entry_anominfo {
@@ -469,13 +471,14 @@ struct acl_pblock {
 #define	ACLPB_UPD_ACLCB_CACHE				0x100000
 #define	ACLPB_ATTR_RULE_EVALUATED			0x200000
 #define ACLPB_DONOT_EVALUATE_PROXY			0x400000
+#define ACLPB_CACHE_RESULT_PER_ENTRY_SKIP               0x800000
 
 
 #define ACLPB_RESET_MASK ( ACLPB_ACCESS_ALLOWED_ON_A_ATTR | ACLPB_ACCESS_DENIED_ON_ALL_ATTRS | \
 			   ACLPB_ACCESS_ALLOWED_ON_ENTRY | ACLPB_ATTR_STAR_MATCHED |  \
 			   ACLPB_FOUND_ATTR_RULE | ACLPB_EVALUATING_FIRST_ATTR |  \
 			   ACLPB_FOUND_A_ENTRY_TEST_RULE )
-#define ACLPB_STATE_ALL				0x3fffff
+#define ACLPB_STATE_ALL				0xffffff
 
 	int						aclpb_res_type;
 
@@ -664,6 +667,7 @@ typedef struct {
 	Acl_PBlock	*aclpb;
 	Slapi_Entry	*resourceEntry;
 	int		ssf;
+	char		*ldapi;
 }lasInfo;
 
 
@@ -805,14 +809,17 @@ int 		acl_access_allowed_main ( Slapi_PBlock *pb, Slapi_Entry *e, char **attrs,
                           		  struct berval *val, int access , int flags, char **errbuf);
 int 		acl_access_allowed( Slapi_PBlock *pb, Slapi_Entry *e, char *attr,
 				        	struct berval *val, int access );
-int 		acl_verify_syntax(const Slapi_DN *e_sdn, const struct berval *bval, char **errbuf);
 aclUserGroup * acl_get_usersGroup ( struct acl_pblock *aclpb , char *n_dn);
 void		acl_print_acllib_err (NSErr_t *errp , char * str);	
 int 		acl_check_mods( Slapi_PBlock *pb, Slapi_Entry *e, LDAPMod **mods, char **errbuf );
-int 		acl_verify_aci_syntax (Slapi_Entry *e, char **errbuf);
 char * 		acl__access2str(int access);
 void		acl_strcpy_special (char *d, char *s);
-int			acl_parse(char *str, aci_t *aci_item, char **errbuf);
+int		acl_parse(Slapi_PBlock *pb, char * str, aci_t *aci_item, char **errbuf);
+int		acl_verify_aci_syntax (Slapi_PBlock *pb, Slapi_Entry *e, char **errbuf);
+int		acl_verify_syntax(Slapi_PBlock *pb, const Slapi_DN *e_sdn,
+			const struct berval *bval, char **errbuf);
+int		acllist_insert_aci_needsLock_ext( Slapi_PBlock *pb, const Slapi_DN *e_sdn,
+			const struct berval* aci_attr);
 char *		acl_access2str ( int access );
 int 		acl_init_ext ();
 void * 		acl_get_ext (ext_type type, void *object);
@@ -861,7 +868,7 @@ void		acllist_acicache_WRITE_UNLOCK(void);
 void		acllist_acicache_WRITE_LOCK(void);
 void		acllist_aciscan_update_scan ( Acl_PBlock *aclpb, char *edn );
 int 		acllist_remove_aci_needsLock( const Slapi_DN *sdn,  const struct berval *attr );
-int 		acllist_insert_aci_needsLock( const Slapi_DN *e_sdn, const struct berval* aci_attr);
+int		acllist_insert_aci_needsLock( const Slapi_DN *e_sdn, const struct berval* aci_attr);
 int 		acllist_init ();
 int			acllist_moddn_aci_needsLock ( Slapi_DN *oldsdn, char *newdn );
 void		acllist_print_tree ( Avlnode *root, int *depth, char *start, char *side);
