@@ -96,6 +96,7 @@ aclutil_str_append_ext(char **dest, size_t *dlen, const char *src, size_t slen)
     char *ptr = NULL;
     int rc = 0;
 
+    PR_ASSERT(NULL != dlen);
     if ( dest == NULL || src == NULL ) {
         return rc;    
     }
@@ -103,7 +104,7 @@ aclutil_str_append_ext(char **dest, size_t *dlen, const char *src, size_t slen)
     if (0 == slen) {
         slen = strlen(src);
     }
-    if (*dest && dlen > 0) {
+    if (*dest && *dlen > 0) {
         size_t dest_strlen = strlen(*dest);
         size_t new_len = dest_strlen + slen + 1;
         if (new_len > *dlen) {
@@ -497,7 +498,7 @@ aclutil_gen_signature ( short c_signature )
 }
 
 void 
-aclutil_print_resource( struct acl_pblock *aclpb, char *right , char *attr, char *clientdn )
+aclutil_print_resource( struct acl_pblock *aclpb, const char *right , char *attr, char *clientdn )
 {
 
 	char		str[BUFSIZ];
@@ -1246,7 +1247,7 @@ acl_replace_str(char * s, char *substr, char* replace_with_str) {
 		char *working_s, *suffix, *prefix, *patched;
 		int replace_with_len, substr_len, prefix_len, suffix_len;
 
-		if (strstr(s, substr) == NULL) {
+		if (strcasestr(s, substr) == NULL) {
 			return(slapi_ch_strdup(s));
 		} else {
 
@@ -1256,7 +1257,7 @@ acl_replace_str(char * s, char *substr, char* replace_with_str) {
 		
 			working_s = slapi_ch_strdup(s);	
 			prefix = working_s;
-			str = strstr(prefix, substr);
+			str = strcasestr(prefix, substr);
 			
 			while (str != NULL) {
 				
@@ -1283,7 +1284,7 @@ acl_replace_str(char * s, char *substr, char* replace_with_str) {
 
 				working_s = patched;
 				prefix = working_s;
-				str = strstr(prefix, substr);		
+				str = strcasestr(prefix, substr);		
 				
 			}
 
@@ -1396,6 +1397,16 @@ void acl_ht_add_and_freeOld(acl_ht_t * acl_ht,
 	PL_HashTableAdd( acl_ht, (const void *)pkey, value);
 }
 
+void acl_ht_remove_and_free(acl_ht_t * acl_ht,
+					PLHashNumber key){
+	char *old_value = NULL;	
+
+	if ( (old_value = (char *)acl_ht_lookup( acl_ht, key)) != NULL ) {
+		acl_ht_remove( acl_ht, key);
+		slapi_ch_free_string(&old_value);
+	}
+}
+
 /*
  * Return a new acl_ht_t *
 */
