@@ -220,10 +220,8 @@ int cb_update_controls( Slapi_PBlock * pb,
             ctrls[dCount]=slapi_dup_control(reqControls[cCount]);
             dCount++;
 
-        } else
-            if (!strcmp(reqControls[cCount]->ldctl_oid,CB_LDAP_CONTROL_CHAIN_SERVER) &&
-                reqControls[cCount]->ldctl_value.bv_val) {
-
+        } else if (!strcmp(reqControls[cCount]->ldctl_oid,CB_LDAP_CONTROL_CHAIN_SERVER) &&
+                   BV_HAS_DATA((&(reqControls[cCount]->ldctl_value)))) {
             /* Max hop count reached ?                 */
             /* Checked earlier by a call to cb_forward_operation()  */
 
