@@ -113,13 +113,19 @@ struct clc_buffer {
 	CSN			*buf_missing_csn;	/* used to detect persistent missing of CSN */
 
 	/* fields for control the CSN sequence sent to the consumer */
-	struct csn_seq_ctrl_block *buf_cscbs [MAX_NUM_OF_MASTERS];
+	struct csn_seq_ctrl_block **buf_cscbs;
 	int			 buf_num_cscbs;		/* number of csn sequence ctrl blocks */
+	int			 buf_max_cscbs;
 
 	/* fields for debugging stat */
 	int		 	 buf_load_cnt;		/* number of loads for session */
 	int		 	 buf_record_cnt;	/* number of changes for session */
 	int		 	 buf_record_skipped;	/* number of changes skipped */
+	int		 	 buf_skipped_new_rid;	/* number of changes skipped due to new_rid */
+	int		 	 buf_skipped_csn_gt_cons_maxcsn;	/* number of changes skipped due to csn greater than consumer maxcsn */
+	int		 	 buf_skipped_up_to_date;	/* number of changes skipped due to consumer being up-to-date for the given rid */
+	int		 	 buf_skipped_csn_gt_ruv;	/* number of changes skipped due to preceedents are not covered by local RUV snapshot */
+	int		 	 buf_skipped_csn_covered;	/* number of changes skipped due to CSNs already covered by consumer RUV */
 
 	/*
 	 * fields that should be accessed via bl_lock or pl_lock
@@ -251,7 +257,15 @@ clcache_get_buffer ( CLC_Buffer **buf, DB *db, ReplicaId consumer_rid, const RUV
 		(*buf)->buf_record_cnt = 0;
 		(*buf)->buf_record_skipped = 0;
 		(*buf)->buf_cursor = NULL;
+		(*buf)->buf_skipped_new_rid = 0;
+		(*buf)->buf_skipped_csn_gt_cons_maxcsn = 0;
+		(*buf)->buf_skipped_up_to_date = 0;
+		(*buf)->buf_skipped_csn_gt_ruv = 0;
+		(*buf)->buf_skipped_csn_covered = 0;
+		(*buf)->buf_cscbs  = (struct csn_seq_ctrl_block **) slapi_ch_calloc(MAX_NUM_OF_MASTERS + 1,
+			                 sizeof(struct csn_seq_ctrl_block *));
 		(*buf)->buf_num_cscbs = 0;
+		(*buf)->buf_max_cscbs = MAX_NUM_OF_MASTERS;
 	}
 	else {
 		*buf = clcache_new_buffer ( consumer_rid );
@@ -287,16 +301,21 @@ clcache_return_buffer ( CLC_Buffer **buf )
 	int i;
 
 	slapi_log_error ( SLAPI_LOG_REPL, (*buf)->buf_agmt_name,
-			"session end: state=%d load=%d sent=%d skipped=%d\n",
-			 (*buf)->buf_state,
-			 (*buf)->buf_load_cnt,
-			 (*buf)->buf_record_cnt - (*buf)->buf_record_skipped,
-			 (*buf)->buf_record_skipped );
+			  "session end: state=%d load=%d sent=%d skipped=%d skipped_new_rid=%d "
+			  "skipped_csn_gt_cons_maxcsn=%d skipped_up_to_date=%d "
+			  "skipped_csn_gt_ruv=%d skipped_csn_covered=%d\n",
+			  (*buf)->buf_state,
+			  (*buf)->buf_load_cnt,
+			  (*buf)->buf_record_cnt - (*buf)->buf_record_skipped,
+			  (*buf)->buf_record_skipped, (*buf)->buf_skipped_new_rid,
+			  (*buf)->buf_skipped_csn_gt_cons_maxcsn,
+			  (*buf)->buf_skipped_up_to_date, (*buf)->buf_skipped_csn_gt_ruv,
+			  (*buf)->buf_skipped_csn_covered);
 
 	for ( i = 0; i < (*buf)->buf_num_cscbs; i++ ) {
 		clcache_free_cscb ( &(*buf)->buf_cscbs[i] );
 	}
-	(*buf)->buf_num_cscbs = 0;
+	slapi_ch_free((void **)&(*buf)->buf_cscbs);
 
 	if ( (*buf)->buf_cursor ) {
 
@@ -539,7 +558,7 @@ clcache_refresh_consumer_maxcsns ( CLC_Buffer *buf )
 static int
 clcache_refresh_local_maxcsn ( const ruv_enum_data *rid_data, void *data )
 {
-	CLC_Buffer *buf = (CLC_Buffer*) data;
+	struct clc_buffer *buf = (struct clc_buffer*) data;
 	ReplicaId rid;
 	int rc = 0;
 	int i;
@@ -560,7 +579,12 @@ clcache_refresh_local_maxcsn ( const ruv_enum_data *rid_data, void *data )
 			break;
 	}
 	if ( i >= buf->buf_num_cscbs ) {
-		buf->buf_cscbs[i] = clcache_new_cscb ();
+		if( i + 1 > buf->buf_max_cscbs){
+			buf->buf_cscbs = (struct csn_seq_ctrl_block **) slapi_ch_realloc((char *)buf->buf_cscbs,
+							 (i + 2) * sizeof(struct csn_seq_ctrl_block *));
+			buf->buf_max_cscbs = i + 1;
+		}
+		buf->buf_cscbs[i] = clcache_new_cscb();
 		if ( buf->buf_cscbs[i] == NULL ) {
 			return -1;
 		}
@@ -676,6 +700,8 @@ clcache_skip_change ( CLC_Buffer *buf )
 	ReplicaId rid;
 	int skip = 1;
 	int i;
+	char buf_cur_csn_str[CSN_STRSIZE];
+	char oth_csn_str[CSN_STRSIZE];
 
 	do {
 
@@ -697,6 +723,14 @@ clcache_skip_change ( CLC_Buffer *buf )
 				 *  The consumer must have been "restored" and needs this newer update.
 				 */
 				skip = 0;
+			} else if (slapi_is_loglevel_set(SLAPI_LOG_REPL)) {
+				csn_as_string(buf->buf_current_csn, 0, buf_cur_csn_str);
+				csn_as_string(cons_maxcsn, 0, oth_csn_str);
+				slapi_log_error(SLAPI_LOG_REPL, buf->buf_agmt_name,
+					"Skipping update because the changelog buffer current csn [%s] is "
+				        "less than or equal to the consumer max csn [%s]\n",
+				        buf_cur_csn_str, oth_csn_str);
+				buf->buf_skipped_csn_gt_cons_maxcsn++;
 			}
 			csn_free(&cons_maxcsn);
 			break;
@@ -714,7 +748,14 @@ clcache_skip_change ( CLC_Buffer *buf )
 
 		/* Skip CSN whose RID is unknown to the local RUV snapshot */
 		if ( i >= buf->buf_num_cscbs ) {
-			buf->buf_state = CLC_STATE_NEW_RID;
+			if (slapi_is_loglevel_set(SLAPI_LOG_REPL)) {
+				csn_as_string(buf->buf_current_csn, 0, buf_cur_csn_str);
+				slapi_log_error(SLAPI_LOG_REPL, buf->buf_agmt_name,
+					"Skipping update because the changelog buffer current csn [%s] rid "
+				        "[%d] is not in the list of changelog csn buffers (length %d)\n",
+				        buf_cur_csn_str, rid, buf->buf_num_cscbs);
+			}
+			buf->buf_skipped_new_rid++;
 			break;
 		}
 
@@ -722,17 +763,20 @@ clcache_skip_change ( CLC_Buffer *buf )
 
 		/* Skip if the consumer is already up-to-date for the RID */
 		if ( cscb->state == CLC_STATE_UP_TO_DATE ) {
+			buf->buf_skipped_up_to_date++;
 			break;
 		}
 
 		/* Skip CSN whose preceedents are not covered by local RUV snapshot */
 		if ( cscb->state == CLC_STATE_CSN_GT_RUV ) {
+			buf->buf_skipped_csn_gt_ruv++;
 			break;
 		}
 
 		/* Skip CSNs already covered by consumer RUV */
 		if ( cscb->consumer_maxcsn &&
 			 csn_compare ( buf->buf_current_csn, cscb->consumer_maxcsn ) <= 0 ) {
+			buf->buf_skipped_csn_covered++;
 				break;
 		}
 
@@ -762,6 +806,7 @@ clcache_skip_change ( CLC_Buffer *buf )
 
 		/* Skip CSNs not covered by local RUV snapshot */
 		cscb->state = CLC_STATE_CSN_GT_RUV;
+		buf->buf_skipped_csn_gt_ruv++;
 
 	} while (0);
 
@@ -842,6 +887,9 @@ clcache_new_buffer ( ReplicaId consumer_rid )
 		buf->buf_agmt_name = get_thread_private_agmtname();
 		buf->buf_consumer_rid = consumer_rid;
 		buf->buf_num_cscbs = 0;
+		buf->buf_max_cscbs = MAX_NUM_OF_MASTERS;
+		buf->buf_cscbs  = (struct csn_seq_ctrl_block **) slapi_ch_calloc(MAX_NUM_OF_MASTERS + 1,
+					sizeof(struct csn_seq_ctrl_block *));
 
 		welldone = 1;
 
