@@ -111,8 +111,9 @@ typedef struct repl5agmt {
 	const Slapi_RDN *rdn; /* RDN of replication agreement entry */
 	char *long_name; /* Long name (rdn + host, port) of entry, for logging */
 	Repl_Protocol *protocol; /* Protocol object - manages protocol */
-	struct changecounter *changecounters[MAX_NUM_OF_MASTERS]; /* changes sent/skipped since server start up */
+	struct changecounter **changecounters; /* changes sent/skipped since server start up */
 	int num_changecounters;
+	int max_changecounters;
 	time_t last_update_start_time; /* Local start time of last update session */
 	time_t last_update_end_time; /* Local end time of last update session */
 	char last_update_status[STATUS_LEN]; /* Status of last update. Format = numeric code <space> textual description */
@@ -141,7 +142,9 @@ typedef struct repl5agmt {
 	char **attrs_to_strip; /* for fractional replication, if a "mod" is empty, strip out these attributes:
 	                        * modifiersname, modifytimestamp, internalModifiersname, internalModifyTimestamp, etc */
 	int agreement_type;
-	PRUint64 protocol_timeout;
+	Slapi_Counter *protocol_timeout;
+	char *maxcsn; /* agmt max csn */
+	Slapi_RWLock *attr_lock; /* RW lock for all the stripped attrs */
 } repl5agmt;
 
 /* Forward declarations */
@@ -264,6 +267,14 @@ agmt_new_from_entry(Slapi_Entry *e)
 			slapi_entry_get_dn_const(e));
 		goto loser;
 	}
+	if ((ra->attr_lock = slapi_new_rwlock()) == NULL)
+	{
+		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "Unable to create new attr lock "
+			"for replication agreement \"%s\" - agreement ignored.\n",
+			slapi_entry_get_dn_const(e));
+		goto loser;
+	}
+	ra->protocol_timeout = slapi_counter_new();
 
 	/* Find all the stuff we need for the agreement */
 
@@ -337,19 +348,14 @@ agmt_new_from_entry(Slapi_Entry *e)
 	tmpstr = slapi_entry_attr_get_charptr(e, type_nsds5ReplicaRoot);
 	if (NULL != tmpstr)
 	{
+		PRUint64 ptimeout = 0;
+
 		ra->replarea = slapi_sdn_new_dn_passin(tmpstr);
 
 		/* If this agmt has its own timeout, grab it, otherwise use the replica's protocol timeout */
-		ra->protocol_timeout = slapi_entry_attr_get_int(e, type_replicaProtocolTimeout);
-		if(ra->protocol_timeout == 0){
-			/* grab the replica protocol timeout */
-			Object *replobj = replica_get_replica_from_dn(ra->replarea);
-			if(replobj){
-				Replica *replica =(Replica*)object_get_data (replobj);
-				ra->protocol_timeout = replica_get_protocol_timeout(replica);
-			} else {
-				ra->protocol_timeout = DEFAULT_PROTOCOL_TIMEOUT;
-			}
+		ptimeout = slapi_entry_attr_get_int(e, type_replicaProtocolTimeout);
+		if(ptimeout){
+			slapi_counter_set_value(ra->protocol_timeout, ptimeout);
 		}
 	}
 
@@ -435,14 +441,17 @@ agmt_new_from_entry(Slapi_Entry *e)
 	/* Initialize status information */
 	ra->last_update_start_time = 0UL;
 	ra->last_update_end_time = 0UL;
-	ra->num_changecounters = 0;
 	ra->last_update_status[0] = '\0';
 	ra->update_in_progress = PR_FALSE;
 	ra->stop_in_progress = PR_FALSE;
 	ra->last_init_end_time = 0UL;
 	ra->last_init_start_time = 0UL;
 	ra->last_init_status[0] = '\0';
-	
+	ra->changecounters = (struct changecounter**) slapi_ch_calloc(MAX_NUM_OF_MASTERS + 1,
+	                     sizeof(struct changecounter *));
+	ra->num_changecounters = 0;
+	ra->max_changecounters = MAX_NUM_OF_MASTERS;
+
 	/* Fractional attributes */
 	slapi_entry_attr_find(e, type_nsds5ReplicatedAttributeList, &sattr);
 
@@ -599,6 +608,7 @@ agmt_delete(void **rap)
 	{
 	    slapi_ch_free((void **)&ra->changecounters[ra->num_changecounters]);
 	}
+	slapi_ch_free((void **)&ra->changecounters);
 
 	if (ra->agreement_type == REPLICA_TYPE_WINDOWS)
 	{
@@ -608,9 +618,18 @@ agmt_delete(void **rap)
 	if(ra->attrs_to_strip){
 		slapi_ch_array_free(ra->attrs_to_strip);
 	}
-
+	if(ra->maxcsn){
+		slapi_ch_free_string(&ra->maxcsn);
+	}
 	schedule_destroy(ra->schedule);
-	slapi_ch_free((void **)&ra->long_name);
+	slapi_ch_free_string(&ra->long_name);
+
+	slapi_counter_destroy(&ra->protocol_timeout);
+
+	/* free the locks */
+	PR_DestroyLock(ra->lock);
+	slapi_destroy_rwlock(ra->attr_lock);
+
 	slapi_ch_free((void **)rap);
 }
 
@@ -2305,7 +2324,12 @@ agmt_inc_last_update_changecount (Repl_Agmt *ra, ReplicaId rid, int skipped)
 		}
 		else
 		{
-			ra->num_changecounters ++;
+			ra->num_changecounters++;
+			if(ra->num_changecounters > ra->max_changecounters){
+				ra->changecounters = (struct changecounter**) slapi_ch_realloc((char *)ra->changecounters,
+				                     (ra->num_changecounters + 1) * sizeof(struct changecounter*));
+				ra->max_changecounters = ra->num_changecounters;
+			}
 			ra->changecounters[i] = (struct changecounter*) slapi_ch_calloc(1, sizeof(struct changecounter));
 			ra->changecounters[i]->rid = rid;
 			if ( skipped )
@@ -2650,12 +2674,26 @@ agmt_update_done(Repl_Agmt *agmt, int is_total)
        during the update into the proper structures so they are in place for the
        next run
     */
-    windows_update_done(agmt, is_total);
+    if(get_agmt_agreement_type(agmt) == REPLICA_TYPE_WINDOWS){
+        windows_update_done(agmt, is_total);
+    }
 }
 
-int
+PRUint64
 agmt_get_protocol_timeout(Repl_Agmt *agmt)
 {
-    return (int)agmt->protocol_timeout;
+	if(agmt){
+		return slapi_counter_get_value(agmt->protocol_timeout);
+	} else {
+		return 0;
+	}
+}
+
+void
+agmt_set_protocol_timeout(Repl_Agmt *agmt, PRUint64 timeout)
+{
+	if(agmt){
+		slapi_counter_set_value(agmt->protocol_timeout, timeout);
+	}
 }
 
