@@ -998,9 +998,9 @@ ruv_covers_csn_cleanallruv(const RUV *ruv, const CSN *csn)
  * or max{maxcsns of all ruv elements} if get_the_max != 0.
  */
 static int
-ruv_get_min_or_max_csn(const RUV *ruv, CSN **csn, int get_the_max, ReplicaId rid)
+ruv_get_min_or_max_csn(const RUV *ruv, CSN **csn, int get_the_max, ReplicaId rid, int ignore_cleaned_rid)
 {
-	int return_value;
+	int return_value = RUV_SUCCESS;
 
 	if (ruv == NULL || csn == NULL) 
 	{
@@ -1012,6 +1012,7 @@ ruv_get_min_or_max_csn(const RUV *ruv, CSN **csn, int get_the_max, ReplicaId rid
 		CSN *found = NULL;
 		RUVElement *replica;
 		int cookie;
+
 		slapi_rwlock_rdlock (ruv->lock);
 		for (replica = dl_get_first (ruv->elements, &cookie); replica;
 			 replica = dl_get_next (ruv->elements, &cookie))
@@ -1028,6 +1029,10 @@ ruv_get_min_or_max_csn(const RUV *ruv, CSN **csn, int get_the_max, ReplicaId rid
 			{
 				continue;
 			}
+			if(ignore_cleaned_rid && is_cleaned_rid(replica->rid)){
+				continue;
+			}
+
 			if(rid){ /* we are only interested in this rid's maxcsn */
 				if(replica->rid == rid){
 					found = replica->csn;
@@ -1041,36 +1046,55 @@ ruv_get_min_or_max_csn(const RUV *ruv, CSN **csn, int get_the_max, ReplicaId rid
 					found = replica->csn;
 				}
 			}
-		} 
+		}
+
 		if (found == NULL)
 		{
-			*csn = NULL;	
+			*csn = NULL;
 		}
 		else
 		{
 			*csn = csn_dup (found);
 		}
 		slapi_rwlock_unlock (ruv->lock);
-		return_value = RUV_SUCCESS;	
 	}
 	return return_value;
 }
 
 int
-ruv_get_rid_max_csn(const RUV *ruv, CSN **csn, ReplicaId rid){
-	return ruv_get_min_or_max_csn(ruv, csn, 1 /* get the max */, rid);
+ruv_get_rid_max_csn(const RUV *ruv, CSN **csn, ReplicaId rid)
+{
+	return ruv_get_rid_max_csn_ext(ruv, csn, rid, 0);
+}
+
+int
+ruv_get_rid_max_csn_ext(const RUV *ruv, CSN **csn, ReplicaId rid, int ignore_cleaned_rid)
+{
+	return ruv_get_min_or_max_csn(ruv, csn, 1 /* get the max */, rid, ignore_cleaned_rid);
 }
 
 int
 ruv_get_max_csn(const RUV *ruv, CSN **csn)
 {
-	return ruv_get_min_or_max_csn(ruv, csn, 1 /* get the max */, 0 /* rid */);
+	return ruv_get_max_csn_ext(ruv, csn, 0);
+}
+
+int
+ruv_get_max_csn_ext(const RUV *ruv, CSN **csn, int ignore_cleaned_rid)
+{
+	return ruv_get_min_or_max_csn(ruv, csn, 1 /* get the max */, 0 /* rid */, ignore_cleaned_rid);
 }
 
 int
 ruv_get_min_csn(const RUV *ruv, CSN **csn)
 {
-	return ruv_get_min_or_max_csn(ruv, csn, 0 /* get the min */, 0 /* rid */);
+	return ruv_get_min_csn_ext(ruv, csn, 0);
+}
+
+int
+ruv_get_min_csn_ext(const RUV *ruv, CSN **csn, int ignore_cleaned_rid)
+{
+	return ruv_get_min_or_max_csn(ruv, csn, 0 /* get the min */, 0 /* rid */, ignore_cleaned_rid);
 }
 
 int 
@@ -1192,6 +1216,82 @@ ruv_get_cleaned_rids(RUV *ruv, ReplicaId *rids)
     }
 }
 
+/* 
+ * This routine dump the ruv (last modified time) into a value array
+ * the call must free the returned value array
+ */
+Slapi_Value **
+ruv_last_modified_to_valuearray(RUV *ruv)
+{
+        RUVElement *ruv_e;
+        int cookie;
+        Slapi_Value *value;
+        Slapi_Value **values = NULL;
+        char *buffer;
+
+
+        /* Acquire the ruv lock */
+        slapi_rwlock_rdlock(ruv->lock);
+
+        /* Now loop for each RUVElement and store its string value into the valueset*/
+        for (ruv_e = dl_get_first(ruv->elements, &cookie);
+                NULL != ruv_e;
+                ruv_e = dl_get_next(ruv->elements, &cookie)) {
+                			
+                buffer = slapi_ch_smprintf("%s%d%s%s} %08lx", prefix_ruvcsn, ruv_e->rid,
+			    ruv_e->replica_purl == NULL ? "" : " ",
+			    ruv_e->replica_purl == NULL ? "" : ruv_e->replica_purl,
+			    ruv_e->last_modified);
+                value = slapi_value_new_string_passin(buffer);
+                valuearray_add_value(&values, value);
+                slapi_value_free(&value);
+        }
+        
+        slapi_rwlock_unlock(ruv->lock);
+
+        return (values);
+}
+
+/* 
+ * This routine dump the ruv (replicageneration and ruvElements) into a value array
+ * the call must free the returned value array
+ */
+Slapi_Value **
+ruv_to_valuearray(RUV *ruv)
+{
+        RUVElement *ruv_e;
+        int cookie;
+        Slapi_Value *value;
+        Slapi_Value **values = NULL;
+        struct berval bv;
+        char *buffer; 
+
+        /* Acquire the ruv lock */
+        slapi_rwlock_rdlock(ruv->lock);
+        
+        /* dump the replicageneration */
+        buffer = slapi_ch_smprintf("%s %s", prefix_replicageneration, ruv->replGen);
+        value = slapi_value_new_string_passin(buffer);
+        valuearray_add_value(&values, value);
+        slapi_value_free(&value);
+
+        /* Now loop for each RUVElement and store its string value into the valueset*/
+        for (ruv_e = dl_get_first(ruv->elements, &cookie);
+                NULL != ruv_e;
+                ruv_e = dl_get_next(ruv->elements, &cookie)) {
+
+                ruv_element_to_string(ruv_e, &bv, NULL, 0);
+                value = slapi_value_new_berval(&bv);
+                slapi_ber_bvdone(&bv);
+                valuearray_add_value(&values, value);
+                slapi_value_free(&value);
+        }
+        
+        slapi_rwlock_unlock(ruv->lock);
+
+        return (values);
+}
+
 int
 ruv_to_smod(const RUV *ruv, Slapi_Mod *smod)
 {
@@ -2194,6 +2294,38 @@ ruv_is_newer (Object *sruvobj, Object *cruvobj)
 	return is_newer;
 }
 
+/*
+ * This routine is called after a disordely shutdown
+ * The Database RUV was found late compare to the changelog RUV
+ */
+void
+ruv_force_csn_update_from_ruv(RUV *src_ruv, RUV *tgt_ruv, char *msg, int logLevel) {
+    RUVElement *replica = NULL;
+    int cookie;
+
+    slapi_rwlock_rdlock(src_ruv->lock);
+    
+    for (replica = dl_get_first(src_ruv->elements, &cookie);
+            NULL != replica;
+            replica = dl_get_next(src_ruv->elements, &cookie)) {
+        /* 
+         * In case the DB RUV (tgt_ruv) is behind the CL RUV (src_ruv)
+         * updates the DB RUV.
+         */
+        if (!ruv_covers_csn(tgt_ruv, replica->csn)) {
+            char csnStr[CSN_STRSIZE];
+
+            ruv_force_csn_update(tgt_ruv, replica->csn);
+            csn_as_string(replica->csn, PR_FALSE, csnStr);
+            slapi_log_error(logLevel, repl_plugin_name, "%s %s\n",
+                    msg, csnStr);
+        }
+    }
+
+    slapi_rwlock_unlock(src_ruv->lock);
+    
+}
+
 void
 ruv_force_csn_update (RUV *ruv, CSN *csn)
 {
