@@ -376,7 +376,7 @@ windows_perform_operation(Repl_Connection *conn, int optype, const char *dn,
 				char *s = NULL;
 		
 				rc = slapi_ldap_get_lderrno(conn->ld, NULL, &s);
-				slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
+				slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name,
 						"%s: Received error %d: %s for %s operation\n",
 						agmt_get_long_name(conn->agmt),
 						rc, s ? s : "NULL", op_string);
@@ -425,7 +425,8 @@ windows_perform_operation(Repl_Connection *conn, int optype, const char *dn,
 				} 
 				else if (err == LDAP_ALREADY_EXISTS && optype == CONN_ADD)
 				{
-					conn->last_ldap_error = LDAP_SUCCESS;
+					/* Caller (windows_process_total_add) needs the ALREADY EXISTS info */
+					conn->last_ldap_error = err;
 					return_value = CONN_OPERATION_SUCCESS;
 				}
 				else if (err == LDAP_NO_SUCH_OBJECT && optype == CONN_DELETE)
@@ -461,7 +462,7 @@ windows_perform_operation(Repl_Connection *conn, int optype, const char *dn,
 				 * enough information to allow them to fix the problem
 				 * and retry - bug 170350 */
 				if (conn->last_ldap_error == LDAP_CONSTRAINT_VIOLATION) {
-					slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
+					slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name,
 						"%s: Received error [%s] when attempting to %s"
 						" entry [%s]: Please correct the attribute specified "
 						"in the error message.  Refer to the Windows Active "
@@ -469,7 +470,7 @@ windows_perform_operation(Repl_Connection *conn, int optype, const char *dn,
 						agmt_get_long_name(conn->agmt),
 						errmsg, op_string, dn);
 				} else {
-					slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+					slapi_log_error(SLAPI_LOG_REPL, windows_repl_plugin_name,
 						"%s: Received result code %d (%s) for %s operation %s%s\n",
 						agmt_get_long_name(conn->agmt),
 						conn->last_ldap_error, errmsg,
@@ -493,7 +494,7 @@ windows_perform_operation(Repl_Connection *conn, int optype, const char *dn,
 		}
 		else
 		{
-			slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
+			slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name,
 					"%s: Failed to send %s operation: LDAP error %d (%s)\n",
 					agmt_get_long_name(conn->agmt),
 					op_string ? op_string : "NULL", rc, ldap_err2string(rc));
@@ -523,14 +524,27 @@ windows_perform_operation(Repl_Connection *conn, int optype, const char *dn,
 }
 
 /* Copied from the chaining backend*/
+/* 
+ * exattrs: exceeded attribute list
+ * If attr value pair exceeds MaxValRange, AD returns, e.g.,
+ *   <attr>;range=0-<maxValRange-1>: <value>
+ * We need to repeat the search with "<attr>;range=1500-*"
+ * until it returns 
+ *   <attr>;range=<num>-*
+ */
 static Slapi_Entry * 
-windows_LDAPMessage2Entry(Repl_Connection *conn, LDAPMessage * msg, int attrsonly) {
+windows_LDAPMessage2Entry(Slapi_Entry *e, Repl_Connection *conn,
+                          LDAPMessage * msg, int attrsonly, char ***exattrs)
+{
 
 	Slapi_Entry *rawentry = NULL;
-	Slapi_Entry *e = NULL;
 	char *a = NULL;
 	BerElement * ber = NULL;
 	LDAP *ld = conn->ld;
+	int exattrlen = 0;
+	int exattridx = 0;
+	char **deletedattrs = NULL;
+	char **dap;
 
 	windows_private_set_raw_entry(conn->agmt, NULL); /* clear it first */
 
@@ -543,9 +557,10 @@ windows_LDAPMessage2Entry(Repl_Connection *conn, LDAPMessage * msg, int attrsonl
 	 * attribute type and values ARE allocated
 	 */
 
-	e = slapi_entry_alloc();
-	if ( e == NULL ) return NULL;
-	slapi_entry_set_dn( e, ldap_get_dn( ld, msg ) );
+	if (NULL == e) {
+		e = slapi_entry_alloc();
+		slapi_entry_set_dn( e, ldap_get_dn( ld, msg ) );
+	}
 	rawentry = slapi_entry_alloc();
 	if ( rawentry == NULL ) {
 		slapi_entry_free(e);
@@ -575,13 +590,59 @@ windows_LDAPMessage2Entry(Repl_Connection *conn, LDAPMessage * msg, int attrsonl
 				slapi_entry_add_value(e, a, (Slapi_Value *)NULL);
 			} else 
 			{
+#define SUBTYPERANGE "range="
 				char *type_to_use = NULL;
+				char *dupa = slapi_ch_strdup(a);
+				char *newa = NULL; /* dup of 'a' with next range */
+				char *p, *wp, *pp; /* work pointers */
+				char *iter;
+				int high = 0;
+				int sizea = strlen(a) + 2;
+				/* handling subtype(s) */
+				ldap_utf8strtok_r(dupa, ";", &iter); /* primry type */
+				p = ldap_utf8strtok_r(NULL, ";", &iter); /* subtype, if any */
+				while (p) {
+					if (0 == strncasecmp(p, SUBTYPERANGE, sizeof(SUBTYPERANGE) - 1)) {
+						/* get rid of range */
+						if (!newa) { /* first time for range= */
+							/* Cannot use strdup,
+							 * since 'a' could be "<attr>;range=0-9";
+							 * then newa is <attr>;10-*; newa is 1 char longer than a. */
+							newa = (char *)slapi_ch_malloc(sizea);
+							PR_snprintf(newa, sizea, "%s", a);
+							*(newa + (p - dupa) - 1) = '\0';
+						}
+						/* get the last count (high + 1) */
+						/* range=low-high */
+						pp = strchr(p, '-');
+						if (*++pp == '*') {
+							high = 0; /* high is *; done! */
+						} else {
+							high = strtol(pp, &p, 10);
+							if (high > 0) {
+								/* next low == high + 1 */
+								high++;
+							}
+						}
+					} else { /* subtype except "range=low-high" */
+						if (newa) {
+							int sizenewa = strlen(newa);
+							/* range= appeared before, copy this subtype */
+							wp = newa + sizenewa;
+							/* append ;<subtype> */
+							PR_snprintf(wp, sizea - sizenewa, ";%s", p);
+						}
+					}
+					p = ldap_utf8strtok_r(NULL, ";", &iter);
+				}
+				slapi_ch_free_string(&dupa);
+
 				/* Work around the fact that we alias street and streetaddress, while Microsoft do not */
-				if (0 == strcasecmp(a,"streetaddress")) 
-				{
+				if (0 == strcasecmp(a, "streetaddress")) {
 					type_to_use = FAKE_STREET_ATTR_NAME;
-				} else
-				{
+				} else if (newa) {
+					type_to_use = newa;
+				} else {
 					type_to_use = a;
 				}
 
@@ -591,13 +652,38 @@ windows_LDAPMessage2Entry(Repl_Connection *conn, LDAPMessage * msg, int attrsonl
 				if (aVal == NULL) {
 					/* Windows will send us an attribute with no values if it was deleted
 					 * on the AD side.  Add this attribute to the deleted attributes list */
-					Slapi_Attr *attr = slapi_attr_new();
-					slapi_attr_init(attr, type_to_use);
-					entry_add_deleted_attribute_wsi(e, attr);
+					/* Set it to the deleted attribute list only if the attribute does
+					 * not exist in the entry.  For the multi-valued attribute (e.g.,
+					 * member), if there are multiple member attributes in an entry, 
+					 * and one of them is deleted, this no value member is sent.  But
+					 * if there are more member attributes in the entry, we should not
+					 * set member to the deleted attribute. */
+					if (!charray_inlist(deletedattrs, type_to_use)) {
+						charray_add(&deletedattrs, slapi_ch_strdup(type_to_use));
+					}
 				} else { 
-					slapi_entry_add_values( e, type_to_use, aVal);
+					slapi_entry_add_values(e, type_to_use, aVal);
 				} 
-                
+
+				/* if the addr for exattrs is given and next range retrieval is needed */
+				if (exattrs && (high > 0)) {
+					if (exattrlen == exattridx) {
+						if (!*exattrs) {
+							exattrlen = 4;
+							exattridx = 0;
+							*exattrs = (char **)slapi_ch_calloc(exattrlen, sizeof(char *));
+						} else {
+							*exattrs = (char **)slapi_ch_realloc((char *)*exattrs, exattrlen * 2 * sizeof(char *));
+							memset(*exattrs + exattrlen, '\0', exattrlen * sizeof(char *));
+							exattrlen *= 2;
+						}
+						PR_snprintf(newa + strlen(newa), strlen(a) + 2 - strlen(newa),
+						                          ";%s%d-*", SUBTYPERANGE, high);
+						(*exattrs)[exattridx++] = newa;
+					}
+				} else if (newa) {
+					slapi_ch_free_string(&newa);
+				}
 			}
 		}
 		ldap_memfree(a);
@@ -607,6 +693,18 @@ windows_LDAPMessage2Entry(Repl_Connection *conn, LDAPMessage * msg, int attrsonl
 	{
         ber_free( ber, 0 );
 	}
+	/* Windows will send us an attribute with no values if it was deleted
+	 * on the AD side.  Add this attribute to the deleted attributes list */
+	/* Set to e_deleted_attrs only if there is no attribute of the type. */
+	for (dap = deletedattrs; dap && *dap; dap++) {
+		Slapi_Attr *attr = NULL;
+		if (slapi_entry_attr_find(e, *dap, &attr)) { /* not found */
+			attr = slapi_attr_new();
+			slapi_attr_init(attr, *dap);
+			entry_add_deleted_attribute_wsi(e, attr);
+		}
+	}
+	charray_free(deletedattrs);
 
 	windows_private_set_raw_entry(conn->agmt, rawentry); /* windows private now owns rawentry */
 
@@ -643,6 +741,7 @@ windows_search_entry_ext(Repl_Connection *conn, char* searchbase, char *filter,
 		char *searchbase_copy = slapi_ch_strdup(searchbase);
 		char *filter_copy = slapi_ch_strdup(filter);
 		char **attrs = NULL;
+		char **exattrs = NULL;
 		LDAPControl **serverctrls_copy = NULL;
 
 		slapi_add_controls(&serverctrls_copy, serverctrls, 1 /* make a copy we can free */);
@@ -651,30 +750,22 @@ windows_search_entry_ext(Repl_Connection *conn, char* searchbase, char *filter,
 
 		winsync_plugin_call_pre_ad_search_cb(conn->agmt, NULL, &searchbase_copy, &scope, &filter_copy,
 											 &attrs, &serverctrls_copy);
-		
+next:
 		ldap_rc = ldap_search_ext_s(conn->ld, searchbase_copy, scope,
 			filter_copy, attrs, 0 /* attrsonly */,
 			serverctrls_copy , NULL /* client controls */,
 			&conn->timeout, 0 /* sizelimit */, &res);
 
 		if ((LDAP_SUCCESS != ldap_rc) && !IS_DISCONNECT_ERROR(ldap_rc)) {
-			slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+			slapi_log_error(SLAPI_LOG_REPL, windows_repl_plugin_name,
 							"Could not retrieve entry from Windows using search "
 							"base [%s] scope [%d] filter [%s]: error %d:%s\n",
 							searchbase_copy, scope, filter_copy, ldap_rc,
 							ldap_err2string(ldap_rc));
 		}
 
-		slapi_ch_free_string(&searchbase_copy);
-		slapi_ch_free_string(&filter_copy);
 		slapi_ch_array_free(attrs);
 		attrs = NULL;
-		ldap_controls_free(serverctrls_copy);
-		serverctrls_copy = NULL;
-
-		/* clear it here in case the search fails and
-		   we are left with a bogus old entry */
-		windows_private_set_raw_entry(conn->agmt, NULL);
 		if (LDAP_SUCCESS == ldap_rc)
 		{
 			LDAPMessage *message = ldap_first_entry(conn->ld, res);
@@ -690,9 +781,18 @@ windows_search_entry_ext(Repl_Connection *conn, char* searchbase, char *filter,
                                    nummessages, numentries, numreferences );
 			}
 
-			if (NULL != entry)
-			{
-				*entry = windows_LDAPMessage2Entry(conn,message,0);
+			if (entry) {
+				exattrs = NULL;
+				*entry = windows_LDAPMessage2Entry(*entry, conn, message, 0, &exattrs);
+				if (exattrs) {
+					/* some attribute returned "<attr>;range=low-high" */
+					attrs = exattrs;
+					if (res) {
+						ldap_msgfree(res);
+						res = NULL;
+					}
+					goto next;
+				}
 			}
 			/* See if there are any more entries : if so then that's an error
 			 * but we still need to get them to avoid gumming up the connection
@@ -710,6 +810,14 @@ windows_search_entry_ext(Repl_Connection *conn, char* searchbase, char *filter,
 		{
 			return_value = CONN_OPERATION_FAILED;
 		}
+		slapi_ch_free_string(&searchbase_copy);
+		slapi_ch_free_string(&filter_copy);
+		ldap_controls_free(serverctrls_copy);
+		serverctrls_copy = NULL;
+
+		/* clear it here in case the search fails and
+		   we are left with a bogus old entry */
+		windows_private_set_raw_entry(conn->agmt, NULL);
 		conn->last_ldap_error = ldap_rc;
 		if (NULL != res)
 		{
@@ -738,13 +846,15 @@ send_dirsync_search(Repl_Connection *conn)
 		const char *op_string = NULL;
 		int rc;
 		int scope = LDAP_SCOPE_SUBTREE;
-		char *filter = slapi_ch_strdup("(objectclass=*)");
+		const char *userfilter = NULL;
+		char *filter = NULL;
 		char **attrs = NULL;
 		LDAPControl **server_controls = NULL;
 		int msgid;
-		/* need to strip the dn down to dc= */
 		const char *old_dn = slapi_sdn_get_ndn( windows_private_get_windows_subtree(conn->agmt) );
+		/* LDAP_SERVER_DIRSYNC_OID requires the search base Naming Context */
 		char *dn = slapi_ch_strdup(strstr(old_dn, "dc="));
+		char **exattrs = NULL;
 
 		if (conn->supports_dirsync == 0)
 		{
@@ -752,8 +862,8 @@ send_dirsync_search(Repl_Connection *conn)
 		} else 
 		{
 			slapi_add_control_ext(&server_controls,
-								  windows_private_dirsync_control(conn->agmt),
-								  0 /* no copy - passin */);
+			                      windows_private_dirsync_control(conn->agmt),
+			                      0 /* no copy - passin */);
 		}
 
 		conn->last_operation = CONN_SEARCH;
@@ -761,9 +871,18 @@ send_dirsync_search(Repl_Connection *conn)
 		op_string = "search";
 
 		LDAPDebug( LDAP_DEBUG_REPL, "Calling dirsync search request plugin\n", 0, 0, 0 );
+		userfilter = windows_private_get_windows_userfilter(conn->agmt);
+		if (userfilter) {
+			filter = slapi_ch_strdup(userfilter);
+		} else {
+			filter = slapi_ch_strdup("(objectclass=*)");
+		}
 
 		winsync_plugin_call_dirsync_search_params_cb(conn->agmt, old_dn, &dn, &scope, &filter,
-													 &attrs, &server_controls);
+		                                             &attrs, &server_controls);
+		exattrs = windows_private_get_range_attrs(conn->agmt);
+		charray_merge(&attrs, exattrs, 0 /* pass in */);
+		slapi_ch_free((void **)&exattrs); /* strings are passed in */
 
 		LDAPDebug( LDAP_DEBUG_REPL, "Sending dirsync search request\n", 0, 0, 0 );
 
@@ -776,7 +895,7 @@ send_dirsync_search(Repl_Connection *conn)
 		}
 		else
 		{
-            slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
+            slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name,
 					"%s: Failed to get %s operation: LDAP error %d (%s)\n",
 					agmt_get_long_name(conn->agmt),
 					op_string, rc, ldap_err2string(rc));
@@ -924,10 +1043,20 @@ Slapi_Entry * windows_conn_get_search_result(Repl_Connection *conn)
 			{
 				if (( dn = ldap_get_dn( conn->ld, res )) != NULL ) 
 				{
+					char **exattrs = NULL;
 					slapi_log_error(SLAPI_LOG_REPL, windows_repl_plugin_name,"received entry from dirsync: %s\n", dn);
-					lm = ldap_first_entry( conn->ld, res );			
-					e = windows_LDAPMessage2Entry(conn,lm,0);
+					lm = ldap_first_entry( conn->ld, res );
+					e = windows_private_get_curr_entry(conn->agmt); /* if range search, e != NULL */
+					e = windows_LDAPMessage2Entry(e, conn, lm, 0, &exattrs);
 					ldap_memfree(dn);
+					if (exattrs) {
+						/* some attribute returned "<attr>;range=low-high" */
+						windows_private_set_curr_entry(conn->agmt, e);
+						windows_private_set_range_attrs(conn->agmt, exattrs);
+					} else {
+						windows_private_set_curr_entry(conn->agmt, NULL);
+						windows_private_set_range_attrs(conn->agmt, NULL);
+					}
 				}
 			}
 			break;
@@ -1055,7 +1184,7 @@ windows_conn_cancel_linger(Repl_Connection *conn)
 	PR_Lock(conn->lock);
 	if (conn->linger_active)
 	{
-		slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+		slapi_log_error(SLAPI_LOG_REPL, windows_repl_plugin_name,
 			"%s: Cancelling linger on the connection\n",
 			agmt_get_long_name(conn->agmt));
 		conn->linger_active = PR_FALSE;
@@ -1068,7 +1197,7 @@ windows_conn_cancel_linger(Repl_Connection *conn)
 	}
 	else
 	{
-		slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+		slapi_log_error(SLAPI_LOG_REPL, windows_repl_plugin_name,
 			"%s: No linger to cancel on the connection\n",
 			agmt_get_long_name(conn->agmt));
 	}
@@ -1092,7 +1221,7 @@ linger_timeout(time_t event_time, void *arg)
 	LDAPDebug( LDAP_DEBUG_TRACE, "=> linger_timeout\n", 0, 0, 0 );
 
 	PR_ASSERT(NULL != conn);
-	slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+	slapi_log_error(SLAPI_LOG_REPL, windows_repl_plugin_name,
 		"%s: Linger timeout has expired on the connection\n",
 		agmt_get_long_name(conn->agmt));
 	PR_Lock(conn->lock);
@@ -1124,12 +1253,12 @@ windows_conn_start_linger(Repl_Connection *conn)
 	LDAPDebug( LDAP_DEBUG_TRACE, "=> windows_conn_start_linger\n", 0, 0, 0 );
 
 	PR_ASSERT(NULL != conn);
-	slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+	slapi_log_error(SLAPI_LOG_REPL, windows_repl_plugin_name,
 		"%s: Beginning linger on the connection\n",
 		agmt_get_long_name(conn->agmt));
 	if (!windows_conn_connected(conn))
 	{
-		slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+		slapi_log_error(SLAPI_LOG_REPL, windows_repl_plugin_name,
 			"%s: No linger on the closed conn\n",
 			agmt_get_long_name(conn->agmt));
 		return;
@@ -1138,7 +1267,7 @@ windows_conn_start_linger(Repl_Connection *conn)
 	PR_Lock(conn->lock);
 	if (conn->linger_active)
 	{
-		slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+		slapi_log_error(SLAPI_LOG_REPL, windows_repl_plugin_name,
 			"%s: Linger already active on the connection\n",
 			agmt_get_long_name(conn->agmt));
 	}
@@ -1208,7 +1337,7 @@ windows_conn_connect(Repl_Connection *conn)
 		/* Pb occured in decryption: stop now, binding will fail */
 		if ( pw_ret == -1 )
 		{
-			slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
+			slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name,
 				"%s: Decoding of the credentials failed.\n",
 				agmt_get_long_name(conn->agmt));
 		
@@ -1235,7 +1364,7 @@ windows_conn_connect(Repl_Connection *conn)
  
 	if (secure > 0) {
 		if (!NSS_IsInitialized()) {
-			slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
+			slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name,
 							"%s: SSL Not Initialized, Replication over SSL FAILED\n",
 							agmt_get_long_name(conn->agmt));
 			return_value = CONN_SSL_NOT_ENABLED;
@@ -1251,7 +1380,7 @@ windows_conn_connect(Repl_Connection *conn)
 #endif
 		/* Now we initialize the LDAP Structure and set options */
 		
-		slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+		slapi_log_error(SLAPI_LOG_REPL, windows_repl_plugin_name,
 			"%s: Trying %s%s slapi_ldap_init_ext\n",
 			agmt_get_long_name(conn->agmt),
 			secure ? "secure" : "non-secure",
@@ -1264,7 +1393,7 @@ windows_conn_connect(Repl_Connection *conn)
 			conn->state = STATE_DISCONNECTED;
 			conn->last_operation = CONN_INIT;
 			conn->last_ldap_error = LDAP_LOCAL_ERROR;
-			slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
+			slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name,
 				"%s: Failed to establish %s%sconnection to the consumer\n",
 				agmt_get_long_name(conn->agmt),
 				secure ? "secure " : "",
@@ -1275,7 +1404,7 @@ windows_conn_connect(Repl_Connection *conn)
 		/* slapi_ch_strdup is OK with NULL strings */
 		binddn = slapi_ch_strdup(conn->binddn);
 
-		slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+		slapi_log_error(SLAPI_LOG_REPL, windows_repl_plugin_name,
 			"%s: binddn = %s,  passwd = %s\n",
 			agmt_get_long_name(conn->agmt),
 			binddn?binddn:"NULL", creds->bv_val?creds->bv_val:"NULL");
@@ -1377,7 +1506,7 @@ close_connection_internal(Repl_Connection *conn)
 	conn->state = STATE_DISCONNECTED;
 	conn->status = STATUS_DISCONNECTED;
 	conn->supports_ds50_repl = -1;
-	slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+	slapi_log_error(SLAPI_LOG_REPL, windows_repl_plugin_name,
 		"%s: Disconnected from the consumer\n", agmt_get_long_name(conn->agmt));
 	LDAPDebug( LDAP_DEBUG_TRACE, "<= close_connection_internal\n", 0, 0, 0 );
 }
@@ -1758,7 +1887,7 @@ bind_and_check_pwp(Repl_Connection *conn, char * binddn, char *password)
 		if (conn->last_ldap_error != rc)
 		{
 			conn->last_ldap_error = rc;
-			slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
+			slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name,
 							"%s: Replication bind with %s auth resumed\n",
 							agmt_get_long_name(conn->agmt),
 							mech ? mech : "SIMPLE");
@@ -1772,7 +1901,7 @@ bind_and_check_pwp(Repl_Connection *conn, char * binddn, char *password)
 				if ( !(strcmp( ctrls[ i ]->ldctl_oid, LDAP_CONTROL_PWEXPIRED)) )
 				{
 					/* Bind is successfull but password has expired */
-					slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, 
+					slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name, 
 						"%s: Successfully bound %s to consumer, "
 						"but password has expired on consumer.\n",
 						agmt_get_long_name(conn->agmt), binddn);
@@ -1784,7 +1913,7 @@ bind_and_check_pwp(Repl_Connection *conn, char * binddn, char *password)
 						 (ctrls[ i ]->ldctl_value.bv_len > 0) )
 					{
 						int password_expiring = atoi( ctrls[ i ]->ldctl_value.bv_val );
-						slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, 
+						slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name, 
 							"%s: Successfully bound %s to consumer, "
 							"but password is expiring on consumer in %d seconds.\n",
 							agmt_get_long_name(conn->agmt), binddn, password_expiring);
@@ -1809,7 +1938,7 @@ bind_and_check_pwp(Repl_Connection *conn, char * binddn, char *password)
 			conn->last_ldap_error = rc;
 			/* errmsg is a pointer directly into the ld structure - do not free */
 			rc = slapi_ldap_get_lderrno( ld, NULL, &errmsg );
-			slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
+			slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name,
 							"%s: Replication bind with %s auth failed: LDAP error %d (%s) (%s)\n",
 							agmt_get_long_name(conn->agmt),
 							mech ? mech : "SIMPLE", rc,
@@ -1818,7 +1947,7 @@ bind_and_check_pwp(Repl_Connection *conn, char * binddn, char *password)
 			char *errmsg = NULL;
 			/* errmsg is a pointer directly into the ld structure - do not free */
 			rc = slapi_ldap_get_lderrno( ld, NULL, &errmsg );
-			slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+			slapi_log_error(SLAPI_LOG_REPL, windows_repl_plugin_name,
 							"%s: Replication bind with %s auth failed: LDAP error %d (%s) (%s)\n",
 							agmt_get_long_name(conn->agmt),
 							mech ? mech : "SIMPLE", rc,
@@ -1854,13 +1983,13 @@ windows_check_user_password(Repl_Connection *conn, Slapi_DN *sdn, char *password
 	rc = ldap_result(conn->ld, msgid, LDAP_MSG_ALL, NULL, &res);
 	if (0 > rc) { /* error */
 		rc = slapi_ldap_get_lderrno(conn->ld, NULL, NULL);
-		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
+		slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name,
 						"Error reading bind response for id "
 						"[%s]: error %d (%s)\n",
 						binddn ? binddn : "(anon)",
 						rc, ldap_err2string(rc));
 	} else if (rc == 0) { /* timeout */
-		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
+		slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name,
 						"Error: timeout reading "
 						"bind response for [%s]\n",
 						binddn ? binddn : "(anon)");
@@ -1868,7 +1997,7 @@ windows_check_user_password(Repl_Connection *conn, Slapi_DN *sdn, char *password
 	} else {
 		parse_rc = ldap_parse_result( conn->ld, res, &rc, NULL, NULL, NULL, NULL, 1 /* Free res */);
 		if (parse_rc != LDAP_SUCCESS) {
-			slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
+			slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name,
 						"Error: unable to parse "
 						"bind result for [%s]: "
 						"error %d\n",
@@ -1902,7 +2031,7 @@ do_simple_bind (Repl_Connection *conn, LDAP *ld, char * binddn, char *password)
 		if (conn->last_ldap_error != ldaperr)
 		{
 			conn->last_ldap_error = ldaperr;
-			slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, 
+			slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name, 
 				"%s: Simple bind failed, " 
 				SLAPI_COMPONENT_NAME_LDAPSDK " error %d (%s) (%s), "
 				SLAPI_COMPONENT_NAME_NSPR " error %d (%s)\n",
@@ -1915,7 +2044,7 @@ do_simple_bind (Repl_Connection *conn, LDAP *ld, char * binddn, char *password)
 	else if (conn->last_ldap_error != LDAP_SUCCESS)
 	{
 		conn->last_ldap_error = LDAP_SUCCESS;
-		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
+		slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name,
 			"%s: Simple bind resumed\n",
 			agmt_get_long_name(conn->agmt));
 	}
@@ -1975,7 +2104,7 @@ repl5_debug_timeout_callback(time_t when, void *arg)
 	sprintf(buf, "%d", s_debug_level);
 	config_set_errorlog_level("nsslapd-errorlog-level", buf, msg, 1);
 
-	slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, 
+	slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name, 
 		"repl5_debug_timeout_callback: set debug level to %d at %ld\n",
 		s_debug_level, when);
 
