@@ -77,9 +77,11 @@ void* g_plg_identity [PLUGIN_MAX];
 
 Slapi_Backend *retrocl_be_changelog = NULL;
 PRLock *retrocl_internal_lock = NULL;
+Slapi_RWLock *retrocl_cn_lock;
 int retrocl_nattributes = 0;
 char **retrocl_attributes = NULL;
 char **retrocl_aliases = NULL;
+int retrocl_log_deleted = 0;
 
 /* ----------------------------- Retrocl Plugin */
 
@@ -389,6 +391,21 @@ static int retrocl_start (Slapi_PBlock *pb)
         slapi_ch_array_free(values);
     }
 
+    retrocl_log_deleted = 0;
+    values = slapi_entry_attr_get_charray(e, "nsslapd-log-deleted");
+    if (values != NULL) {
+	if (values[1] != NULL) {
+		slapi_log_error(SLAPI_LOG_PLUGIN, RETROCL_PLUGIN_NAME,
+			"Multiple values specified for attribute: nsslapd-log-deleted\n");
+	} else if ( 0 == strcasecmp(values[0], "on")) {
+		retrocl_log_deleted = 1;
+	} else if (strcasecmp(values[0], "off")) {
+		slapi_log_error(SLAPI_LOG_PLUGIN, RETROCL_PLUGIN_NAME,
+			"Invalid value (%s) specified for attribute: nsslapd-log-deleted\n", values[0]);
+	}
+        slapi_ch_array_free(values);
+    }
+
     retrocl_started = 1;
 
     return 0;
@@ -410,7 +427,9 @@ static int retrocl_stop (Slapi_PBlock *pb)
   int rc = 0;
 
   slapi_ch_array_free(retrocl_attributes);
+  retrocl_attributes = NULL;
   slapi_ch_array_free(retrocl_aliases);
+  retrocl_aliases = NULL;
 
   retrocl_stop_trimming();  
   retrocl_be_changelog = NULL;
@@ -465,7 +484,8 @@ retrocl_plugin_init(Slapi_PBlock *pb)
 	  if (!is_betxn) {
 	    rc= slapi_register_plugin_ext("internalpostoperation", 1 /* Enabled */, "retrocl_internalpostop_init", retrocl_internalpostop_init, "Retrocl internal postoperation plugin", NULL, identity, precedence);
 	  }
-
+	  retrocl_cn_lock = slapi_new_rwlock();
+	  if(retrocl_cn_lock == NULL) return -1;
 	  retrocl_internal_lock = PR_NewLock();
 	  if (retrocl_internal_lock == NULL) return -1;
 	}
