@@ -218,7 +218,8 @@ preop_add(Slapi_PBlock *pb)
 {
   int result;
   char *violated = NULL;
-
+  char *pwd = NULL;
+  char *origpwd = NULL;
 #ifdef DEBUG
   slapi_log_error(SLAPI_LOG_PLUGIN, plugin_name, "ADD begin\n");
 #endif
@@ -236,12 +237,14 @@ preop_add(Slapi_PBlock *pb)
     const char *dn;
     Slapi_DN *sdn = NULL;
     Slapi_Entry *e;
-    Slapi_Attr *attr;
     char **firstSubtree;
     char **subtreeDN;
     int subtreeCnt;
     int is_replicated_operation;
-
+    struct berval *vals[2];
+    struct berval val;
+    vals[0] = &val;
+    vals[1] = NULL;
     /*
      * Get the arguments
      */
@@ -288,19 +291,26 @@ preop_add(Slapi_PBlock *pb)
     for (attrName = argv; strcmp(*attrName, ",") != 0; attrName++ )
     {
       /* 
-       * if the attribute is userpassword, check unhashed#user#password 
+       * if the attribute is userpassword, check unhashed user password 
        * instead.  "userpassword" is encoded; it will always pass the 7bit 
        * check.
        */
-      char *attr_name; 
+      char *attr_name = NULL;
+      Slapi_Attr *attr = NULL; 
       if ( strcasecmp(*attrName, "userpassword") == 0 )
       {
-         attr_name = "unhashed#user#password";
+         origpwd = pwd = slapi_get_first_clear_text_pw(e);
+         if (pwd == NULL)
+	 {
+            continue;
+         }
+         val.bv_val = pwd;
+         val.bv_len = strlen(val.bv_val);
       } else {
          attr_name = *attrName;
+	 err = slapi_entry_attr_find(e, attr_name, &attr);
+         if (err) continue; /* break;*/  /* no 7-bit attribute */
       }
-      err = slapi_entry_attr_find(e, attr_name, &attr);
-      if (err) continue; /* break;*/  /* no 7-bit attribute */
 
       /*
        * For each DN in the managed list, do 7-bit checking if
@@ -323,7 +333,14 @@ preop_add(Slapi_PBlock *pb)
           /*
            * Check if the value is 7-bit clean
            */
-          result = bit_check(attr, NULL, &violated);
+	  if(pwd)
+	  {
+            result = bit_check(attr, vals, &violated);
+	    if(!result)
+	      pwd = NULL;
+	  }
+          else
+            result = bit_check(attr, NULL, &violated);
           if (result) break;
         }
       }
@@ -335,7 +352,7 @@ preop_add(Slapi_PBlock *pb)
   if (result) {
     issue_error(pb, result, "ADD", violated);
   }
-
+  slapi_ch_free_string(&origpwd);
   return (result==LDAP_SUCCESS)?0:-1;
 }
 
@@ -544,7 +561,7 @@ preop_modrdn(Slapi_PBlock *pb)
     char **argv;
     char **attrName;
     Slapi_DN *target_sdn = NULL;
-    Slapi_DN *superior;
+    Slapi_DN *superior = NULL;
     char *rdn; 
     Slapi_Attr *attr;
     char **firstSubtree;
@@ -584,7 +601,7 @@ preop_modrdn(Slapi_PBlock *pb)
      * its current level in the tree.  Use the target DN for
      * determining which managed tree this belongs to
      */
-    if (!superior) superior = target_sdn;
+    if (!slapi_sdn_get_dn(superior)) superior = target_sdn;
 
     /* Get the new RDN - this has the attribute values */
     err = slapi_pblock_get(pb, SLAPI_MODRDN_NEWRDN, &rdn);
@@ -618,7 +635,7 @@ preop_modrdn(Slapi_PBlock *pb)
     }
 
     /*
-     * arguments before "," are the 7-bit clean attribute names.  Arguemnts
+     * arguments before "," are the 7-bit clean attribute names.  Arguments
      * after "," are subtreeDN's.
      */
     for ( firstSubtree = argv; strcmp(*firstSubtree, ",") != 0;
@@ -633,7 +650,7 @@ preop_modrdn(Slapi_PBlock *pb)
     for (attrName = argv; strcmp(*attrName, ",") != 0; attrName++ )
     {
       /* 
-       * If the attribut type is userpassword, do not replace it by 
+       * If the attribute type is userpassword, do not replace it by
        * unhashed#user#password because unhashed#user#password does not exist  
        * in this case.
        */
@@ -699,9 +716,10 @@ NS7bitAttr_Init(Slapi_PBlock *pb)
   int premdn = SLAPI_PLUGIN_PRE_MODRDN_FN;
 
   BEGIN
+    int attr_count = 0;
     int argc;
     char **argv;
-
+    int valid_suffix = 0;
 
     /* Declare plugin version */
     err = slapi_pblock_set(pb, SLAPI_PLUGIN_VERSION,
@@ -731,18 +749,27 @@ NS7bitAttr_Init(Slapi_PBlock *pb)
      * Arguments before "," are the 7-bit attribute names. Arguments after
      * "," are the subtree DN's. 
      */
-    if (argc < 1) { err = -1; break; }
-    for(;strcmp(*argv, ",") != 0 && argc > 0; argc--, argv++)
-      {};
-    if (argc == 0) { err = -1; break; }
+    if (argc < 1) { err = -2; break; } /* missing arguments */
+    for(;*argv && strcmp(*argv, ",") != 0 && argc > 0; attr_count++, argc--, argv++);
+    if (argc == 0) { err = -3; break; } /* no comma separator */
+    if(attr_count == 0){ err = -4; break; } /* no attributes */
     argv++; argc--;
-
+    if(argc == 0){ err = -5; break; } /* no suffix */
     for(;argc > 0;argc--, argv++) {
+	err = slapi_dn_syntax_check(pb, *argv, 1);
+	if (err) {
+	    slapi_log_error(SLAPI_LOG_FATAL, "NS7bitAttr_Init",
+                      "Invalid suffix: %s\n", *argv);
+	    continue;
+	}
+	if (!valid_suffix)
+	    valid_suffix = 1;
         char *normdn = slapi_create_dn_string_case("%s", *argv);
         slapi_ch_free_string(argv);
         *argv = normdn;
     }
 
+    if (!valid_suffix) { err = -6; break; } /* Invalid suffix list */
     /* Provide descriptive information */
     err = slapi_pblock_set(pb, SLAPI_PLUGIN_DESCRIPTION,
             (void*)&pluginDesc);
@@ -761,9 +788,26 @@ NS7bitAttr_Init(Slapi_PBlock *pb)
   END
 
   if (err) {
-    slapi_log_error(SLAPI_LOG_PLUGIN, "NS7bitAttr_Init",
-             "Error: %d\n", err);
-    err = -1;
+      if(err == -1){
+          slapi_log_error(SLAPI_LOG_PLUGIN, "NS7bitAttr_Init","Error: %d\n", err);
+      } else if(err == -2){
+          slapi_log_error(SLAPI_LOG_FATAL, "NS7bitAttr_Init",
+                      "Invalid plugin arguments - missing arguments\n");
+      } else if(err == -3){
+          slapi_log_error(SLAPI_LOG_FATAL, "NS7bitAttr_Init",
+                      "Invalid plugin arguments - missing \",\" separator argument\n");
+      } else if(err == -4){
+          slapi_log_error(SLAPI_LOG_FATAL, "NS7bitAttr_Init",
+                      "Invalid plugin arguments - missing attributes\n");
+      } else if(err == -5){
+          slapi_log_error(SLAPI_LOG_FATAL, "NS7bitAttr_Init",
+                      "Invalid plugin arguments - missing suffix\n");
+      } else if(err == -6){
+          slapi_log_error(SLAPI_LOG_FATAL, "NS7bitAttr_Init",
+                      "Invalid plugin arguments - Invalid suffix list\n");
+      }
+
+      err = -1;
   }
   else
     slapi_log_error(SLAPI_LOG_PLUGIN, "NS7bitAttr_Init",
