@@ -44,6 +44,7 @@
 struct usn_cleanup_data {
     char *suffix;
     char *maxusn_to_delete;
+    char *bind_dn;
 };
 
 static int usn_cleanup_add(Slapi_PBlock *pb, Slapi_Entry *e,
@@ -86,6 +87,9 @@ usn_cleanup_thread(void *arg)
         goto bail;
     }
 
+    /*  Initialize and set the thread data */
+    slapi_td_set_dn(slapi_ch_strdup(cleanup_data->bind_dn));
+
     /* update task state to show it's running */
     slapi_task_begin(task, total_work);
     if (cleanup_data->maxusn_to_delete) {
@@ -141,9 +145,10 @@ usn_cleanup_thread(void *arg)
     for (ep = entries; ep && *ep; ep++) {
         int delrv = 0;
         const Slapi_DN *sdn = slapi_entry_get_sdn_const(*ep);
+        int opflags = OP_FLAG_TOMBSTONE_ENTRY;
 
         /* check for shutdown */
-        if(g_get_shutdown()){
+        if(slapi_is_shutting_down()){
             slapi_task_log_notice(task, "USN tombstone cleanup task aborted due to shutdown.");
             slapi_task_log_status(task, "USN tombstone cleanup task aborted due to shutdown.");
             slapi_log_error(SLAPI_LOG_FATAL, USN_PLUGIN_SUBSYSTEM,
@@ -152,7 +157,7 @@ usn_cleanup_thread(void *arg)
         }
 
         slapi_delete_internal_set_pb(delete_pb, slapi_sdn_get_dn(sdn),
-                                     NULL, NULL, usn_get_identity(), 0);
+                                     NULL, NULL, usn_get_identity(), opflags);
         slapi_delete_internal_pb(delete_pb);
         slapi_pblock_get(delete_pb, SLAPI_PLUGIN_INTOP_RESULT, &delrv);
         if (LDAP_SUCCESS != delrv) {
@@ -184,6 +189,7 @@ bail:
     }
     slapi_ch_free_string(&cleanup_data->maxusn_to_delete);
     slapi_ch_free_string(&cleanup_data->suffix);
+    slapi_ch_free_string(&cleanup_data->bind_dn);
     slapi_ch_free((void **)&cleanup_data);
 
     /* this will queue the destruction of the task */
@@ -242,6 +248,7 @@ usn_cleanup_add(Slapi_PBlock *pb, Slapi_Entry *e, Slapi_Entry *eAfter,
     char *suffix = NULL;
     char *backend = NULL;
     char *maxusn = NULL;
+    char *bind_dn;
     struct usn_cleanup_data *cleanup_data = NULL;
     int rv = SLAPI_DSE_CALLBACK_OK;
     Slapi_Task *task = NULL;
@@ -253,6 +260,9 @@ usn_cleanup_add(Slapi_PBlock *pb, Slapi_Entry *e, Slapi_Entry *eAfter,
 
     *returncode = LDAP_SUCCESS;
 
+    /* get the requestor dn */
+    slapi_pblock_get(pb, SLAPI_REQUESTOR_DN, &bind_dn);
+
     /* make sure plugin is not closed*/
     if(!usn_is_started()){
         *returncode = LDAP_OPERATIONS_ERROR;
@@ -310,6 +320,7 @@ usn_cleanup_add(Slapi_PBlock *pb, Slapi_Entry *e, Slapi_Entry *eAfter,
       (struct usn_cleanup_data *)slapi_ch_malloc(sizeof(struct usn_cleanup_data));
     cleanup_data->suffix = slapi_ch_strdup(suffix);
     cleanup_data->maxusn_to_delete = slapi_ch_strdup(maxusn);
+    cleanup_data->bind_dn = slapi_ch_strdup(bind_dn);
 
     /* allocate new task now */
     task = slapi_new_task(slapi_entry_get_ndn(e));
