@@ -201,7 +201,7 @@ do_add( Slapi_PBlock *pb )
 				!= LDAP_SUCCESS ) {
 				slapi_log_access( LDAP_DEBUG_STATS, 
 					"conn=%" NSPRIu64 " op=%d ADD dn=\"%s\", add values for type %s failed\n",
-					pb->pb_conn->c_connid, operation->o_opid, 
+					(long long unsigned int)pb->pb_conn->c_connid, operation->o_opid,
 					slapi_entry_get_dn_const(e), normtype );
 				send_ldap_result( pb, rc, NULL, NULL, 0, NULL );
 
@@ -485,7 +485,7 @@ static void op_shared_add (Slapi_PBlock *pb)
 		if ( !internal_op )
 		{
 			slapi_log_access(LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d ADD dn=\"%s\"%s\n",
-							 pb->pb_conn->c_connid, 
+							 (long long unsigned int)pb->pb_conn->c_connid,
 							 operation->o_opid,
 							 slapi_entry_get_dn_const(e),
 							 proxystr ? proxystr : "");
@@ -600,7 +600,8 @@ static void op_shared_add (Slapi_PBlock *pb)
 #endif
 			}
 		}
-		if (unhashed_password_vals) {
+		if (unhashed_password_vals &&
+		    (SLAPD_UNHASHED_PW_OFF != config_get_unhashed_pw_switch())) {
 			/* unhashed_password_vals is consumed if successful. */
 			err = slapi_pw_set_entry_ext(e, unhashed_password_vals,
 			                             SLAPI_EXT_SET_ADD);
@@ -713,7 +714,7 @@ static void op_shared_add (Slapi_PBlock *pb)
 	sdn = slapi_sdn_dup(slapi_entry_get_sdn_const(e));
 	slapi_pblock_set(pb, SLAPI_ADD_TARGET_SDN, (void *)sdn);
 	if (plugin_call_plugins(pb, internal_op ? SLAPI_PLUGIN_INTERNAL_PRE_ADD_FN :
-	                        SLAPI_PLUGIN_PRE_ADD_FN) == 0)
+	                        SLAPI_PLUGIN_PRE_ADD_FN) == SLAPI_PLUGIN_SUCCESS)
 	{
 		int	rc;
 		Slapi_Entry	*ec;
