@@ -55,6 +55,7 @@ char	*attr_changetype	= ATTR_CHANGETYPE;
 char	*attr_newrdn		= ATTR_NEWRDN;
 char	*attr_deleteoldrdn	= ATTR_DELETEOLDRDN;
 char	*attr_modifiersname = ATTR_MODIFIERSNAME;
+static int hide_unhashed_pw = 1;
 
 /* Forward Declarations */
 static void write_audit_file( int optype, const char *dn, void *change, int flag, time_t curtime );
@@ -82,6 +83,9 @@ write_audit_log_entry( Slapi_PBlock *pb )
     case SLAPI_OPERATION_MODIFY:
         slapi_pblock_get( pb, SLAPI_MODIFY_MODS, &change );
         break;
+    case SLAPI_OPERATION_ADD:
+    	slapi_pblock_get( pb, SLAPI_ADD_ENTRY, &change );
+    	break;
     case SLAPI_OPERATION_DELETE:
         {
         char * deleterDN = NULL;
@@ -91,6 +95,7 @@ write_audit_log_entry( Slapi_PBlock *pb )
         break;
     
     case SLAPI_OPERATION_MODDN:
+        /* newrdn: change is just for logging -- case does not matter. */
         slapi_pblock_get( pb, SLAPI_MODRDN_NEWRDN, &change );
         slapi_pblock_get( pb, SLAPI_MODRDN_DELOLDRDN, &flag );
         break;
@@ -98,7 +103,8 @@ write_audit_log_entry( Slapi_PBlock *pb )
         return; /* Unsupported operation type. */
     }
     curtime = current_time();
-    dn = slapi_sdn_get_dn(sdn);
+    /* log the raw, unnormalized DN */
+    dn = slapi_sdn_get_udn(sdn);
     write_audit_file( operation_get_type(op), dn, change, flag, curtime );
 }
 
@@ -151,6 +157,10 @@ write_audit_file(
     	for ( j = 0; mods[j] != NULL; j++ )
 		{
 			int operationtype= mods[j]->mod_op & ~LDAP_MOD_BVALUES;
+
+			if((strcmp(mods[j]->mod_type, PSEUDO_ATTR_UNHASHEDUSERPASSWORD) == 0) && hide_unhashed_pw){
+				continue;
+			}
     	    switch ( operationtype )
 			{
     	    case LDAP_MOD_ADD:
@@ -245,3 +255,15 @@ write_audit_file(
 
     lenstr_free( &l );
 }
+
+void
+auditlog_hide_unhashed_pw()
+{
+	hide_unhashed_pw = 1;
+}
+
+void
+auditlog_expose_unhashed_pw()
+{
+	hide_unhashed_pw = 0;
+}
