@@ -409,6 +409,8 @@ struct cache {
 #define CACHE_ADD(cache, p, a) cache_add((cache), (void *)(p), (void **)(a))
 #define CACHE_RETURN(cache, p) cache_return((cache), (void **)(p))
 #define CACHE_REMOVE(cache, p) cache_remove((cache),  (void *)(p))
+#define CACHE_LOCK(cache) cache_lock((cache))
+#define CACHE_UNLOCK(cache) cache_unlock((cache))
 
 /* various modules keep private data inside the attrinfo structure */
 typedef struct dblayer_private dblayer_private;
@@ -462,6 +464,14 @@ typedef int (*dup_compare_fn_type)(
 #endif
 				const DBT *,const DBT *);
 
+struct index_idlistsizeinfo {
+	int ai_idlistsizelimit; /* max id list size */
+	int ai_indextype; /* index type */
+	unsigned int ai_flags;
+#define INDEX_ALLIDS_FLAG_AND 0x01
+	Slapi_ValueSet *ai_values; /* index keys to apply the max id list size to */
+};
+
 /* for the cache of attribute information (which are indexed, etc.) */
 struct attrinfo {
 	char	*ai_type;	  /* type name (cn, sn, ...)	*/
@@ -510,6 +520,7 @@ struct attrinfo {
 							 * the default length triplet is 2, 3, 2.
                              */
 	Slapi_Attr ai_sattr;	/* interface to syntax and matching rule plugins */
+	DataList *ai_idlistinfo; /* fine grained id list */
 };
 
 #define MAXDBCACHE	20
@@ -652,9 +663,20 @@ struct ldbminfo {
     int li_reslimit_pagedallids_handle; /* allids aka idlistscan */
     int li_rangelookthroughlimit;
     int li_reslimit_rangelookthrough_handle;
+    int li_idl_update;
+    int li_old_idl_maxids;
     int li_online_import_encrypt; /* toggle attribute encryption during ldbm_back_wire_import */
+#define BACKEND_OPT_NO_RUV_UPDATE	0x01
+#define BACKEND_OPT_DBLOCK_INSIDE_TXN	0x02
+#define BACKEND_OPT_MANAGE_ENTRY_BEFORE_DBLOCK 0x04
+    int li_backend_opt_level;
 };
 
+
+#define NO_RUV_UPDATE(li)		(li->li_backend_opt_level & BACKEND_OPT_NO_RUV_UPDATE)
+#define DBLOCK_INSIDE_TXN(li)		(li->li_backend_opt_level & BACKEND_OPT_DBLOCK_INSIDE_TXN)
+#define MANAGE_ENTRY_BEFORE_DBLOCK(li)	(li->li_backend_opt_level & BACKEND_OPT_MANAGE_ENTRY_BEFORE_DBLOCK)
+
 /* li_flags could store these bits defined in ../slapi-plugin.h
  * task flag (pb_task_flags) *
  *   SLAPI_TASK_RUNNING_AS_TASK
