@@ -67,6 +67,7 @@ static IDList * range_candidates(
 );
 static IDList *
 keys2idl(
+    Slapi_PBlock *pb,
     backend     *be,
     char        *type,
     const char  *indextype,
@@ -313,7 +314,7 @@ ava_candidates(
         ivals=ptr;
 
         slapi_attr_assertion2keys_ava_sv( &sattr, &tmp, (Slapi_Value ***)&ivals, LDAP_FILTER_EQUALITY_FAST);
-        idl = keys2idl( be, type, indextype, ivals, err, &unindexed, &txn, allidslimit );
+        idl = keys2idl( pb, be, type, indextype, ivals, err, &unindexed, &txn, allidslimit );
         if ( unindexed ) {
             unsigned int opnote = SLAPI_OP_NOTE_UNINDEXED;
             slapi_pblock_set( pb, SLAPI_OPERATION_NOTES, &opnote );
@@ -345,7 +346,7 @@ ava_candidates(
             idl = idl_allids( be );
             goto done;
         }
-        idl = keys2idl( be, type, indextype, ivals, err, &unindexed, &txn, allidslimit );
+        idl = keys2idl( pb, be, type, indextype, ivals, err, &unindexed, &txn, allidslimit );
         if ( unindexed ) {
             unsigned int opnote = SLAPI_OP_NOTE_UNINDEXED;
             slapi_pblock_set( pb, SLAPI_OPERATION_NOTES, &opnote );
@@ -382,7 +383,7 @@ presence_candidates(
         return( NULL );
     }
     slapi_pblock_get(pb, SLAPI_TXN, &txn.back_txn_txn);
-    idl = index_read_ext_allids( be, type, indextype_PRESENCE,
+    idl = index_read_ext_allids( pb, be, type, indextype_PRESENCE,
                                  NULL, &txn, err, &unindexed, allidslimit );
 
     if ( unindexed ) {
@@ -491,7 +492,7 @@ extensible_candidates(
                         {
                             int unindexed = 0;
                             IDList* idl3 = (mrOP == SLAPI_OP_EQUAL) ?
-                                index_read_ext_allids(be, mrTYPE, mrOID, *key, &txn,
+                                index_read_ext_allids(pb, be, mrTYPE, mrOID, *key, &txn,
                                                       err, &unindexed, allidslimit) :
                                 index_range_read_ext(pb, be, mrTYPE, mrOID, mrOP,
                                                      *key, NULL, 0, &txn, err, allidslimit);
@@ -631,6 +632,27 @@ done:
     return idl;
 }
 
+/*
+ * If the filter type contains subtype, it returns 1; otherwise, returns 0.
+ */
+static int
+filter_is_subtype(Slapi_Filter *f)
+{
+    char *p = NULL;
+    size_t len = 0;
+    int issubtype = 0;
+    if (f) {
+        p = strchr(f->f_type, ';');
+        if (p) {
+            len = p - f->f_type;
+            if (len < strlen(f->f_type)) {
+                issubtype = 1;
+            }
+        }
+    }
+    return issubtype;
+}
+
 static IDList *
 list_candidates(
     Slapi_PBlock  *pb,
@@ -778,7 +800,17 @@ list_candidates(
             /* Fetch the IDL for foo */
             /* Later we'll remember to call idl_notin() */
             LDAPDebug( LDAP_DEBUG_TRACE,"NOT filter\n", 0, 0, 0 );
-            tmp = ava_candidates( pb, be, slapi_filter_list_first(f), LDAP_FILTER_EQUALITY, nextf, range, err, allidslimit );
+            if (filter_is_subtype(slapi_filter_list_first(f))) {
+                /*
+                 * If subtype is included in the filter (e.g., !(cn;fr=<CN>)),
+                 * we have to give up using the index since the subtype info
+                 * is not in the index.
+                 */
+                tmp = idl_allids( be );
+            } else {
+                tmp = ava_candidates(pb, be, slapi_filter_list_first(f),
+                          LDAP_FILTER_EQUALITY, nextf, range, err, allidslimit);
+            }
         } else {
             if (fpairs[0] == f)
             {
@@ -821,12 +853,18 @@ list_candidates(
             }
         } else if ( ftype == LDAP_FILTER_AND ) {
             if (isnot) {
-                IDList *new_idl = NULL;
-                int notin_result = 0;
-                notin_result = idl_notin( be, idl, tmp, &new_idl );
-                if (notin_result) {
-                    idl_free(idl);
-                    idl = new_idl;
+                /*
+                 * If tmp is NULL or ALLID, idl_notin just duplicates idl.
+                 * We don't have to do it.
+                 */
+                if (!tmp && !idl_is_allids(tmp)) {
+                    IDList *new_idl = NULL;
+                    int notin_result = 0;
+                    notin_result = idl_notin( be, idl, tmp, &new_idl );
+                    if (notin_result) {
+                        idl_free(idl);
+                        idl = new_idl;
+                    }
                 }
             } else {
                 idl = idl_intersection(be, idl, tmp);
@@ -928,7 +966,7 @@ substring_candidates(
      * IDLists together.
      */
     slapi_pblock_get(pb, SLAPI_TXN, &txn.back_txn_txn);
-    idl = keys2idl( be, type, indextype_SUB, ivals, err, &unindexed, &txn, allidslimit );
+    idl = keys2idl( pb, be, type, indextype_SUB, ivals, err, &unindexed, &txn, allidslimit );
     if ( unindexed ) {
         slapi_pblock_set( pb, SLAPI_OPERATION_NOTES, &opnote );
         pagedresults_set_unindexed( pb->pb_conn, pb->pb_op, pr_idx );
@@ -942,6 +980,7 @@ substring_candidates(
 
 static IDList *
 keys2idl(
+    Slapi_PBlock *pb,
     backend     *be,
     char        *type,
     const char  *indextype,
@@ -961,7 +1000,7 @@ keys2idl(
     for ( i = 0; ivals[i] != NULL; i++ ) {
         IDList    *idl2;
 
-        idl2 = index_read_ext_allids( be, type, indextype, slapi_value_get_berval(ivals[i]), txn, err, unindexed, allidslimit );
+        idl2 = index_read_ext_allids( pb, be, type, indextype, slapi_value_get_berval(ivals[i]), txn, err, unindexed, allidslimit );
 
 #ifdef LDAP_DEBUG
         /* XXX if ( slapd_ldap_debug & LDAP_DEBUG_TRACE ) { XXX */
