@@ -83,7 +83,8 @@ void ldbm_log_access_message(Slapi_PBlock *pblock,char *string)
         return;
     }
     operation_id = operation->o_opid;
-    slapi_log_access( LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d %s\n",connection_id, operation_id,string);
+    slapi_log_access( LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d %s\n",
+            (long long unsigned int)connection_id, operation_id, string);
 }
 
 int return_on_disk_full(struct ldbminfo  *li)
@@ -296,7 +297,7 @@ ldbm_delete_dirs(char *path)
     PRDirEntry *direntry = NULL;
     char fullpath[MAXPATHLEN];
     int rval = 0;
-    PRFileInfo info;
+    PRFileInfo64 info;
 
     dirhandle = PR_OpenDir(path);
     if (! dirhandle)
@@ -312,7 +313,7 @@ ldbm_delete_dirs(char *path)
             break;
 
         PR_snprintf(fullpath, MAXPATHLEN, "%s/%s", path, direntry->name);
-        rval = PR_GetFileInfo(fullpath, &info);
+        rval = PR_GetFileInfo64(fullpath, &info);
         if (PR_SUCCESS == rval)
         {
             if (PR_FILE_DIRECTORY == info.type)
@@ -343,11 +344,11 @@ get_sep(char *path)
 int
 mkdir_p(char *dir, unsigned int mode)
 {
-    PRFileInfo info;
+    PRFileInfo64 info;
     int rval;
     char sep = get_sep(dir);
 
-    rval = PR_GetFileInfo(dir, &info);
+    rval = PR_GetFileInfo64(dir, &info);
     if (PR_SUCCESS == rval)
     {
         if (PR_FILE_DIRECTORY != info.type)    /* not a directory */
@@ -426,6 +427,7 @@ ldbm_txn_ruv_modify_context( Slapi_PBlock *pb, modify_context *mc )
     /* Either something went wrong when the RUV callback tried to assemble
      * the updates for us, or there were no updates because the op doesn't
      * target a replica. */
+    /* or, the CSN is already covered by the RUV */
     if (1 != rc || NULL == smods || NULL == uniqueid) {
         return (rc);
     }
