@@ -103,6 +103,7 @@ static void ndn_cache_update_lru(struct ndn_cache_lru **node);
 static void ndn_cache_add(char *dn, size_t dn_len, char *ndn, size_t ndn_len);
 static void ndn_cache_delete(char *dn);
 static void ndn_cache_flush();
+static void ndn_cache_free();
 static int ndn_started = 0;
 static PRLock *lru_lock = NULL;
 static Slapi_RWLock *ndn_cache_lock = NULL;
@@ -2011,6 +2012,17 @@ slapi_sdn_new_ndn_byref(const char *ndn)
     return sdn;
 }
 
+/* use when dn is already fully normalized */
+Slapi_DN *
+slapi_sdn_new_ndn_passin(const char *ndn)
+{
+    Slapi_DN *sdn = slapi_sdn_new();
+    slapi_sdn_set_ndn_passin(sdn, ndn);
+    SDN_DUMP( sdn, "slapi_sdn_new_ndn_passin");
+    return sdn;
+}
+
+
 /* use when dn is already normalized */
 Slapi_DN *
 slapi_sdn_new_normdn_byref(const char *normdn)
@@ -2154,6 +2166,20 @@ slapi_sdn_set_ndn_byref(Slapi_DN *sdn, const char *ndn)
     return sdn;
 }
 
+Slapi_DN *
+slapi_sdn_set_ndn_passin(Slapi_DN *sdn, const char *ndn)
+{
+    slapi_sdn_done(sdn);
+    sdn->flag = slapi_setbit_uchar(sdn->flag, FLAG_NDN);
+    sdn->ndn = ndn;
+    if (ndn == NULL) {
+        sdn->ndn_len = 0;
+    } else {
+        sdn->ndn_len = strlen(ndn);
+    }
+    return sdn;
+}
+
 /*
  * Set the RDN of the DN.
  */
@@ -2645,7 +2671,10 @@ slapi_sdn_scope_test_ext( const Slapi_DN *dn, const Slapi_DN *base, int scope, i
         }
         break;
     case LDAP_SCOPE_ONELEVEL:
-        if (flags & SLAPI_ENTRY_FLAG_TOMBSTONE) {
+#define RUVRDN SLAPI_ATTR_UNIQUEID "=" RUV_STORAGE_ENTRY_UNIQUEID ","
+        if ((flags & SLAPI_ENTRY_FLAG_TOMBSTONE) &&
+            (strncmp(slapi_sdn_get_ndn(dn), RUVRDN, sizeof(RUVRDN) - 1))) {
+            /* tombstones except RUV tombstone */
             Slapi_DN parent;
             slapi_sdn_init(&parent);
             slapi_sdn_get_parent(dn, &parent);
@@ -2708,18 +2737,22 @@ sdn_dump( const Slapi_DN *sdn, const char *text)
 size_t
 slapi_sdn_get_size(const Slapi_DN *sdn)
 {
-    size_t sz = sizeof(Slapi_DN);
+    size_t sz = 0;
     /* slapi_sdn_get_ndn_len returns the normalized dn length
      * if dn or ndn exists.  If both does not exist, it
      * normalizes udn and set it to dn and returns the length.
      */
-    sz += slapi_sdn_get_ndn_len(sdn);
+    if (NULL == sdn) {
+        return sz;
+    }
+    sz += slapi_sdn_get_ndn_len(sdn) + 1 /* '\0' */;
     if (sdn->dn && sdn->ndn) {
-        sz += slapi_sdn_get_ndn_len(sdn);
+        sz *= 2;
     }
     if (sdn->udn) {
         sz += strlen(sdn->udn) + 1;
     }
+    sz += sizeof(Slapi_DN);
     return sz;
 }
 
@@ -2748,7 +2781,7 @@ ndn_hash_string(const void *key)
 void
 ndn_cache_init()
 {
-    if(!config_get_ndn_cache_enabled()){
+    if(!config_get_ndn_cache_enabled() || ndn_started){
         return;
     }
     ndn_cache_hashtable = PL_NewHashTable( NDN_CACHE_BUCKETS, ndn_hash_string, PL_CompareStrings, PL_CompareValues, 0, 0);
@@ -2761,24 +2794,49 @@ ndn_cache_init()
     ndn_cache->cache_size = sizeof(struct ndn_cache_ctx) + sizeof(PLHashTable) + sizeof(PLHashTable);
     ndn_cache->head = NULL;
     ndn_cache->tail = NULL;
-
+    ndn_started = 1;
     if ( NULL == ( lru_lock = PR_NewLock()) ||  NULL == ( ndn_cache_lock = slapi_new_rwlock())) {
-        char *errorbuf = NULL;
-        if(ndn_cache_hashtable){
-            PL_HashTableDestroy(ndn_cache_hashtable);
-        }
-        ndn_cache_hashtable = NULL;
-        config_set_ndn_cache_enabled(CONFIG_NDN_CACHE, "off", errorbuf, 1 );
-        slapi_counter_destroy(&ndn_cache->cache_hits);
-        slapi_counter_destroy(&ndn_cache->cache_tries);
-        slapi_counter_destroy(&ndn_cache->cache_misses);
-        slapi_ch_free((void **)&ndn_cache);
+        ndn_cache_destroy();
         slapi_log_error( SLAPI_LOG_FATAL, "ndn_cache_init", "Failed to create locks.  Disabling cache.\n" );
-    } else {
-        ndn_started = 1;
     }
 }
 
+void
+ndn_cache_destroy()
+{
+    char *errorbuf = NULL;
+
+    if(!ndn_started){
+        return;
+    }
+    if(lru_lock){
+        PR_DestroyLock(lru_lock);
+        lru_lock = NULL;
+    }
+    if(ndn_cache_lock){
+        slapi_destroy_rwlock(ndn_cache_lock);
+        ndn_cache_lock = NULL;
+    }
+    if(ndn_cache_hashtable){
+    	ndn_cache_free();
+        PL_HashTableDestroy(ndn_cache_hashtable);
+        ndn_cache_hashtable = NULL;
+    }
+    config_set_ndn_cache_enabled(CONFIG_NDN_CACHE, "off", errorbuf, 1 );
+    slapi_counter_destroy(&ndn_cache->cache_hits);
+    slapi_counter_destroy(&ndn_cache->cache_tries);
+    slapi_counter_destroy(&ndn_cache->cache_misses);
+    slapi_ch_free((void **)&ndn_cache);
+
+    ndn_started = 0;
+}
+
+int
+ndn_cache_started()
+{
+    return ndn_started;
+}
+
 /*
  *  Look up this dn in the ndn cache
  */
@@ -2974,7 +3032,7 @@ ndn_cache_flush()
     int i;
 
     node = ndn_cache->tail;
-    for(i = 0; i < NDN_FLUSH_COUNT && ndn_cache->cache_count > NDN_MIN_COUNT; i++){
+    for(i = 0; node && i < NDN_FLUSH_COUNT && ndn_cache->cache_count > NDN_MIN_COUNT; i++){
         flush_node = node;
         /* update the lru */
         next = node->prev;
@@ -2991,19 +3049,48 @@ ndn_cache_flush()
     slapi_log_error( SLAPI_LOG_CACHE, "ndn_cache_flush","Flushed cache.\n");
 }
 
+static void
+ndn_cache_free()
+{
+    struct ndn_cache_lru *node, *next, *flush_node;
+
+    if(!ndn_cache){
+        return;
+    }
+
+    node = ndn_cache->tail;
+    while(node && ndn_cache->cache_count){
+        flush_node = node;
+        /* update the lru */
+        next = node->prev;
+        if(next){
+            next->next = NULL;
+        }
+        ndn_cache->tail = next;
+        node = next;
+        /* now update the hash */
+        ndn_cache->cache_count--;
+        ndn_cache_delete(flush_node->key);
+        slapi_ch_free_string(&flush_node->key);
+        slapi_ch_free((void **)&flush_node);
+    }
+}
+
 /* this is already "write" locked from ndn_cache_add */
 static void
 ndn_cache_delete(char *dn)
 {
-    struct ndn_hash_val *ht_val;
+    struct ndn_hash_val *ht_entry;
 
-    ht_val = (struct ndn_hash_val *)PL_HashTableLookupConst(ndn_cache_hashtable, dn);
-    if(ht_val){
-        ndn_cache->cache_size -= ht_val->size;
-        slapi_ch_free_string(&ht_val->ndn);
+    ht_entry = (struct ndn_hash_val *)PL_HashTableLookupConst(ndn_cache_hashtable, dn);
+    if(ht_entry){
+        ndn_cache->cache_size -= ht_entry->size;
+        slapi_ch_free_string(&ht_entry->ndn);
+        slapi_ch_free((void **)&ht_entry);
         PL_HashTableRemove(ndn_cache_hashtable, dn);
     }
 }
+
 /* stats for monitor */
 void
 ndn_cache_get_stats(PRUint64 *hits, PRUint64 *tries, size_t *size, size_t *max_size, long *count)
@@ -3017,3 +3104,71 @@ ndn_cache_get_stats(PRUint64 *hits, PRUint64 *tries, size_t *size, size_t *max_s
     slapi_rwlock_unlock(ndn_cache_lock);
 }
 
+/* Common ancestor sdn is allocated.
+ * caller is responsible to free it */
+Slapi_DN *
+slapi_sdn_common_ancestor(Slapi_DN *dn1, Slapi_DN *dn2)
+{
+    const char *dn1str = NULL;
+    const char *dn2str = NULL;
+    char **dns1 = NULL;
+    char **dns2 = NULL;
+    char **dn1p, **dn2p;
+    char **dn1end;
+    int dn1len = 0;
+    int dn2len = 0;
+    char *common = NULL;
+    char *cp = 0;
+    if ((NULL == dn1) || (NULL == dn2)) {
+        return NULL;
+    }
+    dn1str = slapi_sdn_get_ndn(dn1);
+    dn2str = slapi_sdn_get_ndn(dn2);
+    if (0 == strcmp(dn1str, dn2str)) {
+        /* identical */
+        return slapi_sdn_dup(dn1);
+    }
+    dn1len = strlen(dn1str);
+    dn2len = strlen(dn2str);
+    if (dn1len > dn2len) {
+        if (slapi_sdn_isparent(dn2, dn1)) {
+            /* dn2 is dn1's parent */
+            return slapi_sdn_dup(dn2);
+        }
+    } else if (dn1len < dn2len) {
+        if (slapi_sdn_isparent(dn1, dn2)) {
+            /* dn1 is dn2's parent */
+            return slapi_sdn_dup(dn1);
+        }
+    }
+    dns1 = slapi_ldap_explode_dn(slapi_sdn_get_ndn(dn1), 0);
+    dns2 = slapi_ldap_explode_dn(slapi_sdn_get_ndn(dn2), 0);
+    for (dn1p = dns1; dn1p && *dn1p; dn1p++) ;
+    for (dn2p = dns2; dn2p && *dn2p; dn2p++) ;
+    dn1end = dn1p;
+    while (--dn1p && --dn2p && (dn1p >= dns1) && (dn2p >= dns2)) {
+        if (strcmp(*dn1p, *dn2p)) {
+            break;
+        }
+    }
+    if (dn1end == ++dn1p) {
+        /* No common ancestor */
+        charray_free(dns1);
+        charray_free(dns2);
+        return NULL;
+    }
+    dn1len += 1;
+    cp = common = slapi_ch_malloc(dn1len);
+    *common = '\0';
+    do {
+        PR_snprintf(cp, dn1len, "%s,", *dn1p);
+        cp += strlen(*dn1p) + 1/*,*/;
+    } while (++dn1p < dn1end);
+    dn1len = strlen(common);
+    if (',' == *(common + dn1len - 1)) {
+        *(common + dn1len - 1) = '\0';
+    }
+    charray_free(dns1);
+    charray_free(dns2);
+    return slapi_sdn_new_ndn_passin(common);
+}
