@@ -81,6 +81,7 @@
 #define SLAPI_DSE_TRACELEVEL	LDAP_DEBUG_TRACE
 #endif /* SLAPI_DSE_DEBUG */
 
+#define SCHEMA_VIOLATION -2
 #define STOP_TRAVERSAL -2
 
 /* This is returned by dupentry_replace if the duplicate entry was found and
@@ -610,7 +611,7 @@ dse_updateNumSubordinates(Slapi_Entry *entry, int op)
         struct berval val;
         vals[0] = &val;
         vals[1] = NULL;
-        sprintf(value_buffer,"%lu",current_sub_count);
+        sprintf(value_buffer,"%lu",(long unsigned int)current_sub_count);
         val.bv_val = value_buffer;
         val.bv_len = strlen (val.bv_val);
         switch(mod_op)
@@ -659,9 +660,9 @@ int
 dse_check_file(char *filename, char *backupname)
 {
     int rc= 0; /* Fail */
-    PRFileInfo prfinfo;
+    PRFileInfo64 prfinfo;
 
-    if (PR_GetFileInfo( filename, &prfinfo ) == PR_SUCCESS) {
+    if (PR_GetFileInfo64( filename, &prfinfo ) == PR_SUCCESS) {
 	if ( prfinfo.size > 0)
 		return (1);
 	else {
@@ -674,7 +675,7 @@ dse_check_file(char *filename, char *backupname)
     else 
 	return (0);
 
-    if ( PR_GetFileInfo( filename, &prfinfo ) == PR_SUCCESS && prfinfo.size > 0 ) {
+    if ( PR_GetFileInfo64( filename, &prfinfo ) == PR_SUCCESS && prfinfo.size > 0 ) {
 	slapi_log_error(SLAPI_LOG_FATAL, "dse",
              "The configuration file %s was restored from backup %s\n", filename, backupname);
 	return (1);
@@ -695,7 +696,7 @@ dse_read_one_file(struct dse *pdse, const char *filename, Slapi_PBlock *pb,
     char *lastp = NULL;
     int rc= 0; /* Fail */
     PRInt32 nr = 0;
-    PRFileInfo prfinfo;
+    PRFileInfo64 prfinfo;
     PRFileDesc *prfd = 0;
     int schema_flags = 0;
 
@@ -704,11 +705,11 @@ dse_read_one_file(struct dse *pdse, const char *filename, Slapi_PBlock *pb,
     if ( (NULL != pdse) && (NULL != filename) )
     {
         /* check if the "real" file exists and cam be used, if not try tmp as backup */
-	rc = dse_check_file((char *)filename, pdse->dse_tmpfile);
-	if (!rc) 
-	    rc = dse_check_file((char *)filename, pdse->dse_fileback);
+        rc = dse_check_file((char *)filename, pdse->dse_tmpfile);
+        if (!rc)
+            rc = dse_check_file((char *)filename, pdse->dse_fileback);
 
-        if ( (rc = PR_GetFileInfo( filename, &prfinfo )) != PR_SUCCESS )
+        if ( (rc = PR_GetFileInfo64( filename, &prfinfo )) != PR_SUCCESS )
         {
             slapi_log_error(SLAPI_LOG_FATAL, "dse",
                             "The configuration file %s could not be accessed, error %d\n",
@@ -732,8 +733,8 @@ dse_read_one_file(struct dse *pdse, const char *filename, Slapi_PBlock *pb,
             if (( nr = slapi_read_buffer( prfd, buf, prfinfo.size )) < 0 )
             {
                 slapi_log_error(SLAPI_LOG_FATAL, "dse",
-                                "Could only read %d of %d bytes from config file %s\n",
-                                nr, prfinfo.size, filename);
+                                "Could only read %d of %ld bytes from config file %s\n",
+                                nr, (long int)prfinfo.size, filename);
                 rc = 0; /* Fail */
                 done= 1;
             }
@@ -799,9 +800,14 @@ dse_read_one_file(struct dse *pdse, const char *filename, Slapi_PBlock *pb,
                                     DSE_FLAG_PREOP, e, NULL, &returncode,
                                     returntext) == SLAPI_DSE_CALLBACK_OK)
                         {
-                            /* this will free the entry if not added, so it is
-                               definitely consumed by this call */
-                            dse_add_entry_pb(pdse, e, pb);
+                            /*
+                             * This will free the entry if not added, so it is
+                             * definitely consumed by this call
+                             */
+                            if(dse_add_entry_pb(pdse, e, pb) == SCHEMA_VIOLATION){
+                                /* schema violation, return failure */
+                                rc = 0;
+                            }
                         }
                         else /* free entry if not used */
                         {
@@ -1119,7 +1125,11 @@ dse_write_entry( caddr_t data, caddr_t arg )
   
 /*
  * Adds an entry to the dse backend.  The passed in entry will be
- * free'd always. */
+ * free'd always.
+ *
+ * return -1 for duplicate entry
+ * return -2 for schema violation (SCHEMA_VIOLATION)
+ */
 static int
 dse_add_entry_pb(struct dse* pdse, Slapi_Entry *e, Slapi_PBlock *pb)
 {
@@ -1190,7 +1200,9 @@ dse_add_entry_pb(struct dse* pdse, Slapi_Entry *e, Slapi_PBlock *pb)
 		 * Verify that the new or merged entry conforms to the schema.
 		 *		Errors are logged by slapi_entry_schema_check().
 		 */
-		(void)slapi_entry_schema_check( pb, schemacheckentry );
+		if(slapi_entry_schema_check( pb, schemacheckentry )){
+			rc = SCHEMA_VIOLATION;
+		}
 		slapi_entry_free(schemacheckentry);
 	}
 
