@@ -58,6 +58,8 @@ extern __declspec(dllimport) int slapd_ldap_debug;
 #endif
 extern Slapi_Counter *ops_initiated;
 extern Slapi_Counter *ops_completed;
+extern Slapi_Counter *max_threads_count;
+extern Slapi_Counter *conns_in_maxthreads;
 extern PRThread *listener_tid;
 extern PRThread *listener_tid;
 extern Slapi_Counter *num_conns;
@@ -93,6 +95,8 @@ int handle_bad_certificate (void* clientData, PRFileDesc *prfd);
  * connection.c
  */
 void op_thread_cleanup();
+/* do this after all worker threads have terminated */
+void connection_post_shutdown_cleanup();
 
 /*
  * ntuserpin.c - Prompts for the key database passphrase.
@@ -107,10 +111,11 @@ void SVRCORE_DestroyNTUserPinObj(SVRCORENTUserPinObj *obj);
  * connection.c
  */
 void connection_abandon_operations( Connection *conn );
-int connection_activity( Connection *conn );
+int connection_activity( Connection *conn, int maxthreads );
 void init_op_threads();
 int connection_new_private(Connection *conn);
 void connection_remove_operation( Connection *conn, Operation *op );
+void connection_remove_operation_ext( Slapi_PBlock *pb, Connection *conn, Operation *op );
 int connection_operations_pending( Connection *conn, Operation *op2ignore,
 		int test_resultsent );
 void connection_done(Connection *conn);
