@@ -124,11 +124,11 @@ get_filter( Connection *conn, BerElement *ber, int scope,
 }
 
 
-#define FILTER_EQ_FMT       "(%s=%s)"
-#define FILTER_GE_FMT       "(%s>=%s)"
-#define FILTER_LE_FMT       "(%s<=%s)"
-#define FILTER_APROX_FMT    "(%s~=%s)"
-#define FILTER_EXTENDED_FMT "(%s%s%s%s:=%s)"
+#define FILTER_EQ_FMT       "(%s=%s%s)"
+#define FILTER_GE_FMT       "(%s>=%s%s)"
+#define FILTER_LE_FMT       "(%s<=%s%s)"
+#define FILTER_APROX_FMT    "(%s~=%s%s)"
+#define FILTER_EXTENDED_FMT "(%s%s%s%s:=%s%s)"
 #define FILTER_EQ_LEN	4
 #define FILTER_GE_LEN	5
 #define FILTER_LE_LEN	5
@@ -140,16 +140,14 @@ get_filter( Connection *conn, BerElement *ber, int scope,
 static char *
 filter_escape_filter_value_extended(struct slapi_filter *f) 
 {
-    char ebuf[BUFSIZ], *ptr;
-    const char *estr;
+    char *ptr;
     
-    estr  = escape_filter_value( f->f_mr_value.bv_val, f->f_mr_value.bv_len, ebuf );
-	ptr = slapi_ch_smprintf(FILTER_EXTENDED_FMT,
+    ptr = slapi_filter_sprintf(FILTER_EXTENDED_FMT,
         f->f_mr_type ? f->f_mr_type : "",
         f->f_mr_dnAttrs ? ":dn" : "",
         f->f_mr_oid ? ":" : "",
         f->f_mr_oid ? f->f_mr_oid : "",
-        estr );
+        ESC_NEXT_VAL,  f->f_mr_value.bv_val);
     return ptr;
 }
 
@@ -158,12 +156,11 @@ filter_escape_filter_value_extended(struct slapi_filter *f)
 static char *
 filter_escape_filter_value(struct slapi_filter *f, const char *fmt, size_t len) 
 {
-    char ebuf[BUFSIZ], *ptr;
-    const char *estr;
+    char *ptr;
+
+    filter_compute_hash(f);
+    ptr = slapi_filter_sprintf(fmt, f->f_avtype, ESC_NEXT_VAL, f->f_avvalue.bv_val );
     
-	estr  = escape_filter_value( f->f_avvalue.bv_val, f->f_avvalue.bv_len, ebuf );
-	filter_compute_hash(f);
-    ptr = slapi_ch_smprintf(fmt, f->f_avtype, estr );
     return ptr;
 }
 
@@ -474,13 +471,13 @@ get_substring_filter(
 {
 	ber_tag_t	tag, rc;
 	ber_len_t	len = -1;
-	char		*val, *last, *type = NULL;
-	char		ebuf[BUFSIZ];
+	char		*val, *eval, *last, *type = NULL;
+	size_t fstr_len;
 
 	LDAPDebug( LDAP_DEBUG_FILTER, "=> get_substring_filter\n", 0, 0, 0 );
 
 	if ( ber_scanf( ber, "{a", &type ) == LBER_ERROR ) {
-        slapi_ch_free_string(&type);
+		slapi_ch_free_string(&type);
 		return( LDAP_PROTOCOL_ERROR );
 	}
 	f->f_sub_type = slapi_attr_syntax_normalize( type );
@@ -489,7 +486,9 @@ get_substring_filter(
 	f->f_sub_any = NULL;
 	f->f_sub_final = NULL;
 
-	*fstr = slapi_ch_malloc( strlen( f->f_sub_type ) + 3 );
+	/* borrowing the handy macro: 256 */
+	fstr_len = strlen( f->f_sub_type ) + SLAPD_TYPICAL_ATTRIBUTE_NAME_MAX_LENGTH;
+	*fstr = slapi_ch_malloc(fstr_len);
 	sprintf( *fstr, "(%s=", f->f_sub_type );
 	for ( tag = ber_first_element( ber, &len, &last );
 	    tag != LBER_ERROR && tag != LBER_END_OF_SEQORSET;
@@ -516,22 +515,30 @@ get_substring_filter(
 				return( LDAP_PROTOCOL_ERROR );
 			}
 			f->f_sub_initial = val;
-    	    /* jcm: Had to cast away a const */
-			val = (char*)escape_filter_value( val, -1, ebuf );
-			*fstr = slapi_ch_realloc( *fstr, strlen( *fstr ) +
-			    strlen( val ) + 1 );
-			strcat( *fstr, val );
+			eval = (char*)slapi_escape_filter_value(val, -1);
+			if(eval) {
+				if (fstr_len < strlen(*fstr) + strlen(eval) + 1) {
+				    fstr_len += (strlen(eval) + 1) * 2;
+				    *fstr = slapi_ch_realloc(*fstr, fstr_len);
+				}
+				strcat(*fstr, eval);
+				slapi_ch_free_string(&eval);
+			}
 			break;
 
 		case LDAP_SUBSTRING_ANY:
 			LDAPDebug( LDAP_DEBUG_FILTER, "  ANY\n", 0, 0, 0 );
-			charray_add( &f->f_sub_any, val );
-    	    /* jcm: Had to cast away a const */
-			val = (char*)escape_filter_value( val, -1, ebuf );
-			*fstr = slapi_ch_realloc( *fstr, strlen( *fstr ) +
-			    strlen( val ) + 2 );
-			strcat( *fstr, "*" );
-			strcat( *fstr, val );
+			charray_add(&f->f_sub_any, val);
+			eval = (char*)slapi_escape_filter_value(val, -1);
+			if(eval){
+				if (fstr_len < strlen(*fstr) + strlen(eval) + 1) {
+				    fstr_len += (strlen(eval) + 1) * 2;
+				    *fstr = slapi_ch_realloc(*fstr, fstr_len);
+				}
+				strcat(*fstr, "*");
+				strcat(*fstr, eval);
+				slapi_ch_free_string(&eval);
+			}
 			break;
 
 		case LDAP_SUBSTRING_FINAL:
@@ -540,12 +547,16 @@ get_substring_filter(
 				return( LDAP_PROTOCOL_ERROR );
 			}
 			f->f_sub_final = val;
-    	    /* jcm: Had to cast away a const */
-			val = (char*)escape_filter_value( val, -1, ebuf );
-			*fstr = slapi_ch_realloc( *fstr, strlen( *fstr ) +
-			    strlen( val ) + 2 );
-			strcat( *fstr, "*" );
-			strcat( *fstr, val );
+			eval = (char*)slapi_escape_filter_value( val, -1);
+			if(eval){
+				if (fstr_len < strlen(*fstr) + strlen(eval) + 1) {
+				    fstr_len += (strlen(eval) + 1) * 2;
+				    *fstr = slapi_ch_realloc(*fstr, fstr_len);
+				}
+				strcat(*fstr, "*");
+				strcat(*fstr, eval);
+				slapi_ch_free_string(&eval);
+			}
 			break;
 
 		default:
@@ -564,7 +575,10 @@ get_substring_filter(
 	}
 
 	filter_compute_hash(f);
-	*fstr = slapi_ch_realloc( *fstr, strlen( *fstr ) + 3 );
+	if (fstr_len < strlen(*fstr) + 3) {
+		fstr_len += 3;
+		*fstr = slapi_ch_realloc(*fstr, fstr_len);
+	}
 	if ( f->f_sub_final == NULL ) {
 		strcat( *fstr, "*" );
 	}
