@@ -49,6 +49,7 @@
 #include <sys/socket.h>
 #endif
 #include "slap.h"
+#include "fe.h"
 
 int
 slapi_op_abandoned( Slapi_PBlock *pb )
@@ -161,32 +162,12 @@ ber_special_free(void* buf, BerElement *ber)
 }
 #endif
 
-/*
- * Allocate a new Slapi_Operation.
- * The flag parameter indicates whether the the operation is
- * external (from an LDAP Client), or internal (from a plugin).
- */
-Slapi_Operation *
-operation_new(int flags)
+void
+operation_init(Slapi_Operation *o, int flags)
 {
-	/* To improve performance, we allocate the Operation, BerElement and
-	 * ber buffer in one block, instead of a separate malloc() for each.
-	 * Subsequently, ber_special_free() frees them all; we're careful
-	 * not to free the Operation separately, and the ber software knows
-	 * not to free the buffer separately.
-	 */
-	BerElement *ber = NULL;
-	Slapi_Operation *o;
-	if(flags & OP_FLAG_INTERNAL)
-	{
-	   	o = (Slapi_Operation *) slapi_ch_malloc(sizeof(Slapi_Operation));
-	}
-	else
-	{
-		o= (Slapi_Operation *) ber_special_alloc( sizeof(Slapi_Operation), &ber );
-	}
 	if (NULL != o)
 	{
+		BerElement *ber = o->o_ber; /* may have already been set */
 		memset(o,0,sizeof(Slapi_Operation));
 		o->o_ber = ber;
 		o->o_msgid = -1;
@@ -207,29 +188,90 @@ operation_new(int flags)
 		}
 		o->o_pagedresults_sizelimit = -1;
 	}
+
+}
+
+/*
+ * Allocate a new Slapi_Operation.
+ * The flag parameter indicates whether the the operation is
+ * external (from an LDAP Client), or internal (from a plugin).
+ */
+Slapi_Operation *
+operation_new(int flags)
+{
+	/* To improve performance, we allocate the Operation, BerElement and
+	 * ber buffer in one block, instead of a separate malloc() for each.
+	 * Subsequently, ber_special_free() frees them all; we're careful
+	 * not to free the Operation separately, and the ber software knows
+	 * not to free the buffer separately.
+	 */
+	Slapi_Operation *o;
+	BerElement *ber = NULL;
+	if(flags & OP_FLAG_INTERNAL)
+	{
+	   	o = (Slapi_Operation *) slapi_ch_malloc(sizeof(Slapi_Operation));
+	}
+	else
+	{
+		o= (Slapi_Operation *) ber_special_alloc( sizeof(Slapi_Operation), &ber );
+	}
+	if (NULL != o)
+	{
+	   	o->o_ber = ber;
+		operation_init(o, flags);
+	}
 	return o;
 }
 
 void
-operation_free( Slapi_Operation **op, Connection *conn )
+operation_done( Slapi_Operation **op, Connection *conn )
 {
 	if(op!=NULL && *op!=NULL)
 	{
+
 		/* Call the plugin extension destructors */
-	    factory_destroy_extension(get_operation_object_type(),*op,conn,&((*op)->o_extension));
+		factory_destroy_extension(get_operation_object_type(),*op,conn,&((*op)->o_extension));
 		slapi_sdn_done(&(*op)->o_sdn);
 		slapi_sdn_free(&(*op)->o_target_spec);
 		slapi_ch_free_string( &(*op)->o_authtype );
 		if ( (*op)->o_searchattrs != NULL ) {
 			charray_free( (*op)->o_searchattrs );
+			(*op)->o_searchattrs = NULL;
 		}
 		if ( NULL != (*op)->o_params.request_controls ) {
 			ldap_controls_free( (*op)->o_params.request_controls );
+			(*op)->o_params.request_controls = NULL;
 		}
 		if ( NULL != (*op)->o_results.result_controls ) {
 			ldap_controls_free( (*op)->o_results.result_controls );
+			(*op)->o_results.result_controls = NULL;
 		}
 		slapi_ch_free_string(&(*op)->o_results.result_matched);
+#if defined(USE_OPENLDAP)
+		int options = 0;
+		/* save the old options */
+		if ((*op)->o_ber) {
+			ber_get_option((*op)->o_ber, LBER_OPT_BER_OPTIONS, &options);
+			/* we don't have a way to reuse the BerElement buffer so just free it */
+			ber_free_buf((*op)->o_ber);
+			/* clear out the ber for the next operation */
+			ber_init2((*op)->o_ber, NULL, options);
+		}
+#else
+		if((*op)->o_ber){
+			ber_special_free(*op, (*op)->o_ber); /* have to free everything here */
+			*op = NULL;
+		}
+#endif
+	}
+}
+
+void
+operation_free( Slapi_Operation **op, Connection *conn )
+{
+	operation_done(op, conn);
+	if(op!=NULL && *op!=NULL)
+	{
 		if(operation_is_flag_set(*op, OP_FLAG_INTERNAL))
 		{
 			slapi_ch_free((void**)op);
@@ -515,3 +557,60 @@ void operation_parameters_free(struct slapi_operation_parameters **sop)
 		slapi_ch_free ((void**)sop);
 	}
 }
+
+int slapi_connection_acquire(Slapi_Connection *conn)
+{
+    int rc;
+
+    PR_Lock(conn->c_mutex);
+    /* rc = connection_acquire_nolock(conn); */
+    /* connection in the closing state can't be acquired */
+    if (conn->c_flags & CONN_FLAG_CLOSING)
+    {
+	/* This may happen while other threads are still working on this connection */
+        slapi_log_error(SLAPI_LOG_FATAL, "connection",
+		                "conn=%" NSPRIu64 " fd=%d Attempt to acquire connection in the closing state\n",
+		                (long long unsigned int)conn->c_connid, conn->c_sd);
+        rc = -1;
+    }
+    else
+    {
+        conn->c_refcnt++;
+        rc = 0;
+    }
+    PR_Unlock(conn->c_mutex);
+    return(rc);
+}
+
+int
+slapi_connection_remove_operation( Slapi_PBlock *pb, Slapi_Connection *conn, Slapi_Operation *op, int release)
+{
+	int rc = 0;
+	Slapi_Operation **olist= &conn->c_ops;
+	Slapi_Operation **tmp;
+	PR_Lock( conn->c_mutex );
+	/* connection_remove_operation_ext(pb, conn,op); */
+	for ( tmp = olist; *tmp != NULL && *tmp != op; tmp = &(*tmp)->o_next )
+		;	/* NULL */
+	if ( *tmp == NULL ) {
+		LDAPDebug( LDAP_DEBUG_ANY, "connection_remove_operation: can't find op %d for conn %" NSPRIu64 "\n",
+		    (int)op->o_msgid, conn->c_connid, 0 );
+	} else {
+		*tmp = (*tmp)->o_next;
+	}
+
+	if (release) {
+		/* connection_release_nolock(conn); */
+		if (conn->c_refcnt <= 0) {
+        		slapi_log_error(SLAPI_LOG_FATAL, "connection",
+		                "conn=%" NSPRIu64 " fd=%d Attempt to release connection that is not acquired\n",
+		                (long long unsigned int)conn->c_connid, conn->c_sd);
+        		rc = -1;
+		} else {
+        		conn->c_refcnt--;
+			rc = 0;
+		}
+	}
+	PR_Unlock( conn->c_mutex );
+	return (rc);
+}
