@@ -534,7 +534,7 @@ passwd_modify_extop( Slapi_PBlock *pb )
 	/* Get the ber value of the extended operation */
 	slapi_pblock_get(pb, SLAPI_EXT_OP_REQ_VALUE, &extop_value);
 
-	if (extop_value->bv_val == NULL)
+	if (!BV_HAS_DATA(extop_value))
 	{
 		/* The request field wasn't provided.  We'll
 		 * now try to determine the userid and verify
