@@ -268,6 +268,7 @@ slapi_register_plugin_ext(
 )
 {
 	int ii = 0;
+        int found_precedence;
     int rc = 0;
 	Slapi_Entry *e = NULL;
 	char *dn = slapi_ch_smprintf("cn=%s,%s", name, PLUGIN_BASE_DN);
@@ -284,7 +285,21 @@ slapi_register_plugin_ext(
 		slapi_entry_attr_set_charptr(e, ATTR_PLUGIN_ENABLED, "off");
 
 	slapi_entry_attr_set_charptr(e, ATTR_PLUGIN_INITFN, initsymbol);
-	slapi_entry_attr_set_int(e, ATTR_PLUGIN_PRECEDENCE, precedence);
+        /* If the plugin belong to a group, get the precedence from the group */
+        found_precedence = precedence;
+        if ((found_precedence == PLUGIN_DEFAULT_PRECEDENCE) && group_identity) {
+                struct slapi_componentid * cid = (struct slapi_componentid *) group_identity;
+                if (cid->sci_plugin && 
+                        (cid->sci_plugin->plg_precedence != PLUGIN_DEFAULT_PRECEDENCE)) {
+                        slapi_log_error(SLAPI_LOG_PLUGIN, NULL,
+                                "Plugin precedence (%s) reset to group precedence (%s): %d \n", 
+                                name ? name : "",
+                                cid->sci_plugin->plg_name ? cid->sci_plugin->plg_name : "",
+                                cid->sci_plugin->plg_precedence);
+                        found_precedence = cid->sci_plugin->plg_precedence;
+                }
+        }
+	slapi_entry_attr_set_int(e, ATTR_PLUGIN_PRECEDENCE, found_precedence);
 
 	for (ii = 0; argv && argv[ii]; ++ii) {
 		char argname[64];
@@ -586,6 +601,15 @@ plugin_get_pwd_storage_scheme_list(int index)
 	return( names_list );
 }
 
+int slapi_send_ldap_intermediate( Slapi_PBlock *pb, LDAPControl **ectrls,
+	char *responseName, struct berval *responseValue)
+{
+	/* no SLAPI_PLUGIN_DB_INTERMEDIATE_FN defined
+	 * always directly call slapd_ function
+	 */
+	return send_ldap_intermediate(pb, ectrls, responseName, responseValue);
+}
+
 int
 slapi_send_ldap_search_entry( Slapi_PBlock *pb, Slapi_Entry *e, LDAPControl **ectrls,
 	char **attrs, int attrsonly )
@@ -1181,11 +1205,6 @@ plugin_dependency_startall(int argc, char** argv, char *errmsg, int operation)
 					Slapi_Entry *newe;
 
 					pblock_init(&newpb);
-					/* 
-					 * config[plugin_index].e is freed up by
-					 * below function calls, but we may need
-					 * it later, so create a copy
-					 */
 					newe = slapi_entry_dup( config[plugin_index].e );
 					slapi_add_entry_internal_set_pb(&newpb, newe, NULL,
 									plugin_get_default_component_id(), plugin_actions);
@@ -1247,6 +1266,10 @@ plugin_dependency_startall(int argc, char** argv, char *errmsg, int operation)
 							index++;
 						}	
 					}
+				} else {
+					pblock_done(&(config[plugin_index].pb));
+					slapi_entry_free(config[plugin_index].e);
+					config[plugin_index].e = NULL;
 				}
 
 				/* decrement the type counter for this plugin type */
@@ -1366,6 +1389,8 @@ plugin_dependency_closeall()
 		    /* set plg_closed to 1 to prevent any further plugin pre/post op function calls */
 		    global_plugin_shutdown_order[index].plugin->plg_closed = 1;
 			plugins_closed++;
+			slapi_entry_free(global_plugin_shutdown_order[index].e);
+			global_plugin_shutdown_order[index].e = NULL;
 		}
 
 		index++;
@@ -1405,7 +1430,18 @@ plugin_startall(int argc, char** argv, int start_backends, int start_global)
 void 
 plugin_closeall(int close_backends, int close_globals)
 {
+	entry_and_plugin_t *iterp, *nextp;
+
 	plugin_dependency_closeall();
+	/* free the plugin dependency entry list */
+	iterp = dep_plugin_entries;
+	while (iterp) {
+		nextp = iterp->next;
+		slapi_entry_free(iterp->e);
+		slapi_ch_free((void **)&iterp);
+		iterp = nextp;
+	}
+	dep_plugin_entries = NULL;
 }
 
 
@@ -1467,7 +1503,8 @@ plugin_call_func (struct slapdplugin *list, int operation, Slapi_PBlock *pb, int
 				}
 				else if (SLAPI_PLUGIN_BEPREOPERATION == list->plg_type ||
 				         SLAPI_PLUGIN_BETXNPREOPERATION == list->plg_type ||
-				         SLAPI_PLUGIN_BEPOSTOPERATION == list->plg_type)
+				         SLAPI_PLUGIN_BEPOSTOPERATION == list->plg_type ||
+				         SLAPI_PLUGIN_BETXNPOSTOPERATION == list->plg_type )
 				{
 					/* 
 					 * respect fatal error SLAPI_PLUGIN_FAILURE (-1);
@@ -3227,3 +3264,14 @@ slapi_set_plugin_open_rootdn_bind(Slapi_PBlock *pb){
 
 	ptd_set_special_data(&(config->plgc_bind_subtrees), PLGC_DATA_BIND_ROOT);
 }
+
+PRBool
+slapi_disordely_shutdown(PRBool set)
+{
+    static PRBool is_disordely_shutdown = PR_FALSE;
+    
+    if (set) {
+        is_disordely_shutdown = PR_TRUE;
+    }
+    return (is_disordely_shutdown);
+}
