@@ -121,11 +121,10 @@ int attr_syntax_exists (const char *attr_name);
 void attr_syntax_delete ( struct asyntaxinfo *asip );
 #define SLAPI_SYNTAXLENGTH_NONE		(-1)	/* for syntaxlength parameter */
 int attr_syntax_create( const char *attr_oid, char *const*attr_names,
-		int num_names, const char *attr_desc, const char *attr_superior,
+		const char *attr_desc, const char *attr_superior,
 		const char *mr_equality, const char *mr_ordering,
-		const char *mr_substring, char *const *attr_origins,
-		const char *attr_syntax, int syntaxlength, unsigned long flags,
-		struct asyntaxinfo **asip );
+		const char *mr_substring, schemaext *extensions, const char *attr_syntax,
+		int syntaxlength, unsigned long flags, struct asyntaxinfo **asip );
 void attr_syntax_free( struct asyntaxinfo *a );
 int attr_syntax_add( struct asyntaxinfo *asip );
 char *attr_syntax_normalize_no_lookup( const char *s );
@@ -158,6 +157,7 @@ int valuearray_init_bervalarray(struct berval **bvals, Slapi_Value ***cvals);
 int valuearray_init_bervalarray_with_flags(struct berval **bvals, Slapi_Value ***cvals, unsigned long flags);
 int valuearray_get_bervalarray(Slapi_Value **cvals, struct berval ***bvals); /* JCM SLOW FUNCTION */
 void valuearray_free(Slapi_Value ***va);
+void valuearray_free_ext(Slapi_Value ***va, int ii);
 Slapi_Value *valuearray_remove_value(const Slapi_Attr *a, Slapi_Value **va, const Slapi_Value *v);
 void valuearray_remove_value_atindex(Slapi_Value **va, int index);
 int valuearray_isempty( Slapi_Value **va);
@@ -180,15 +180,16 @@ Slapi_Value **valueset_get_valuearray(const Slapi_ValueSet *vs);
 size_t valueset_size(const Slapi_ValueSet *vs);
 void slapi_valueset_add_valuearray(const Slapi_Attr *a, Slapi_ValueSet *vs, Slapi_Value **addvals);
 void valueset_add_valuearray(Slapi_ValueSet *vs, Slapi_Value **addvals);
-void valueset_add_valuearray_ext(Slapi_ValueSet *vs, Slapi_Value **addvals, PRUint32 flags);
 void valueset_add_string(const Slapi_Attr *a, Slapi_ValueSet *vs, const char *s, CSNType t, const CSN *csn);
 void valueset_update_csn(Slapi_ValueSet *vs, CSNType t, const CSN *csn);
+int valueset_intersectswith_valuearray(Slapi_ValueSet *vs, const Slapi_Attr *a, Slapi_Value **values, int *duplicate_index);
 void valueset_set_valueset(Slapi_ValueSet *vs1, const Slapi_ValueSet *vs2);
 Slapi_ValueSet *valueset_dup(const Slapi_ValueSet *dupee);
 void valueset_remove_string(const Slapi_Attr *a, Slapi_ValueSet *vs, const char *s);
 int valueset_replace_valuearray(Slapi_Attr *a, Slapi_ValueSet *vs, Slapi_Value **vals);
 int valueset_replace_valuearray_ext(Slapi_Attr *a, Slapi_ValueSet *vs, Slapi_Value **vals, int dupcheck);
 void valueset_update_csn_for_valuearray(Slapi_ValueSet *vs, const Slapi_Attr *a, Slapi_Value **valuestoupdate, CSNType t, const CSN *csn, Slapi_Value ***valuesupdated);
+void valueset_update_csn_for_valuearray_ext(Slapi_ValueSet *vs, const Slapi_Attr *a, Slapi_Value **valuestoupdate, CSNType t, const CSN *csn, Slapi_Value ***valuesupdated, int csnref_updated);
 void valueset_set_valuearray_byval(Slapi_ValueSet *vs, Slapi_Value **addvals);
 void valueset_set_valuearray_passin(Slapi_ValueSet *vs, Slapi_Value **addvals);
 int valuearray_subtract_bvalues(Slapi_Value **va, struct berval **bvals);
@@ -267,6 +268,7 @@ int config_set_localhost( const char *attrname, char *value, char *errorbuf, int
 int config_set_listenhost( const char *attrname, char *value, char *errorbuf, int apply );
 int config_set_securelistenhost( const char *attrname, char *value, char *errorbuf, int apply );
 int config_set_ldapi_filename( const char *attrname, char *value, char *errorbuf, int apply );
+int config_set_snmp_index( const char *attrname, char *value, char *errorbuf, int apply );
 int config_set_ldapi_switch( const char *attrname, char *value, char *errorbuf, int apply );
 int config_set_ldapi_bind_switch( const char *attrname, char *value, char *errorbuf, int apply );
 int config_set_ldapi_root_dn( const char *attrname, char *value, char *errorbuf, int apply );
@@ -396,6 +398,12 @@ int config_set_unhashed_pw_switch(const char *attrname, char *value, char *error
 int config_set_return_orig_type_switch(const char *attrname, char *value, char *errorbuf, int apply);
 int config_set_sasl_maxbufsize(const char *attrname, char *value, char *errorbuf, int apply );
 int config_set_listen_backlog_size(const char *attrname, char *value, char *errorbuf, int apply);
+int config_set_ignore_time_skew(const char *attrname, char *value, char *errorbuf, int apply);
+#if defined(LINUX)
+int config_set_malloc_mxfast(const char *attrname, char *value, char *errorbuf, int apply);
+int config_set_malloc_trim_threshold(const char *attrname, char *value, char *errorbuf, int apply);
+int config_set_malloc_mmap_threshold(const char *attrname, char *value, char *errorbuf, int apply);
+#endif
 
 #if !defined(_WIN32) && !defined(AIX)
 int config_set_maxdescriptors( const char *attrname, char *value, char *errorbuf, int apply );
@@ -547,7 +555,7 @@ void config_set_accesslog_enabled(int value);
 void config_set_auditlog_enabled(int value);
 int config_get_accesslog_logging_enabled();
 int config_get_disk_monitoring();
-PRUint64 config_get_disk_threshold();
+PRInt64 config_get_disk_threshold();
 int config_get_disk_grace_period();
 int config_get_disk_logging_critical();
 int config_get_ndn_cache_count();
@@ -565,10 +573,23 @@ int config_get_unhashed_pw_switch();
 int config_get_sasl_maxbufsize();
 int config_get_enable_turbo_mode();
 int config_set_enable_turbo_mode(const char *attrname, char *value, char *errorbuf, int apply);
+int config_get_connection_buffer();
+int config_set_connection_buffer(const char *attrname, char *value, char *errorbuf, int apply);
+int config_get_connection_nocanon();
+int config_get_plugin_logging();
+int config_set_connection_nocanon(const char *attrname, char *value, char *errorbuf, int apply);
+int config_set_plugin_logging(const char *attrname, char *value, char *errorbuf, int apply);
 int config_get_listen_backlog_size(void);
 
 PLHashNumber hashNocaseString(const void *key);
 PRIntn hashNocaseCompare(const void *v1, const void *v2);
+int config_get_ignore_time_skew();
+
+#if defined(LINUX)
+int config_get_malloc_mxfast();
+int config_get_malloc_trim_threshold();
+int config_get_malloc_mmap_threshold();
+#endif
 
 int is_abspath(const char *);
 char* rel2abspath( char * );
@@ -823,7 +844,9 @@ char *slapd_get_version_value( void );
 /*
  * operation.c
  */
+void operation_init(Slapi_Operation *op, int flags);
 Slapi_Operation *operation_new(int flags);
+void operation_done( Slapi_Operation **op, Connection *conn );
 void operation_free( Slapi_Operation **op, Connection *conn );
 int slapi_op_abandoned( Slapi_PBlock *pb );
 void operation_out_of_disk_space();
@@ -946,6 +969,8 @@ int send_ldap_search_entry_ext( Slapi_PBlock *pb, Slapi_Entry *e, LDAPControl **
 	char **attrs, int attrsonly, int send_result, int nentries, struct berval **urls );
 void send_ldap_result_ext( Slapi_PBlock *pb, int err, char *matched, char *text,
 	int nentries, struct berval **urls, BerElement	*ber );
+int send_ldap_intermediate( Slapi_PBlock *pb, LDAPControl **ectrls,
+	char *responseName, struct berval *responseValue);
 void send_nobackend_ldap_result( Slapi_PBlock *pb );
 int send_ldap_referral( Slapi_PBlock *pb, Slapi_Entry *e, struct berval **refs,
 	struct berval ***urls );
@@ -985,6 +1010,9 @@ void slapi_schema_expand_objectclasses( Slapi_Entry *e );
 int slapi_validate_schema_files(char *schemadir);
 /* API to reload the schema files */
 int slapi_reload_schema_files(char *schemadir);
+void schema_free_extensions(schemaext *extensions);
+schemaext *schema_copy_extensions(schemaext *extensions);
+int schema_objectclasses_superset_check(struct berval **remote_schema, char *type);
 
 /*
  * schemaparse.c
@@ -1009,6 +1037,7 @@ int slapd_ssl_init2(PRFileDesc **fd, int startTLS);
 int slapd_security_library_is_initialized();
 int slapd_ssl_listener_is_initialized();
 int slapd_SSL_client_auth (LDAP* ld);
+SECKEYPrivateKey *slapd_get_unlocked_key_for_cert(CERTCertificate *cert, void *pin_arg);
 
 /*
  * security_wrappers.c
@@ -1344,10 +1373,6 @@ void subentry_create_filter(Slapi_Filter** filter);
  */
 void vattr_init();
 void vattr_cleanup();
-void vattrcache_entry_READ_LOCK(const Slapi_Entry *e);
-void vattrcache_entry_READ_UNLOCK(const Slapi_Entry *e);
-void vattrcache_entry_WRITE_LOCK(const Slapi_Entry *e);
-void vattrcache_entry_WRITE_UNLOCK(const Slapi_Entry *e);
 
 /*
  * slapd_plhash.c - supplement to NSPR plhash
