@@ -72,7 +72,6 @@ typedef struct _ps_entry_queue_node {
 typedef struct _psearch {
     Slapi_PBlock	*ps_pblock;
     PRLock		*ps_lock;
-    PRThread		*ps_tid;
     PRInt32		ps_complete;
     PSEQNode		*ps_eq_head;
     PSEQNode		*ps_eq_tail;
@@ -171,8 +170,13 @@ ps_stop_psearch_system()
 	}
 }
 
-
-
+static Slapi_PBlock *
+pblock_copy(Slapi_PBlock *src)
+{
+	Slapi_PBlock *dest = slapi_pblock_new();
+	*dest = *src;
+	return dest;
+}
 
 /*
  * Add the given pblock to the list of outstanding persistent searches.
@@ -183,11 +187,12 @@ void
 ps_add( Slapi_PBlock *pb, ber_int_t changetypes, int send_entchg_controls )
 {
     PSearch	*ps;
+    PRThread		*ps_tid;
 
     if ( PS_IS_INITIALIZED() && NULL != pb ) {
 	/* Create the new node */
 	ps = psearch_alloc();
-	ps->ps_pblock = pb;
+	ps->ps_pblock = pblock_copy(pb);
 	ps->ps_changetypes = changetypes;
 	ps->ps_send_entchg_controls = send_entchg_controls;
 
@@ -195,7 +200,7 @@ ps_add( Slapi_PBlock *pb, ber_int_t changetypes, int send_entchg_controls )
 	ps_add_ps( ps );
 
 	/* Start a thread to send the results */
-	ps->ps_tid = PR_CreateThread( PR_USER_THREAD, ps_send_results,
+	ps_tid = PR_CreateThread( PR_USER_THREAD, ps_send_results,
 		(void *) ps, PR_PRIORITY_NORMAL, PR_GLOBAL_THREAD,
 		PR_UNJOINABLE_THREAD, SLAPD_DEFAULT_THREAD_STACKSIZE );
 
@@ -203,7 +208,7 @@ ps_add( Slapi_PBlock *pb, ber_int_t changetypes, int send_entchg_controls )
          * if the thread is not created succesfully.... we send 
          * error messages to the Log file 
          */ 
-       if(NULL == (ps->ps_tid)){ 
+       if(NULL == ps_tid){
             int prerr;
             prerr = PR_GetError(); 
             LDAPDebug(LDAP_DEBUG_ANY,"persistent search PR_CreateThread()failed in the " 
@@ -295,6 +300,7 @@ ps_send_results( void *arg )
 	char *fstr = NULL;
 	char **pbattrs = NULL;
 	int conn_acq_flag = 0;
+	Slapi_Connection *conn = NULL;
     
     g_incr_active_threadcnt();
 
@@ -418,22 +424,22 @@ ps_send_results( void *arg )
 	slapi_pblock_set(ps->ps_pblock, SLAPI_SEARCH_FILTER, NULL );
 	slapi_filter_free(filter, 1);
 
+    conn = ps->ps_pblock->pb_conn; /* save to release later - connection_remove_operation_ext will NULL the pb_conn */
     /* Clean up the connection structure */
-    PR_Lock( ps->ps_pblock->pb_conn->c_mutex );
+    PR_Lock( conn->c_mutex );
 
 	slapi_log_error(SLAPI_LOG_CONNS, "Persistent Search",
 					"conn=%" NSPRIu64 " op=%d Releasing the connection and operation\n",
-					(long long unsigned int)ps->ps_pblock->pb_conn->c_connid, ps->ps_pblock->pb_op->o_opid);
+					(long long unsigned int)conn->c_connid, ps->ps_pblock->pb_op->o_opid);
     /* Delete this op from the connection's list */
-    connection_remove_operation( ps->ps_pblock->pb_conn, ps->ps_pblock->pb_op );
-    operation_free(&(ps->ps_pblock->pb_op),ps->ps_pblock->pb_conn);
-    ps->ps_pblock->pb_op=NULL;
+    connection_remove_operation_ext( ps->ps_pblock, conn, ps->ps_pblock->pb_op );
 
     /* Decrement the connection refcnt */
     if (conn_acq_flag == 0) { /* we acquired it, so release it */
-	connection_release_nolock (ps->ps_pblock->pb_conn);
+	connection_release_nolock (conn);
     }
-    PR_Unlock( ps->ps_pblock->pb_conn->c_mutex );
+    PR_Unlock( conn->c_mutex );
+    conn = NULL;
 
     PR_DestroyLock ( ps->ps_lock );
     ps->ps_lock = NULL;
@@ -466,7 +472,6 @@ psearch_alloc()
 	slapi_ch_free((void **)&ps);
 	return( NULL );
     }
-    ps->ps_tid = (PRThread *) NULL;
     ps->ps_complete = 0;
     ps->ps_eq_head = ps->ps_eq_tail = (PSEQNode *) NULL;
     ps->ps_lasttime = (time_t) 0L;
