@@ -447,7 +447,6 @@ pw_rever_decode(char *cipher, char **plain, const char * attr_name)
 		{
 			if (slapi_attr_types_equivalent(L_attr, attr_name))
 			{
-				typedef int (*CMPFP)(char *, char *);
 				typedef char * (*ENCFP)(char *);
 
 				pwsp =  (struct pw_scheme *) slapi_ch_calloc (1, sizeof(struct pw_scheme));
@@ -523,7 +522,6 @@ pw_rever_encode(Slapi_Value **vals, char * attr_name)
 		{
 			if (slapi_attr_types_equivalent(L_attr, attr_name))
 			{
-				typedef int (*CMPFP)(char *, char *);
 				typedef char * (*ENCFP)(char *);
 
 				pwsp =  (struct pw_scheme *) slapi_ch_calloc (1, sizeof(struct pw_scheme));
@@ -585,6 +583,7 @@ int
 update_pw_info ( Slapi_PBlock *pb , char *old_pw)
 {
 	Slapi_Operation *operation = NULL;
+	Slapi_Entry *e = NULL;
 	Slapi_DN *sdn = NULL;
 	Slapi_Mods	smods;
 	passwdPolicy *pwpolicy = NULL;
@@ -594,9 +593,10 @@ update_pw_info ( Slapi_PBlock *pb , char *old_pw)
 	char *timestr;
 	int internal_op = 0;
 
-	slapi_pblock_get(pb, SLAPI_OPERATION, &operation);
+	slapi_pblock_get( pb, SLAPI_OPERATION, &operation);
 	slapi_pblock_get( pb, SLAPI_TARGET_SDN, &sdn );
 	slapi_pblock_get( pb, SLAPI_REQUESTOR_NDN, &bind_dn);
+	slapi_pblock_get( pb, SLAPI_ENTRY_PRE_OP, &e);
 	internal_op = slapi_operation_is_flag_set(operation, SLAPI_OP_FLAG_INTERNAL);
 	target_dn = slapi_sdn_get_ndn(sdn);
 	pwpolicy = new_passwdPolicy(pb, target_dn);
@@ -633,8 +633,10 @@ update_pw_info ( Slapi_PBlock *pb , char *old_pw)
 		set_retry_cnt_mods (pb, &smods, 0 );
 	}
 
-	/* Clear the passwordgraceusertime from the user entry */
-	slapi_mods_add_string(&smods, LDAP_MOD_REPLACE, "passwordgraceusertime", "0");
+	if(!slapi_entry_attr_hasvalue(e,"passwordgraceusertime", "0")){
+		/* Clear the passwordgraceusertime from the user entry */
+		slapi_mods_add_string(&smods, LDAP_MOD_REPLACE, "passwordgraceusertime", "0");
+	}
 
 	/*
 	 * If the password is reset by a different user, mark it the first time logon.  If this is an internal
@@ -809,6 +811,32 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 	slapi_pblock_get(pb, SLAPI_OPERATION, &operation);
 	internal_op = slapi_operation_is_flag_set(operation, SLAPI_OP_FLAG_INTERNAL);
 
+	/*
+	 * Check if password is already hashed and reject if so.  We need to
+	 * allow the root DN, password admins, and replicated ops to send
+	 * pre-hashed passwords. We also check for a connection object
+	 * when processing an internal operation to handle a special
+	 * case for the password modify extended operation.
+	 */
+	for ( i = 0; vals[ i ] != NULL; ++i ){
+		if (slapi_is_encoded((char *)slapi_value_get_string(vals[i]))) {
+			if ((!is_replication && ((internal_op && pb->pb_conn && !slapi_dn_isroot(pb->pb_conn->c_dn)) ||
+				(!internal_op && !pw_is_pwp_admin(pb, pwpolicy))))) {
+				PR_snprintf( errormsg, BUFSIZ,
+					"invalid password syntax - passwords with storage scheme are not allowed");
+				if ( pwresponse_req == 1 ) {
+					slapi_pwpolicy_make_response_control ( pb, -1, -1,
+							LDAP_PWPOLICY_INVALIDPWDSYNTAX );
+				}
+				pw_send_ldap_result ( pb, LDAP_CONSTRAINT_VIOLATION, NULL, errormsg, 0, NULL );
+				return( 1 );
+			} else {
+				/* We want to skip syntax checking since this is a pre-hashed password */
+				return( 0 );
+			}
+		}
+	}
+
 	if ( pwpolicy->pw_syntax == 1 ) {
 		for ( i = 0; vals[ i ] != NULL; ++i ) {
 			int syntax_violation = 0;
@@ -822,29 +850,6 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 			int max_repeated = 0;
 			int num_categories = 0;
 
-			/* Check if password is already hashed and reject if so.  We
-			 * We need to allow the root DN and replicated ops to send
-			 * pre-hashed passwords. We also check for a connection object
-			 * when processing an internal operation to handle a special
-			 * case for the password modify extended operation. */
-			if (slapi_is_encoded((char *)slapi_value_get_string(vals[i]))) {
-				if ((!is_replication && ((internal_op && pb->pb_conn && !slapi_dn_isroot(pb->pb_conn->c_dn)) ||
-					(!internal_op && !pw_is_pwp_admin(pb, pwpolicy))))) {
-					PR_snprintf( errormsg, BUFSIZ,
-						"invalid password syntax - passwords with storage scheme are not allowed");
-					if ( pwresponse_req == 1 ) {
-						slapi_pwpolicy_make_response_control ( pb, -1, -1,
-								LDAP_PWPOLICY_INVALIDPWDSYNTAX );
-					}
-					pw_send_ldap_result ( pb, LDAP_CONSTRAINT_VIOLATION, NULL, errormsg, 0, NULL );
-					return( 1 );
-				} else {
-					/* We want to skip syntax checking since this is a pre-hashed
-					 * password from replication or the root DN. */
-					return( 0 );
-				}
-			}
-
 			/* check for the minimum password length */
 			if ( pwpolicy->pw_minlength >
 				ldap_utf8characters((char *)slapi_value_get_string( vals[i] )) )
@@ -1061,8 +1066,9 @@ check_pw_syntax_ext ( Slapi_PBlock *pb, const Slapi_DN *sdn, Slapi_Value **vals,
 
 	if ( mod_op ) {
 		/* free e only when called by modify operation */
-		slapi_entry_free( e ); 
+		slapi_entry_free( e );
 	}
+
 	return 0; 	/* success */
 
 }
@@ -2684,3 +2690,52 @@ slapi_get_first_clear_text_pw(Slapi_Entry *entry)
 	slapi_rwlock_unlock(extp->pw_entry_lock);
 	return slapi_ch_strdup(password_str); /* slapi_ch_strdup(NULL) is okay */
 }
+
+void
+slapi_free_pw_scheme(struct pw_scheme *pwsp)
+{
+    free_pw_scheme(pwsp);
+}
+
+struct pw_scheme *
+slapi_pw_val2scheme( char *val, char **valpwdp, int first_is_default )
+{
+    return pw_val2scheme( val, valpwdp, first_is_default );
+}
+
+int
+slapi_pw_cmp(struct pw_scheme *pass_scheme, char *clear_pw, char *encoded_pw)
+{
+    return ((*(pass_scheme->pws_cmp))( clear_pw, encoded_pw ));
+}
+
+char *
+slapi_pw_get_scheme_name(struct pw_scheme *pass_scheme)
+{
+    return pass_scheme->pws_name;
+}
+
+int
+pw_get_ext_size(Slapi_Entry *entry, size_t *size)
+{
+    Slapi_Value **pw_entry_values;
+
+    if (NULL == size) {
+        return LDAP_PARAM_ERROR;
+    }
+    *size = 0;
+    if (NULL == entry->e_extension) {
+        return LDAP_SUCCESS;
+    }
+    *size += sizeof(struct slapi_pw_entry_ext);
+    *size += sizeof(Slapi_RWLock);
+    if (LDAP_SUCCESS == slapi_pw_get_entry_ext(entry, &pw_entry_values)) {
+        Slapi_Value *cvalue;
+        int idx = valuearray_first_value(pw_entry_values, &cvalue);
+        while (idx >= 0) {
+            *size += value_size(cvalue);
+            idx = valuearray_next_value(pw_entry_values, idx, &cvalue);
+        }
+    }
+    return LDAP_SUCCESS;
+}
