@@ -73,10 +73,16 @@ static void log_result( Slapi_PBlock *pb, Operation *op, int err,
 						ber_tag_t tag, int nentries );
 static void log_entry( Operation *op, Slapi_Entry *e );
 static void log_referral( Operation *op );
+static int process_read_entry_controls(Slapi_PBlock *pb, char *oid);
+static struct berval * encode_read_entry(Slapi_PBlock *pb, Slapi_Entry *e,
+                                         char **attrs, int alluserattrs,
+                                         int some_named_attrs);
+static char * op_to_string(int tag);
 
 #define _LDAP_SEND_RESULT	0
 #define _LDAP_SEND_REFERRAL	1
 #define _LDAP_SEND_ENTRY	2
+#define _LDAP_SEND_INTERMED	4
 
 #define SLAPI_SEND_VATTR_FLAG_REALONLY          0x01
 #define SLAPI_SEND_VATTR_FLAG_VIRTUALONLY       0x02
@@ -218,6 +224,79 @@ send_ldap_result(
        send_ldap_result_ext(pb, err, matched, text, nentries, urls, NULL);
 }
 
+int send_ldap_intermediate( Slapi_PBlock *pb,  LDAPControl **ectrls,
+	char *responseName, struct berval *responseValue)
+{
+	ber_tag_t	tag;
+	BerElement	*ber;
+	Slapi_Connection *connection;
+	Slapi_Operation	*operation;
+	int 		rc = 0;
+	int		logit = 0;
+			
+	LDAPDebug( LDAP_DEBUG_TRACE, "=> send_ldap_intermediate\n", 0, 0, 0 );
+	slapi_pblock_get (pb, SLAPI_OPERATION, &operation);
+	slapi_pblock_get (pb, SLAPI_CONNECTION, &connection);
+
+	if (operation->o_status == SLAPI_OP_STATUS_RESULT_SENT) {
+		return(rc); /* result already sent */
+	}
+	tag = LDAP_RES_INTERMEDIATE;
+	if ( (ber = der_alloc()) == NULL ) {
+		LDAPDebug( LDAP_DEBUG_ANY, "ber_alloc failed\n", 0, 0, 0 );
+		goto log_and_return;
+	}
+	/* add the intermediate message */
+	rc = ber_printf( ber, "{it{", operation->o_msgid, tag);
+	/* print responsename */
+	rc = ber_printf ( ber, "ts", LDAP_TAG_IM_RES_OID, responseName);
+	/* print responsevalue */
+	rc = ber_printf ( ber, "tO", LDAP_TAG_IM_RES_VALUE, responseValue);
+
+
+
+	if ( rc != LBER_ERROR ) {
+		rc = ber_printf( ber, "}" ); /* one more } to come */
+	}
+
+	if ( ectrls != NULL 
+	    && connection->c_ldapversion >= LDAP_VERSION3
+	    && write_controls( ber, ectrls ) != 0 ) {
+		rc = (int)LBER_ERROR;
+	}
+
+	if ( rc != LBER_ERROR ) {	/* end the LDAPMessage sequence */
+		rc = ber_put_seq( ber );
+	}
+	if ( rc == LBER_ERROR ) {
+		LDAPDebug( LDAP_DEBUG_ANY, "ber_printf failed\n", 0, 0, 0 );
+		ber_free( ber, 1 /* freebuf */ );
+		goto log_and_return;
+	}
+
+	/* write only one pdu at a time - wait til it's our turn */
+	if ( flush_ber( pb, connection, operation, ber, _LDAP_SEND_INTERMED ) == 0 ) {
+		logit = 1;
+	}
+log_and_return:
+	/* operation->o_status = SLAPI_OP_STATUS_RESULT_SENT;
+	 * there could be multiple intermediate messages on
+	 * the same connection, unlike in send_result do not
+	 * set o_status
+	 */
+
+	if ( logit && operation_is_flag_set( operation,
+	    OP_FLAG_ACTION_LOG_ACCESS )) {
+		log_result( pb, operation, rc, tag, 0 );
+	}
+
+	LDAPDebug( LDAP_DEBUG_TRACE, "<= send_ldap_intermediate\n", 0, 0, 0 );
+	if (rc == LBER_ERROR) {
+		return(1);
+	} else {
+		return(0);
+	}
+}
 
 static int
 check_and_send_extended_result(Slapi_PBlock *pb, ber_tag_t tag, BerElement *ber)
@@ -484,7 +563,7 @@ send_ldap_result_ext(
 		/*
 		 * there are v3 referrals to add to the result
 		 */
-	        /* count the referral */
+		/* count the referral */
 		if (! config_check_referral_mode())
 		    slapi_counter_increment(g_get_global_snmp_vars()->ops_tbl.dsReferrals);
 		rc = ber_printf( ber, "{it{esst{s", operation->o_msgid, tag, err,
@@ -521,6 +600,19 @@ send_ldap_result_ext(
 			rc = ber_printf( ber, "}" ); /* one more } to come */
 		}
 	}
+	if(err == LDAP_SUCCESS){
+		/*
+		 * Process the Read Entry Controls (if any)
+		 */
+		if(process_read_entry_controls(pb, LDAP_CONTROL_PRE_READ_ENTRY)){
+			err = LDAP_UNAVAILABLE_CRITICAL_EXTENSION;
+			goto log_and_return;
+		}
+		if(process_read_entry_controls(pb, LDAP_CONTROL_POST_READ_ENTRY)){
+			err = LDAP_UNAVAILABLE_CRITICAL_EXTENSION;
+			goto log_and_return;
+		}
+	}
 	if ( operation->o_results.result_controls != NULL 
 	    && conn->c_ldapversion >= LDAP_VERSION3
 	    && write_controls( ber, operation->o_results.result_controls ) != 0 ) {
@@ -550,14 +642,177 @@ send_ldap_result_ext(
 log_and_return:
 	operation->o_status = SLAPI_OP_STATUS_RESULT_SENT;	/* in case this has not yet been set */
 
-	if ( logit && operation_is_flag_set( operation,
-	    OP_FLAG_ACTION_LOG_ACCESS )) {
+	if ( logit && (operation_is_flag_set( operation, OP_FLAG_ACTION_LOG_ACCESS ) ||
+	               (internal_op && config_get_plugin_logging() )))
+	{
 		log_result( pb, operation, err, tag, nentries );
 	}
 
 	LDAPDebug( LDAP_DEBUG_TRACE, "<= send_ldap_result\n", 0, 0, 0 );
 }
 
+/*
+ * For RFC 4527 - Read Entry Controls
+ *
+ * If this is the correct operation for the control, then retrieve the
+ * requested attributes.  Then start building the ber-encoded string
+ * value of the entry.  We also need to check access control for the
+ * requested attributes.  Then an octet string containing a BER-encoded
+ * SearchResultEntry is added to the response control.
+ */
+static int
+process_read_entry_controls(Slapi_PBlock *pb, char *oid)
+{
+    struct berval *req_value = NULL;
+    struct berval *res_value = NULL;
+    LDAPControl **req_ctls = NULL;
+    Slapi_Entry *e = NULL;
+    char **attrs = NULL;
+    int attr_count = 0;
+    int iscritical = 0;
+    int all_attrs = 0;
+    int no_attrs = 0;
+    int rc = 0;
+
+    slapi_pblock_get(pb, SLAPI_REQCONTROLS, &req_ctls);
+
+    /*
+     * Check for the PRE Read Entry Control, and return the pre-modified entry
+     */
+    if (slapi_control_present(req_ctls, oid, &req_value, &iscritical))
+    {
+        BerElement *req_ber = NULL;
+        Operation *op = pb->pb_op;
+
+        if(strcmp(oid,LDAP_CONTROL_PRE_READ_ENTRY) == 0){
+            /* first verify this is the correct operation for a pre-read entry control */
+            if(op->o_tag == LDAP_REQ_MODIFY || op->o_tag == LDAP_REQ_DELETE ||
+               op->o_tag == LDAP_REQ_MODDN){
+                slapi_pblock_get(pb, SLAPI_ENTRY_PRE_OP, &e);
+            } else {
+                /* Ok, read control not used for this type of operation */
+            	LDAPDebug( LDAP_DEBUG_ANY, "process_read_entry_controls: Read Entry Controls "
+            	        "can not be used for a %s operation.\n", op_to_string(op->o_tag), 0, 0);
+                rc = -1;
+                goto done;
+            }
+        } else {
+            /* first verify this is the correct operation for a post-read entry control */
+            if(op->o_tag == LDAP_REQ_MODIFY || op->o_tag == LDAP_REQ_ADD ||
+               op->o_tag == LDAP_REQ_MODDN){
+                slapi_pblock_get(pb, SLAPI_ENTRY_POST_OP, &e);
+            } else {
+                /* Ok, read control not used for this type of operation */
+            	LDAPDebug( LDAP_DEBUG_ANY, "process_read_entry_controls: Read Entry Controls "
+                        "can not be used for a %s operation.\n", op_to_string(op->o_tag), 0, 0);
+                rc = -1;
+                goto done;
+            }
+        }
+        if(e == NULL){
+            LDAPDebug( LDAP_DEBUG_ANY, "process_read_entry_controls: unable to retrieve entry\n",0,0,0);
+            rc = -1;
+            goto done;
+        }
+
+#if !defined(DISABLE_ACL_CHECK)
+        /* even though we can modify the entry, that doesn't mean we can read it */
+        if ( plugin_call_acl_plugin (pb, e, attrs, NULL, SLAPI_ACL_READ,
+            ACLPLUGIN_ACCESS_READ_ON_ENTRY, NULL ) != LDAP_SUCCESS )
+        {
+            LDAPDebug( LDAP_DEBUG_ACL, "process_read_entry_controls: access to entry not allowed (%s)\n",
+                slapi_entry_get_dn_const(e), 0, 0 );
+            rc = -1;
+            goto done;
+        }
+#endif
+        /*
+         *  Check the ctl_value for any requested attributes
+         */
+        if( req_value && req_value->bv_len != 0 && req_value->bv_val){
+            if ((req_ber = ber_init(req_value)) == NULL){
+                rc = -1;
+                goto free;
+            }
+            if (ber_scanf(req_ber, "{") == LBER_ERROR){
+                rc = -1;
+                goto free;
+            }
+            /* process the attributes */
+            while(1){
+                char *payload = NULL;
+
+                if (ber_get_stringa(req_ber, &payload) != LBER_ERROR){
+                    if(strcmp(payload, LDAP_ALL_USER_ATTRS) == 0){
+                        all_attrs = 1;
+                        slapi_ch_free_string(&payload);
+                    } else if(strcmp(payload, LDAP_NO_ATTRS) == 0){
+                        no_attrs = 1;
+                        slapi_ch_free_string(&payload);
+                    } else {
+                        charray_add(&attrs, payload);
+                        attr_count++;
+                    }
+                } else {
+                    /* we're done */
+                    break;
+                }
+            }
+            if(no_attrs && (all_attrs || attr_count)){
+                /* Can't have both no attrs and some attributes */
+                LDAPDebug( LDAP_DEBUG_ANY, "process_read_entry_controls: Error, both no attributes \"1.1\" and "
+                    "specific attributes were requeseted.\n", 0, 0, 0 );
+                rc = -1;
+                goto free;
+            }
+
+            if (ber_scanf(req_ber, "}") == LBER_ERROR){
+                rc = -1;
+                goto free;
+            }
+        } else {
+            /* this is a problem, malformed request control value */
+            LDAPDebug( LDAP_DEBUG_ANY, "process_read_entry_controls: invalid control value.\n",0,0,0);
+            rc = -1;
+            goto free;
+        }
+
+        /*
+         * Get the ber encoded string, and add it to the response control
+         */
+        res_value = encode_read_entry(pb, e, attrs, all_attrs, attr_count);
+        if(res_value && res_value->bv_len > 0){
+            LDAPControl	new_ctrl = {0};
+
+            new_ctrl.ldctl_oid = oid;
+            new_ctrl.ldctl_value = *res_value;
+            new_ctrl.ldctl_iscritical = iscritical;
+            slapi_pblock_set( pb, SLAPI_ADD_RESCONTROL, &new_ctrl );
+            ber_bvfree(res_value);
+        } else {
+            /* failed to encode the result entry */
+            LDAPDebug( LDAP_DEBUG_ANY, "Failed to process READ ENTRY Control (%s), error encoding result entry\n",
+                        oid, 0, 0 );
+            rc = -1;
+        }
+
+free:
+        if (NULL != req_ber){
+            ber_free(req_ber, 1);
+        }
+        if(rc != 0){
+            /* log an error */
+            LDAPDebug( LDAP_DEBUG_ANY, "Failed to process READ ENTRY Control (%s) ber decoding error\n",
+                        oid, 0, 0 );
+        }
+    }
+done:
+    if(iscritical){
+        return rc;
+    } else {
+        return 0;
+    }
+}
 
 void
 send_nobackend_ldap_result( Slapi_PBlock *pb )
@@ -747,17 +1002,17 @@ encode_attr_2(
 	attrs[0] = (char*)attribute_type;
 
 #if !defined(DISABLE_ACL_CHECK)
-	if ( plugin_call_acl_plugin (pb, e, attrs, NULL, SLAPI_ACL_READ, 
-				     ACLPLUGIN_ACCESS_READ_ON_ATTR, NULL ) != LDAP_SUCCESS ) {
+	if (plugin_call_acl_plugin(pb, e, attrs, NULL, SLAPI_ACL_READ, 
+	                           ACLPLUGIN_ACCESS_READ_ON_ATTR, NULL) != LDAP_SUCCESS) {
 		return( 0 );
 	}
 #endif
 
-	if ( ber_printf( ber, "{s[", returned_type ) == -1 ) {
+	if (ber_printf(ber, "{s[", returned_type?returned_type:attribute_type) == -1) {
 		LDAPDebug( LDAP_DEBUG_ANY, "ber_printf failed\n", 0, 0, 0 );
 		ber_free( ber, 1 );
-		send_ldap_result( pb, LDAP_OPERATIONS_ERROR, NULL,
-		    "ber_printf type", 0, NULL );
+		send_ldap_result(pb, LDAP_OPERATIONS_ERROR, NULL,
+		                 "ber_printf type", 0, NULL);
 		return( -1 );
 	}
 
@@ -891,7 +1146,7 @@ static const char *idds_map_attrt_v3(
 
 /* Helper functions */
 
-static int send_all_attrs(Slapi_Entry *e,char **attrs,Slapi_Operation *op,Slapi_PBlock *pb,BerElement *ber,int attrsonly,int ldapversion,int *dontsendattr, int real_attrs_only, int some_named_attrs)
+static int send_all_attrs(Slapi_Entry *e,char **attrs,Slapi_Operation *op,Slapi_PBlock *pb,BerElement *ber,int attrsonly,int ldapversion, int real_attrs_only, int some_named_attrs)
 {
 	int i = 0;
 	int rc = 0;
@@ -1004,10 +1259,11 @@ static int send_all_attrs(Slapi_Entry *e,char **attrs,Slapi_Operation *op,Slapi_
 					&attr_free_flags, 
 					&item_count
 					);
-			if (0 == rc && item_count > 0) {
+			if ((0 == rc) && (item_count > 0)) {
 
 				for(iter=0; iter<item_count; iter++)
 				{
+					int skipit;
 					if ( rc != 0 ) {
 						/* we hit an error - we need to free all of the stuff allocated by
 						   slapi_vattr_namespace_values_get_sp */
@@ -1019,31 +1275,28 @@ static int send_all_attrs(Slapi_Entry *e,char **attrs,Slapi_Operation *op,Slapi_
 						name_to_return = actual_type_name[iter]; 
 					}
 
-					/*
-					* The dontsendattr array is used to track whether attributes
-					* that were explicitly requested by the client have been
-					* returned. Check here to see if the attribute we just
-					* arranged to send back was explicitly requested, and if so,
-					* set its dontsendattr flag so the send_specific_attrs()
-					* function does not return it a second time.
-					*/					
-					for ( i = 0; attrs != NULL && attrs[i] != NULL; i++ ) {
-						if ( !dontsendattr[i] && slapi_attr_type_cmp( current_type_name, attrs[i], SLAPI_TYPE_CMP_SUBTYPE ) == 0 ) {
-						/* Client is also asking for an attr which is in '*', zap it. */
-							dontsendattr[i]= 1;
+					/* If current_type_name is in attrs, we could rely on send_specific_attrs. */
+					skipit = 0;
+					for (i = 0; attrs && attrs[i]; i++) {
+						if (slapi_attr_type_cmp(current_type_name, attrs[i], SLAPI_TYPE_CMP_SUBTYPE) == 0) {
+							skipit = 1;
+							break;
 						}
 					}
 
-					rc = encode_attr_2( pb, ber, e, values[iter], attrsonly, current_type_name, name_to_return );
+					if (!skipit) {
+						rc = encode_attr_2(pb, ber, e, values[iter], attrsonly, 
+						                   current_type_name, name_to_return );
 
-					if (rewrite_rfc1274 != 0) {
-						v2name = idds_map_attrt_v3(current_type_name);
-						if (v2name != NULL) {
-							/* also return values with RFC1274 attr name */
-							rc = encode_attr_2(pb, ber, e, values[iter], 
-								   attrsonly, 
-								   current_type_name, 
-								   v2name);
+						if (rewrite_rfc1274 != 0) {
+							v2name = idds_map_attrt_v3(current_type_name);
+							if (v2name != NULL) {
+								/* also return values with RFC1274 attr name */
+								rc = encode_attr_2(pb, ber, e, values[iter], 
+								                   attrsonly, 
+								                   current_type_name, 
+								                   v2name);
+							}
 						}
 					}
 
@@ -1074,137 +1327,143 @@ exit:
 	return rc;
 }
 
-int send_specific_attrs(Slapi_Entry *e,char **attrs,Slapi_Operation *op,Slapi_PBlock *pb,BerElement *ber,int attrsonly,int ldapversion,int *dontsendattr, int real_attrs_only)
+/*
+ * attrs need to expand including the subtypes found in the entry
+ * e.g., if "sn" is no the attrs and 'e' has sn, sn;en, and sn;fr,
+ * attrs should have sn, sn;en, and sn;fr, as well.
+ */
+int
+send_specific_attrs(Slapi_Entry *e, char **attrs, Slapi_Operation *op,
+                    Slapi_PBlock *pb, BerElement *ber, int attrsonly,
+                    int ldapversion, int real_attrs_only)
 {
-	int i,j = 0;
+	int i = 0;
 	int rc = 0;
 	int vattr_flags = 0;
 	vattr_context *ctx;
+	char **attrs_ext = NULL;
+	char **my_searchattrs = NULL;
 
-	if(real_attrs_only == SLAPI_SEND_VATTR_FLAG_REALONLY)
+	if (real_attrs_only == SLAPI_SEND_VATTR_FLAG_REALONLY) {
 		vattr_flags = SLAPI_REALATTRS_ONLY;
-	else
-	{
+	} else {
 		vattr_flags = SLAPI_VIRTUALATTRS_REQUEST_POINTERS;
 		if(real_attrs_only == SLAPI_SEND_VATTR_FLAG_VIRTUALONLY)
 			vattr_flags |= SLAPI_VIRTUALATTRS_ONLY;
 	}
+
+	/* Create a copy of attrs with no duplicates */
+	if (attrs) {
+		for (i = 0; attrs[i]; i++) {
+			if (!charray_inlist(attrs_ext, attrs[i])) {
+				slapi_ch_array_add(&attrs_ext, slapi_ch_strdup(attrs[i]));
+				slapi_ch_array_add(&my_searchattrs, slapi_ch_strdup(op->o_searchattrs[i]));
+			}
+		}
+	}
+	if (attrs_ext) {
+		attrs = attrs_ext;
+	}
 	
-	for ( i = 0; attrs != NULL && attrs[i] != NULL; i++ )
-	{
+	for ( i = 0; attrs && attrs[i] != NULL; i++ ) {
 		char *current_type_name = attrs[i];
-        if(!dontsendattr[i]) {
-			Slapi_ValueSet **values = NULL;
-			int attr_free_flags = 0;
-			char *name_to_return = NULL;
-			char **actual_type_name= NULL;
-			int *type_name_disposition = 0;
-			int item_count = 0;
-			int iter = 0;
-			Slapi_DN *namespace_dn;
-			Slapi_Backend *backend=0;
+		Slapi_ValueSet **values = NULL;
+		int attr_free_flags = 0;
+		char *name_to_return = NULL;
+		char **actual_type_name= NULL;
+		int *type_name_disposition = 0;
+		int item_count = 0;
+		int iter = 0;
+		Slapi_DN *namespace_dn;
+		Slapi_Backend *backend=0;
 
-    		/*
-    		 * Here we call the computed attribute code to see whether
-    		 * the requested attribute is to be computed. 
-    		 * The subroutine compute_attribute calls encode_attr on our behalf, in order
-    		 * to avoid the inefficiency of returning a complex structure
-    		 * which we'd have to free
-    		 */
-    		rc = compute_attribute(attrs[i],pb,ber,e,attrsonly,op->o_searchattrs[i]);
-    		if (0 == rc) {
-    			continue; /* Means this was a computed attr and we prcessed it OK. */
-    		}
-    		if (-1 != rc) {
-    			/* Means that some error happened */
-    			return rc;
-    		}
-    		else {
-    			rc = 0; /* Means that we just didn't recognize this as a computed attr */
-    		}
+		/*
+		 * Here we call the computed attribute code to see whether
+		 * the requested attribute is to be computed. 
+		 * The subroutine compute_attribute calls encode_attr on our behalf, in order
+		 * to avoid the inefficiency of returning a complex structure
+		 * which we'd have to free
+		 */
+		rc = compute_attribute(attrs[i], pb, ber, e, attrsonly, my_searchattrs[i]);
+		if (0 == rc) {
+			continue; /* Means this was a computed attr and we prcessed it OK. */
+		}
+		if (-1 != rc) {
+			/* Means that some error happened */
+			return rc;
+		}
+		else {
+			rc = 0; /* Means that we just didn't recognize this as a computed attr */
+		}
 
-			/* get the namespace dn */
-			slapi_pblock_get( pb, SLAPI_BACKEND, (void *)&backend);
-			namespace_dn = (Slapi_DN*)slapi_be_getsuffix(backend, 0);
+		/* get the namespace dn */
+		slapi_pblock_get( pb, SLAPI_BACKEND, (void *)&backend);
+		namespace_dn = (Slapi_DN*)slapi_be_getsuffix(backend, 0);
 
-			/* Get the attribute value from the vattr service */
-			/* ctx will be freed by attr_context_ungrok() */
-			ctx = vattr_context_new ( pb );
-			rc = slapi_vattr_namespace_values_get_sp(
-					ctx,
-					e,
-					namespace_dn,
-					current_type_name,
-					&values,
-					&type_name_disposition,
-					&actual_type_name,
-					vattr_flags,
-					&attr_free_flags, 
-					&item_count
-					);
-			if (0 == rc && item_count > 0) {
-
-				for(iter=0; iter<item_count; iter++)
-				{
-					if ( rc != 0 ) {
-						/* we hit an error - we need to free all of the stuff allocated by
-						   slapi_vattr_namespace_values_get_sp */
-						slapi_vattr_values_free(&(values[iter]), &(actual_type_name[iter]), attr_free_flags);
+		/* Get the attribute value from the vattr service */
+		/* This call handles subtype, as well.
+		 * e.g., current_type_name: cn 
+		 * ==>
+		 * item_count: 5; actual_type_name: cn;sub0, ..., cn;sub4 */
+		/* ctx will be freed by attr_context_ungrok() */
+		ctx = vattr_context_new ( pb );
+		rc = slapi_vattr_namespace_values_get_sp(
+				ctx,
+				e,
+				namespace_dn,
+				current_type_name,
+				&values,
+				&type_name_disposition,
+				&actual_type_name,
+				vattr_flags,
+				&attr_free_flags, 
+				&item_count
+				);
+		if ((0 == rc) && (item_count > 0)) {
+			for (iter = 0; iter < item_count; iter++) {
+				if ( rc != 0 ) {
+					/* we hit an error - we need to free all of the stuff allocated by
+					   slapi_vattr_namespace_values_get_sp */
+					slapi_vattr_values_free(&(values[iter]), &(actual_type_name[iter]), attr_free_flags);
+					continue;
+				}
+				if (SLAPI_VIRTUALATTRS_TYPE_NAME_MATCHED_SUBTYPE == type_name_disposition[iter]) {
+					name_to_return = actual_type_name[iter]; 
+					if ((iter > 0) && charray_inlist(attrs, name_to_return)) {
+						/* subtype retrieved by slapi_vattr_namespace_values_get_sp is
+						 * included in the attr list.  Skip the dup. */
 						continue;
 					}
-					if (SLAPI_VIRTUALATTRS_TYPE_NAME_MATCHED_SUBTYPE == type_name_disposition[iter]) {
-						name_to_return = actual_type_name[iter]; 
-					} else {
-						name_to_return = op->o_searchattrs[i];
-					}
-
-					/*
-					 * The client may have specified a list of attributes
-					 * with duplicates, 'cn cn cn'.
-					 * We need to determine which of any duplicates take precedence
-					 * For subtypes, the attribute which is most generic should be
-					 * returned (since it will also trigger the return of the less
-					 * generic attribute subtypes.
-					 */
-					for ( j = i+1; attrs[j] != NULL && dontsendattr[i]==0; j++ )
-					{
-						if ( !dontsendattr[j] && slapi_attr_type_cmp( attrs[j], actual_type_name[iter], SLAPI_TYPE_CMP_SUBTYPE ) == 0 )
-						{
-							/* discover which is the more generic attribute and cancel the other*/
-							int attrbase = slapi_attr_type_cmp( attrs[j], current_type_name, SLAPI_TYPE_CMP_EXACT );
-
-							if(attrbase >= 0)
-								dontsendattr[j]= 1;
-							else
-								dontsendattr[i]= 1; /* the current value is superceeded later */
-						}
-					}
-
-					/* we may have just cancelled ourselves so check */
-					if(!dontsendattr[i])
-						rc = encode_attr_2( pb, ber, e, values[iter], attrsonly, current_type_name, name_to_return );
-					
-					slapi_vattr_values_free(&(values[iter]), &(actual_type_name[iter]), attr_free_flags);
+				} else {
+					name_to_return = my_searchattrs[i];
 				}
 
-				slapi_ch_free((void**)&actual_type_name);
-				slapi_ch_free((void**)&type_name_disposition);
-				slapi_ch_free((void**)&values);
-				if ( rc != 0 ) {
-					goto exit;
-				}
-
-			} else {
-				/* if we got here, then either values is NULL or values contains no elements
-				   either way we can free it */
-				slapi_ch_free((void**)&values);
-				slapi_ch_free((void**)&actual_type_name);
-				slapi_ch_free((void**)&type_name_disposition);
-				rc = 0;
+				/* need to pass actual_type_name (e.g., sn;en to evaluate the ACL */
+				rc = encode_attr_2(pb, ber, e, values[iter], attrsonly, 
+				                   actual_type_name[iter], name_to_return);
+				
+				slapi_vattr_values_free(&(values[iter]), &(actual_type_name[iter]), attr_free_flags);
 			}
-        } 	
+
+			slapi_ch_free((void**)&actual_type_name);
+			slapi_ch_free((void**)&type_name_disposition);
+			slapi_ch_free((void**)&values);
+			if ( rc != 0 ) {
+				goto exit;
+			}
+
+		} else {
+			/* if we got here, then either values is NULL or values contains no elements
+			   either way we can free it */
+			slapi_ch_free((void**)&values);
+			slapi_ch_free((void**)&actual_type_name);
+			slapi_ch_free((void**)&type_name_disposition);
+			rc = 0;
+		}
 	}
 exit:
+	slapi_ch_array_free(attrs_ext);
+	slapi_ch_array_free(my_searchattrs);
 	return rc;
 
 }
@@ -1227,7 +1486,6 @@ send_ldap_search_entry_ext(
 	BerElement	*ber = NULL;
 	int		i, rc = 0, logit = 0;
 	int		alluserattrs, noattrs, some_named_attrs;
-	int *dontsendattr= NULL;
 	Slapi_Operation *operation;
 	int real_attrs_only = 0;
 	LDAPControl		**ctrlp = 0;
@@ -1239,7 +1497,7 @@ send_ldap_search_entry_ext(
 	slapi_pblock_get (pb, SLAPI_OPERATION, &operation);
 
 	LDAPDebug( LDAP_DEBUG_TRACE, "=> send_ldap_search_entry (%s)\n",
-	    e?slapi_entry_get_dn_const(e):"null", 0, 0 );
+	    e ? slapi_entry_get_dn_const(e) : "null", 0, 0 );
 
 	/* set current entry */
 	slapi_pblock_set(pb, SLAPI_SEARCH_ENTRY_ORIG, e);
@@ -1341,11 +1599,6 @@ send_ldap_search_entry_ext(
 			    "Accepting illegal other attributes specified with "
 			    "special \"1.1\" attribute\n", 0, 0, 0 );
 		}
-        /*
-         * We maintain a flag array so that we can remove requests
-         * for duplicate attributes.
-         */
-    	dontsendattr= (int*) slapi_ch_calloc( i+1, sizeof(int) );
 	}
 
 
@@ -1370,12 +1623,13 @@ send_ldap_search_entry_ext(
 
 	/* look through each attribute in the entry */
 	if ( alluserattrs ) {
-		rc = send_all_attrs(e,attrs,op,pb,ber,attrsonly,conn->c_ldapversion,dontsendattr, real_attrs_only, some_named_attrs);
+		rc = send_all_attrs(e, attrs, op, pb, ber, attrsonly, conn->c_ldapversion,
+		                    real_attrs_only, some_named_attrs);
 	}
 	
 	/* if the client explicitly specified a list of attributes look through each attribute requested */
-	if( (rc == 0) && (attrs!=NULL)) {
-		rc = send_specific_attrs(e,attrs,op,pb,ber,attrsonly,conn->c_ldapversion,dontsendattr,real_attrs_only);
+	if( (rc == 0) && (attrs!=NULL) && !noattrs) {
+		rc = send_specific_attrs(e,attrs,op,pb,ber,attrsonly,conn->c_ldapversion,real_attrs_only);
 	}
 
 	/* Append effective rights to the stream of attribute list */
@@ -1483,7 +1737,6 @@ cleanup:
 		ldap_controls_free(searchctrlp);
 	}
 	ber_free( ber, 1 );
-	slapi_ch_free( (void **) &dontsendattr );
 	LDAPDebug( LDAP_DEBUG_TRACE, "<= send_ldap_search_entry\n", 0, 0, 0 );
 
 	return( rc );
@@ -1514,6 +1767,8 @@ flush_ber(
 	case _LDAP_SEND_REFERRAL:
 		rc = plugin_call_plugins( pb, SLAPI_PLUGIN_PRE_REFERRAL_FN );
 		break;
+	case _LDAP_SEND_INTERMED:
+		break; /* not a plugin entry point */
 	}
 
 	if ( rc != 0 ) {
@@ -1583,6 +1838,8 @@ flush_ber(
 		slapi_counter_increment(g_get_global_snmp_vars()->ops_tbl.dsEntriesReturned);
 		plugin_call_plugins( pb, SLAPI_PLUGIN_POST_ENTRY_FN );
 		break;
+	case _LDAP_SEND_INTERMED:
+		break; /* not a plugin entry point */
 	}
 
 	return( rc );
@@ -1617,7 +1874,8 @@ struct slapi_note_map {
 
 static struct slapi_note_map notemap[] = {
 	{ SLAPI_OP_NOTE_UNINDEXED,   "U" },
-	{ SLAPI_OP_NOTE_SIMPLEPAGED, "P" }
+	{ SLAPI_OP_NOTE_SIMPLEPAGED, "P" },
+        { SLAPI_OP_NOTE_FULL_UNINDEXED, "A" }
 };
 
 #define SLAPI_NOTEMAP_COUNT  ( sizeof(notemap) / sizeof(struct slapi_note_map))
@@ -1676,33 +1934,30 @@ notes2str( unsigned int notes, char *buf, size_t buflen )
 #define ETIME_BUFSIZ 16         /* room for 99999999.999999 */
 
 static void
-log_result( Slapi_PBlock *pb, Operation *op, int err, ber_tag_t tag,
-	int nentries )
+log_result( Slapi_PBlock *pb, Operation *op, int err, ber_tag_t tag, int nentries )
 {
 	char	*notes_str, notes_buf[ 256 ];
 	int	internal_op;
 	CSN *operationcsn = NULL;
 	char csn_str[CSN_STRSIZE + 5];
-        char etime[ETIME_BUFSIZ];
+	char etime[ETIME_BUFSIZ];
 
 	internal_op = operation_is_flag_set( op, OP_FLAG_INTERNAL );
 
-        if ( (config_get_accesslog_level() & LDAP_DEBUG_TIMING) &&
-             (op->o_interval != (PRIntervalTime) 0) ) {
-            PRIntervalTime delta = PR_IntervalNow() - op->o_interval;
-            PR_snprintf(etime, ETIME_BUFSIZ, "%f", 
-                        (PRFloat64)delta/PR_TicksPerSecond());
-        } else {
-            PR_snprintf(etime, ETIME_BUFSIZ, "%ld", current_time() - op->o_time);
-        }
+	if ( (config_get_accesslog_level() & LDAP_DEBUG_TIMING) &&
+			(op->o_interval != (PRIntervalTime) 0) ) {
+		PRIntervalTime delta = PR_IntervalNow() - op->o_interval;
+		PR_snprintf(etime, ETIME_BUFSIZ, "%f", (PRFloat64)delta/PR_TicksPerSecond());
+	} else {
+		PR_snprintf(etime, ETIME_BUFSIZ, "%ld", current_time() - op->o_time);
+	}
 
 	if ( 0 == pb->pb_operation_notes ) {
 		notes_str = "";
 	} else {
 		notes_str = notes_buf;
 		*notes_buf = ' ';
-		notes2str( pb->pb_operation_notes, notes_buf + 1,
-		    sizeof( notes_buf ) - 1 );
+		notes2str( pb->pb_operation_notes, notes_buf + 1, sizeof( notes_buf ) - 1 );
 	} 
 
 	csn_str[0] = '\0';
@@ -1716,92 +1971,124 @@ log_result( Slapi_PBlock *pb, Operation *op, int err, ber_tag_t tag,
 		}
 	}
 
-        if (op->o_tag == LDAP_REQ_BIND && err == LDAP_SASL_BIND_IN_PROGRESS) {
-            /* 
-             * Not actually an error. 
-             * Make that clear in the log.
-             */
-			if ( !internal_op )
-			{
-				slapi_log_access( LDAP_DEBUG_STATS,
-								  "conn=%" NSPRIu64 " op=%d RESULT err=%d"
-								  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s"
-								  ", SASL bind in progress\n",
-								  (long long unsigned int)op->o_connid,
-								  op->o_opid,
-								  err, tag, nentries, 
-								  etime, 
-								  notes_str, csn_str );
-			}
-			else
-			{
-				slapi_log_access( LDAP_DEBUG_ARGS,
-								  "conn=%s op=%d RESULT err=%d"
-								  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s"
-								  ", SASL bind in progress\n",
-									LOG_INTERNAL_OP_CON_ID,
-									LOG_INTERNAL_OP_OP_ID,
-								  err, tag, nentries, 
-								  etime, 
-								  notes_str, csn_str );
-			}
-        } else if (op->o_tag == LDAP_REQ_BIND && err == LDAP_SUCCESS) {
-            char *dn = NULL;
+	if (op->o_tag == LDAP_REQ_BIND && err == LDAP_SASL_BIND_IN_PROGRESS) {
+		/*
+		 * Not actually an error.
+		 * Make that clear in the log.
+		 */
+		if ( !internal_op )
+		{
+			slapi_log_access( LDAP_DEBUG_STATS,
+							  "conn=%" NSPRIu64 " op=%d RESULT err=%d"
+							  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s"
+							  ", SASL bind in progress\n",
+							  (long long unsigned int)op->o_connid,
+							  op->o_opid,
+							  err, tag, nentries,
+							  etime,
+							  notes_str, csn_str );
+		}
+		else
+		{
+			slapi_log_access( LDAP_DEBUG_ARGS,
+							  "conn=%s op=%d RESULT err=%d"
+							  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s"
+							  ", SASL bind in progress\n",
+								LOG_INTERNAL_OP_CON_ID,
+								LOG_INTERNAL_OP_OP_ID,
+							  err, tag, nentries,
+							  etime,
+							  notes_str, csn_str );
+		}
+	} else if (op->o_tag == LDAP_REQ_BIND && err == LDAP_SUCCESS) {
+		char *dn = NULL;
 
-            /* 
-             * For methods other than simple, the dn in the bind request
-             * may be irrelevant. Log the actual authenticated dn.
-             */
-            slapi_pblock_get(pb, SLAPI_CONN_DN, &dn);
-			if ( !internal_op )
+		/*
+		 * For methods other than simple, the dn in the bind request
+		 * may be irrelevant. Log the actual authenticated dn.
+		 */
+		slapi_pblock_get(pb, SLAPI_CONN_DN, &dn);
+		if ( !internal_op )
+		{
+			slapi_log_access( LDAP_DEBUG_STATS,
+							  "conn=%" NSPRIu64 " op=%d RESULT err=%d"
+							  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s"
+							  " dn=\"%s\"\n",
+							  (long long unsigned int)op->o_connid,
+							  op->o_opid,
+							  err, tag, nentries,
+							  etime,
+							  notes_str, csn_str, dn ? dn : "");
+		}
+		else
+		{
+			slapi_log_access( LDAP_DEBUG_ARGS,
+							  "conn=%s op=%d RESULT err=%d"
+							  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s"
+							  " dn=\"%s\"\n",
+								LOG_INTERNAL_OP_CON_ID,
+								LOG_INTERNAL_OP_OP_ID,
+							  err, tag, nentries,
+							  etime,
+							  notes_str, csn_str, dn ? dn : "");
+		}
+		slapi_ch_free((void**)&dn);
+	} else {
+		if ( !internal_op )
+		{
+			slapi_log_access( LDAP_DEBUG_STATS,
+							  "conn=%" NSPRIu64 " op=%d RESULT err=%d"
+							  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s\n",
+							  (long long unsigned int)op->o_connid,
+							  op->o_opid,
+							  err, tag, nentries,
+							  etime,
+							  notes_str, csn_str );
+		}
+		else
+		{
+			int optype;
+			slapi_log_access( LDAP_DEBUG_ARGS,
+							  "conn=%s op=%d RESULT err=%d"
+							  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s\n",
+							  LOG_INTERNAL_OP_CON_ID,
+							  LOG_INTERNAL_OP_OP_ID,
+							  err, tag, nentries,
+							  etime,
+							  notes_str, csn_str );
+			/*
+			 *  If this is an unindexed search we should log it in the error log if
+			 *  we didn't log it in the access log.
+			 */
+			slapi_pblock_get( pb, SLAPI_OPERATION_TYPE, &optype );
+			if(optype == SLAPI_OPERATION_SEARCH && /* search, */
+			   strcmp(notes_str,"") &&  /* that's unindexed, */
+			   !(config_get_accesslog_level() & LDAP_DEBUG_ARGS)) /* and not logged in access log */
 			{
-				slapi_log_access( LDAP_DEBUG_STATS,
-								  "conn=%" NSPRIu64 " op=%d RESULT err=%d"
-								  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s"
-								  " dn=\"%s\"\n",
-								  (long long unsigned int)op->o_connid,
-								  op->o_opid,
-								  err, tag, nentries, 
-								  etime, 
-								  notes_str, csn_str, dn ? dn : "");
-			}
-			else
-			{
-				slapi_log_access( LDAP_DEBUG_ARGS,
-								  "conn=%s op=%d RESULT err=%d"
-								  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s"
-								  " dn=\"%s\"\n",
-									LOG_INTERNAL_OP_CON_ID,
-									LOG_INTERNAL_OP_OP_ID,	
-								  err, tag, nentries, 
-								  etime, 
-								  notes_str, csn_str, dn ? dn : "");
-			}
-            slapi_ch_free((void**)&dn);
-        } else {
-			if ( !internal_op )
-			{
-				slapi_log_access( LDAP_DEBUG_STATS,
-								  "conn=%" NSPRIu64 " op=%d RESULT err=%d"
-								  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s\n",
-								  (long long unsigned int)op->o_connid,
-								  op->o_opid,
-								  err, tag, nentries, 
-								  etime, 
-								  notes_str, csn_str );
-			}
-			else
-			{
-				slapi_log_access( LDAP_DEBUG_ARGS,
-								  "conn=%s op=%d RESULT err=%d"
-								  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s\n",
-									LOG_INTERNAL_OP_CON_ID,
-									LOG_INTERNAL_OP_OP_ID,
-								  err, tag, nentries, 
-								  etime, 
-								  notes_str, csn_str );
+				struct slapdplugin *plugin = NULL;
+				struct slapi_componentid *cid = NULL;
+				char *filter_str;
+				char *plugin_dn;
+				char *base_dn;
+
+				slapi_pblock_get(pb, SLAPI_SEARCH_STRFILTER, &filter_str);
+				slapi_pblock_get(pb, SLAPI_TARGET_DN, &base_dn);
+				slapi_pblock_get (pb, SLAPI_PLUGIN_IDENTITY, &cid);
+				if (cid){
+					plugin=(struct slapdplugin *) cid->sci_plugin;
+				} else {
+					slapi_pblock_get (pb, SLAPI_PLUGIN, &plugin);
+				}
+				plugin_dn = plugin_get_dn(plugin);
+
+				slapi_log_error (SLAPI_LOG_FATAL, "Internal search" , "Unindexed search: source (%s) "
+						"search base=\"%s\" filter=\"%s\" etime=%s nentries=%d %s\n",
+						plugin_dn, base_dn, filter_str, etime, nentries, notes_str);
+
+				slapi_ch_free_string(&plugin_dn);
 			}
 		}
+	}
 }
 
 
@@ -1851,3 +2138,117 @@ log_referral( Operation *op )
 		}
 	}
 }
+
+/*
+ * Generate a octet string ber-encoded searchResultEntry for
+ * pre & post Read Entry Controls
+ */
+static struct berval *
+encode_read_entry (Slapi_PBlock *pb, Slapi_Entry *e, char **attrs, int alluserattrs, int attr_count)
+{
+    Slapi_Operation *op = pb->pb_op;
+    Connection*conn = pb->pb_conn;
+    LDAPControl **ctrlp = NULL;
+    struct berval *bv = NULL;
+    BerElement *ber = NULL;
+    int real_attrs_only = 0;
+    int rc = 0;
+
+    if ( (ber = der_alloc()) == NULL ) {
+        rc = -1;
+        goto cleanup;
+    }
+
+    /* Start the ber encoding with the DN */
+    rc = ber_printf( ber, "{s{", slapi_entry_get_dn_const(e) );
+    if ( rc == -1 ) {
+        rc = -1;
+        goto cleanup;
+    }
+
+    /* determine whether we are to return virtual attributes */
+    slapi_pblock_get(pb, SLAPI_REQCONTROLS, &ctrlp);
+    if(slapi_control_present(ctrlp, LDAP_CONTROL_REAL_ATTRS_ONLY, NULL, NULL)){
+        real_attrs_only = SLAPI_SEND_VATTR_FLAG_REALONLY;
+    }
+    if(slapi_control_present(ctrlp, LDAP_CONTROL_VIRT_ATTRS_ONLY, NULL, NULL)){
+        if(real_attrs_only != SLAPI_SEND_VATTR_FLAG_REALONLY){
+            real_attrs_only = SLAPI_SEND_VATTR_FLAG_VIRTUALONLY;
+        } else {
+            /* we cannot service a request for virtual only and real only */
+            LDAPDebug( LDAP_DEBUG_ANY,"encode_read_entry: Error, both real and virtual attributes "
+                                      "only controls requested.\n", 0, 0, 0 );
+            rc = -1;
+            goto cleanup;
+        }
+    }
+
+    /*
+     * We maintain a flag array so that we can remove requests
+     * for duplicate attributes.  We also need to set o_searchattrs
+     * for the attribute processing, as modify op's don't have search attrs.
+     */
+    op->o_searchattrs = attrs;
+
+    /* Send all the attributes */
+    if ( alluserattrs ) {
+        rc = send_all_attrs(e, attrs, op, pb, ber, 0, conn->c_ldapversion,
+                            real_attrs_only, attr_count);
+        if(rc){
+            goto cleanup;
+        }
+    }
+
+    /* Send a specified list of attributes */
+    if( attrs != NULL) {
+        rc = send_specific_attrs(e, attrs, op, pb, ber, 0, conn->c_ldapversion, real_attrs_only);
+        if(rc){
+            goto cleanup;
+        }
+    }
+
+    /* wrap up the ber encoding */
+    rc = ber_printf( ber, "}}" );
+    if(rc != -1){
+        /* generate our string encoded value */
+        rc = ber_flatten(ber, &bv);
+    }
+
+cleanup:
+
+    ber_free( ber, 1 );
+    if(rc != 0){
+        return NULL;
+    } else {
+        return bv;
+    }
+}
+
+static char *
+op_to_string(int tag)
+{
+    char *op = NULL;
+
+    if(tag == LDAP_REQ_BIND){
+        op = "BIND";
+    } else if (tag == LDAP_REQ_UNBIND){
+        op = "UNBIND";
+    } else if (tag == LDAP_REQ_SEARCH){
+        op = "SEARCH";
+    } else if (tag == LDAP_REQ_ADD){
+        op = "ADD";
+    } else if (tag == LDAP_REQ_DELETE){
+        op = "DELETE";
+    } else if (tag == LDAP_REQ_RENAME){
+        op = "RENAME";
+    } else if (tag == LDAP_REQ_COMPARE){
+        op = "COMPARE";
+    } else if (tag == LDAP_REQ_ABANDON){
+        op = "ABANDON";
+    } else if (tag == LDAP_REQ_EXTENDED){
+        op = "EXTENDED";
+    }
+
+    return op;
+}
+
