@@ -392,6 +392,8 @@ Slapi_DN *slapi_sdn_init_normdn_passin(Slapi_DN *sdn, const char *dn);
 char *slapi_dn_normalize_original( char *dn );
 char *slapi_dn_normalize_case_original( char *dn );
 void ndn_cache_init();
+void ndn_cache_destroy();
+int ndn_cache_started();
 void ndn_cache_get_stats(PRUint64 *hits, PRUint64 *tries, size_t *size, size_t *max_size, long *count);
 #define NDN_DEFAULT_SIZE 20971520 /* 20mb - size of normalized dn cache */
 
@@ -498,7 +500,7 @@ typedef enum{
 #define SLAPI_ENTRY_VATTR_RESOLVED_ABSENT -2
 #define SLAPI_ENTRY_VATTR_RESOLVED_EXISTS 0
 
-int slapi_entry_vattrcache_merge_sv(Slapi_Entry *e, const char *type, Slapi_ValueSet *vals);
+int slapi_entry_vattrcache_merge_sv(Slapi_Entry *e, const char *type, Slapi_ValueSet *vals, int buffer_flags);
 int slapi_entry_vattrcache_find_values_and_type_ex( const Slapi_Entry *e,
 											const char *type,
 											Slapi_ValueSet ***results,
@@ -762,6 +764,9 @@ char * slapi_schema_get_superior_name(const char *ocname_or_oid);
 CSN *dup_global_schema_csn();
 
 /* misc function for the chaining backend */
+#define CHAIN_ROOT_UPDATE_REJECT	0
+#define CHAIN_ROOT_UPDATE_LOCAL		1
+#define CHAIN_ROOT_UPDATE_REFERRAL	2
 char * slapi_get_rootdn();	/* return the directory manager dn in use */
 
 /* plugin interface to bulk import */
@@ -1126,6 +1131,11 @@ void schema_expand_objectclasses_nolock( Slapi_Entry *e );
                                               * reload_schemafile_lock;
                                               * no further lock needed */
 #define DSE_SCHEMA_USER_DEFINED_ONLY 0x0100  /* refresh user defined schema */
+#define DSE_SCHEMA_USE_PRIV_SCHEMA   0x0200  /* Use a provided private schema */
+
+/* */
+#define OC_CONSUMER "consumer"
+#define OC_SUPPLIER "supplier"
 
 #define SLAPI_RTN_BIT_FETCH_EXISTING_DN_ENTRY 0
 #define SLAPI_RTN_BIT_FETCH_PARENT_ENTRY 1
@@ -1178,14 +1188,6 @@ void addlenstr( lenstr *l, const char *str );
 void lenstr_free( lenstr ** );
 lenstr *lenstr_new(void);
 
-/* event queue routines and data types */
-typedef void* Slapi_Eq_Context;
-typedef void (*slapi_eq_fn_t)(time_t when, void *arg);
-Slapi_Eq_Context slapi_eq_once(slapi_eq_fn_t fn, void *arg, time_t when);
-Slapi_Eq_Context slapi_eq_repeat(slapi_eq_fn_t fn, void *arg, time_t when, unsigned long interval);
-int slapi_eq_cancel(Slapi_Eq_Context ctx);
-void *slapi_eq_get_arg (Slapi_Eq_Context ctx);
-
 /* config DN */
 char *get_config_DN(void);
 
@@ -1268,6 +1270,7 @@ int slapi_add_internal_attr_syntax( const char *name, const char *oid, const cha
 /* pw.c */
 void pw_exp_init ( void );
 int pw_copy_entry_ext(Slapi_Entry *src_e, Slapi_Entry *dest_e);
+int pw_get_ext_size(Slapi_Entry *e, size_t *size);
 
 /* op_shared.c */
 void modify_update_last_modified_attr(Slapi_PBlock *pb, Slapi_Mods *smods);
