@@ -58,12 +58,14 @@ sort_make_sort_response_control ( Slapi_PBlock *pb, int code, char *error_type)
     slapi_pblock_get(pb, SLAPI_PAGED_RESULTS_INDEX, &pr_idx);
 
     if (code == CONN_GET_SORT_RESULT_CODE) {
-        code = pagedresults_get_sort_result_code(pb->pb_conn, pr_idx);
+        code = pagedresults_get_sort_result_code(pb->pb_conn,
+                                                 pb->pb_op, pr_idx);
     } else {
         Slapi_Operation *operation;
         slapi_pblock_get (pb, SLAPI_OPERATION, &operation);
         if (op_is_pagedresults(operation)) {
-            pagedresults_set_sort_result_code(pb->pb_conn, code, pr_idx);
+            pagedresults_set_sort_result_code(pb->pb_conn,
+                                              pb->pb_op, code, pr_idx);
         }
     }
 
