@@ -176,6 +176,25 @@ static void at_disconnect(AddThread *at)
 }
 #endif
 
+#if defined(USE_OPENLDAP)
+/* need mutex around ldap_initialize - see https://fedorahosted.org/389/ticket/348 */
+static PRCallOnceType ol_init_callOnce = {0,0};
+static PRLock *ol_init_lock = NULL;
+
+static PRStatus
+internal_ol_init_init(void)
+{
+    PR_ASSERT(NULL == ol_init_lock);
+    if ((ol_init_lock = PR_NewLock()) == NULL) {
+        PRErrorCode errorCode = PR_GetError();
+        fprintf(stderr, "internal_ol_init_init PR_NewLock failed %d\n", errorCode);
+        return PR_FAILURE;
+    }
+
+    return PR_SUCCESS;
+}
+#endif
+
 static void at_bind(AddThread *at)
 {
     int ret;
@@ -185,7 +204,14 @@ static void at_bind(AddThread *at)
 
     at->ld = NULL;
     ldapurl = PR_smprintf("ldap://%s:%d", hostname, port);
+    if (PR_SUCCESS != PR_CallOnce(&ol_init_callOnce, internal_ol_init_init)) {
+        fprintf(stderr, "Could not perform internal ol_init init\n");
+        return;
+    }
+
+    PR_Lock(ol_init_lock);
     ret = ldap_initialize(&at->ld, ldapurl);
+    PR_Unlock(ol_init_lock);
     PR_smprintf_free(ldapurl);
     ldapurl = NULL;
 	if (ret) {
