@@ -253,8 +253,10 @@ static PRIntn
 filter_stuff_func(void *arg, const char *val, PRUint32 slen)
 {
     struct filter_ctx *ctx = (struct filter_ctx *)arg;
+#if defined (USE_OPENLDAP)
     struct berval escaped_filter;
     struct berval raw_filter;
+#endif
     char *buf = (char *)val;
     int extra_space;
     int filter_len = slen;
@@ -342,7 +344,7 @@ filter_stuff_func(void *arg, const char *val, PRUint32 slen)
 #else
             char *val2 = NULL;
             buf = slapi_ch_calloc(sizeof(char), filter_len*3 + 1);
-            val2 = do_escape_string(val, filter_len, buf, special_filter);
+            val2 = (char *)do_escape_string(val, filter_len, buf, special_filter);
             if(val2 == NULL){
                 LDAPDebug(LDAP_DEBUG_TRACE, "slapi_filter_sprintf: failed to escape filter value(%s)\n",val,0,0);
                 ctx->next_arg_needs_esc_norm = 0;
@@ -446,8 +448,10 @@ slapi_filter_sprintf(const char *fmt, ...)
 char*
 slapi_escape_filter_value(char* filter_str, int len)
 {
+#if defined (USE_OPENLDAP)
     struct berval escaped_filter;
     struct berval raw_filter;
+#endif
     int filter_len;
 
     /*
@@ -477,8 +481,14 @@ slapi_escape_filter_value(char* filter_str, int len)
     }
 #else
     char *buf = slapi_ch_calloc(sizeof(char), filter_len*3+1);
+    char *esc_str = (char *)do_escape_string(filter_str, filter_len, buf, special_filter);
 
-    return do_escape_string(filter_str, filter_len, buf, special_filter);
+    if(esc_str != buf){
+        slapi_ch_free_string(&buf);
+        return slapi_ch_strdup(esc_str);
+    } else {
+        return buf;
+    }
 #endif
 }
 
@@ -604,7 +614,7 @@ int slapi_mods2entry (Slapi_Entry **e, const char *idn, LDAPMod **iattrs)
         if (rc != LDAP_SUCCESS)
         {
             LDAPDebug2Args(LDAP_DEBUG_ANY,
-                "slapi_add_internal: add_values for type %s failed (rc: %d)\n",
+                "slapi_mods2entry: add_values for type %s failed (rc: %d)\n",
                 normtype, rc );
             slapi_entry_free (*e);
             *e = NULL;
@@ -1395,3 +1405,42 @@ slapi_str_to_u64(const char *s)
 		(v6 << 36) | (v7 << 32) | (v8 << 28) | (v9 << 24) | (v10 << 20) | (v11 << 16) |
 		(v12 << 12) | (v13 << 8) | (v14 << 4) | v15;
 }
+
+/* PR_GetLibraryName does almost everything we need, and unfortunately
+   a little bit more - it adds "lib" to be beginning of the library
+   name if the library name does not end with the current platform
+   DLL suffix - so
+   foo.so -> /path/foo.so
+   libfoo.so -> /path/libfoo.so
+   BUT
+   foo -> /path/libfoo.so
+   libfoo -> /path/liblibfoo.so
+   /path/libfoo -> lib/path/libfoo.so
+*/
+char *
+slapi_get_plugin_name(const char *path, const char *lib)
+{
+    const char *libstr = "/lib";
+    size_t libstrlen = 4;
+    char *fullname = PR_GetLibraryName(path, lib);
+    char *ptr = PL_strrstr(fullname, lib);
+
+    /* see if /lib was added */
+    if (ptr && ((ptr - fullname) >= libstrlen)) {
+        /* ptr is at the libname in fullname, and there is something before it */
+        ptr -= libstrlen; /* ptr now points at the "/" in "/lib" if it is there */
+        if (0 == PL_strncmp(ptr, libstr, libstrlen)) {
+            /* just copy the remainder of the string on top of here */
+            ptr++; /* ptr now points at the "l" in "/lib" - keep the "/" */
+            memmove(ptr, ptr+3, strlen(ptr+3)+1);
+        }
+    } else if ((NULL == path) && (0 == strncmp(fullname, "lib", 3))) {
+        /* 
+         * case: /path/libfoo -> lib/path/libfoo.so
+         * remove "lib".
+         */
+        memmove(fullname, fullname+3, strlen(fullname)-2);
+    }
+
+    return fullname;
+}
