@@ -500,6 +500,7 @@ valuearray_purge(Slapi_Value ***va, const CSN *csn)
 		slapi_ch_free((void**)va);
 		*va= NULL;
 	}
+
 	/* return the number of remaining values */
 	return(i);
 }
@@ -511,11 +512,11 @@ valuearray_size(Slapi_Value **va)
 	if(va!=NULL && va[0]!=NULL)
 	{
 		int i;
-	    for (i = 0; va[i]; i++)
+		for (i = 0; va[i]; i++)
 		{
 			s += value_size(va[i]);
 		}
-            s += (i + 1) * sizeof(Slapi_Value*);
+		s += (i + 1) * sizeof(Slapi_Value*);
 	}
 	return s;
 }
@@ -570,7 +571,6 @@ valuearrayfast_add_value_passin(struct valuearrayfast *vaf,Slapi_Value *v)
 	vaf->num++;
 }
 
-
 /* <=========================== Value Set =======================> */
 
 #define VALUESET_ARRAY_SORT_THRESHOLD 10
@@ -644,7 +644,7 @@ void
 valueset_set_valuearray_byval(Slapi_ValueSet *vs, Slapi_Value **addvals)
 {
 	int i, j=0;
-	slapi_valueset_init(vs);
+ 	slapi_valueset_init(vs);
 	vs->num = valuearray_count(addvals);
 	vs->max = vs->num + 1;
 	vs->va = (Slapi_Value **) slapi_ch_malloc( vs->max * sizeof(Slapi_Value *));
@@ -696,12 +696,24 @@ slapi_valueset_join_attr_valueset(const Slapi_Attr *a, Slapi_ValueSet *vs1, cons
 int
 slapi_valueset_first_value( Slapi_ValueSet *vs, Slapi_Value **v )
 {
+	if (NULL == vs) {
+		if (v) {
+			*v = NULL;
+		}
+		return 0;
+	}
 	return valuearray_first_value(vs->va,v);
 }
 
 int
 slapi_valueset_next_value( Slapi_ValueSet *vs, int index, Slapi_Value **v)
 {
+	if (NULL == vs) {
+		if (v) {
+			*v = NULL;
+		}
+		return index;
+	}
 	return valuearray_next_value(vs->va,index,v);
 }
 
@@ -728,6 +740,9 @@ slapi_valueset_isempty( const Slapi_ValueSet *vs)
 int
 valueset_isempty( const Slapi_ValueSet *vs)
 {
+	if (NULL == vs) {
+		return 1; /* true */
+	}
 	return valuearray_isempty(vs->va);
 }
 
@@ -736,16 +751,15 @@ Slapi_Value *
 slapi_valueset_find(const Slapi_Attr *a, const Slapi_ValueSet *vs, const Slapi_Value *v)
 {
 	Slapi_Value *r= NULL;
-	if(vs->num > 0 )
+	if(vs && (vs->num > 0))
 	{
 		if (vs->sorted) {
 			r = valueset_find_sorted(a,vs,v,NULL);
 		} else {
-		int i= valuearray_find(a,vs->va,v);
-		if(i!=-1)
-		{
-			r= vs->va[i];
-		}
+			int i= valuearray_find(a,vs->va,v);
+			if(i!=-1) {
+				r= vs->va[i];
+			}
 		}
 	}
 	return r;
@@ -779,22 +793,23 @@ valueset_remove_value_sorted(const Slapi_Attr *a, Slapi_ValueSet *vs, const Slap
 	}
 	return r;
 }
+
 Slapi_Value *
 valueset_remove_value(const Slapi_Attr *a, Slapi_ValueSet *vs, const Slapi_Value *v)
 {
+	Slapi_Value *r = NULL;
 	if (vs->sorted) {
-		return (valueset_remove_value_sorted(a, vs, v));
+		r = valueset_remove_value_sorted(a, vs, v);
 	} else {
-	Slapi_Value *r= NULL;
-	if(!valuearray_isempty(vs->va))
-	{
-		r= valuearray_remove_value(a, vs->va, v);
-		if (r)
-			vs->num--;
+		if(!valuearray_isempty(vs->va))
+		{
+			r= valuearray_remove_value(a, vs->va, v);
+			if (r)
+				vs->num--;
+		}
 	}
 	PR_ASSERT((vs->sorted == NULL) || (vs->num == 0) || ((vs->sorted[0] >= 0) && (vs->sorted[0] < vs->num)));
 	return r;
-	}
 }
 
 /* 
@@ -804,8 +819,7 @@ int
 valueset_purge(Slapi_ValueSet *vs, const CSN *csn)
 {
 	int r= 0;
-	if(!valuearray_isempty(vs->va))
-	{
+ 	if(!valuearray_isempty(vs->va)) {
 		/* valuearray_purge is not valueset and sorting aware,
 		 * maybe need to rewrite, at least keep the valueset 
 		 * consistent
@@ -823,7 +837,6 @@ valueset_purge(Slapi_ValueSet *vs, const CSN *csn)
 			vs->sorted = NULL;
 		}
 		PR_ASSERT((vs->sorted == NULL) || (vs->num == 0) || ((vs->sorted[0] >= 0) && (vs->sorted[0] < vs->num)));
-		
 	}
 	return 0;
 }
@@ -837,10 +850,9 @@ valueset_get_valuearray(const Slapi_ValueSet *vs)
 size_t
 valueset_size(const Slapi_ValueSet *vs)
 {
-	size_t s= 0;
-	if(!valuearray_isempty(vs->va))
-	{
-		s= valuearray_size(vs->va);
+	size_t s = 0;
+	if (vs && !valuearray_isempty(vs->va)) {
+		s = valuearray_size(vs->va);
 	}
 	return s;
 }
@@ -866,15 +878,6 @@ valueset_add_valuearray(Slapi_ValueSet *vs, Slapi_Value **addvals)
 	}
 }
 
-void
-valueset_add_valuearray_ext(Slapi_ValueSet *vs, Slapi_Value **addvals, PRUint32 flags)
-{
-	if(!valuearray_isempty(addvals))
-	{
-		slapi_valueset_add_attr_valuearray_ext (NULL, vs, addvals, valuearray_count(addvals), flags, NULL);
-	}
-}
-
 /*
  * The value is passed in by value.
  */
@@ -905,8 +908,8 @@ valueset_value_syntax_cmp( const Slapi_Attr *a, const Slapi_Value *v1, const Sla
 	 */
 	const Slapi_Value *oneval[3];
 	Slapi_Value **keyvals;
-	int rc;
- 
+	int rc = -1;
+
 	keyvals = NULL;
 	oneval[0] = v1;
 	oneval[1] = v2;
@@ -915,13 +918,13 @@ valueset_value_syntax_cmp( const Slapi_Attr *a, const Slapi_Value *v1, const Sla
 	    || keyvals == NULL
 	    || keyvals[0] == NULL || keyvals[1] == NULL)
 	{
+		/* this should never happen since always a syntax plugin to
+		 * generate the keys will be found (there exists a default plugin)
+		 * log an error and continue.
+		 */
 		LDAPDebug( LDAP_DEBUG_ANY, "valueset_value_syntax_cmp: "
 		    "slapi_attr_values2keys_sv failed for type %s\n",
 		    a->a_type, 0, 0 );
-		/* this should never happen since always a default syntax plugin
-		 * will be found. Log an error and continue
-		 */
-		rc = strcasecmp(v1->bv.bv_val, v2->bv.bv_val);
 	} else {
         	struct berval *bv1, *bv2;
 		bv1 = &keyvals[0]->bv;
@@ -1165,7 +1168,6 @@ slapi_valueset_add_attr_value_ext(const Slapi_Attr *a, Slapi_ValueSet *vs, Slapi
 	return (rc);
 }
 
-
 /*
  * The string is passed in by value.
  */
@@ -1285,48 +1287,48 @@ valueset_remove_valuearray(Slapi_ValueSet *vs, const Slapi_Attr *a, Slapi_Value
 		 * For larger valuesets the valuarray is sorted, values can be deleted individually
 		 *
 		 */
-			for ( i = 0; rc==LDAP_SUCCESS && valuestodelete[i] != NULL; ++i )
+		for ( i = 0; rc==LDAP_SUCCESS && valuestodelete[i] != NULL; ++i )
+		{
+			Slapi_Value *found = valueset_remove_value(a, vs, valuestodelete[i]);
+			if(found!=NULL)
 			{
-				Slapi_Value *found = valueset_remove_value(a, vs, valuestodelete[i]);
-				if(found!=NULL)
+				if ( va_out )
 				{
-					if ( va_out )
+					if (found->v_csnset &&
+						(flags & (SLAPI_VALUE_FLAG_PRESERVECSNSET|
+               				                       SLAPI_VALUE_FLAG_USENEWVALUE)))
 					{
-						if (found->v_csnset &&
-							(flags & (SLAPI_VALUE_FLAG_PRESERVECSNSET|
-                                      SLAPI_VALUE_FLAG_USENEWVALUE)))
-						{
-							valuestodelete[i]->v_csnset = csnset_dup (found->v_csnset);
-						}
-						if (flags & SLAPI_VALUE_FLAG_USENEWVALUE)
-						{
-							valuearrayfast_add_value_passin(&vaf_out,valuestodelete[i]);
-							valuestodelete[i] = found;
-						}
-						else
-						{
-							valuearrayfast_add_value_passin(&vaf_out,found);
-						}
+						valuestodelete[i]->v_csnset = csnset_dup (found->v_csnset);
+					}
+					if (flags & SLAPI_VALUE_FLAG_USENEWVALUE)
+					{
+						valuearrayfast_add_value_passin(&vaf_out,valuestodelete[i]);
+						valuestodelete[i] = found;
 					}
 					else
 					{
-						if (flags & SLAPI_VALUE_FLAG_PRESERVECSNSET)
-						{
-							valuestodelete[i]->v_csnset = found->v_csnset;
-							found->v_csnset = NULL;
-						}
-						slapi_value_free ( & found );
+						valuearrayfast_add_value_passin(&vaf_out,found);
 					}
 				}
 				else
 				{
-					if((flags & SLAPI_VALUE_FLAG_IGNOREERROR) == 0)
+					if (flags & SLAPI_VALUE_FLAG_PRESERVECSNSET)
 					{
-						LDAPDebug( LDAP_DEBUG_ARGS,"could not find value %d for attr %s\n", i-1, a->a_type, 0 );
-						rc= LDAP_NO_SUCH_ATTRIBUTE;
+						valuestodelete[i]->v_csnset = found->v_csnset;
+						found->v_csnset = NULL;
 					}
+					slapi_value_free ( & found );
 				}
 			}
+			else
+			{
+				if((flags & SLAPI_VALUE_FLAG_IGNOREERROR) == 0)
+				{
+					LDAPDebug( LDAP_DEBUG_ARGS,"could not find value %d for attr %s\n", i-1, a->a_type, 0 );
+					rc= LDAP_NO_SUCH_ATTRIBUTE;
+				}
+			}
+		}
 		if ( va_out )
 		{
 			*va_out= vaf_out.va;
@@ -1429,6 +1431,11 @@ valueset_replace_valuearray_ext(Slapi_Attr *a, Slapi_ValueSet *vs, Slapi_Value *
  */
 void
 valueset_update_csn_for_valuearray(Slapi_ValueSet *vs, const Slapi_Attr *a, Slapi_Value **valuestoupdate, CSNType t, const CSN *csn, Slapi_Value ***valuesupdated)
+{
+	valueset_update_csn_for_valuearray_ext(vs, a, valuestoupdate, t, csn, valuesupdated, 0);
+}
+void
+valueset_update_csn_for_valuearray_ext(Slapi_ValueSet *vs, const Slapi_Attr *a, Slapi_Value **valuestoupdate, CSNType t, const CSN *csn, Slapi_Value ***valuesupdated, int csnref_updated)
 {
 	if(!valuearray_isempty(valuestoupdate) &&
 		!valuearray_isempty(vs->va))
@@ -1443,6 +1450,8 @@ valueset_update_csn_for_valuearray(Slapi_ValueSet *vs, const Slapi_Attr *a, Slap
 			if(index!=-1)
 			{
 				value_update_csn(vs->va[index],t,csn);
+				if (csnref_updated)
+					valuestoupdate[i]->v_csnset = (CSNSet *)value_get_csnset(vs->va[index]);
 				valuearrayfast_add_value_passin(&vaf_valuesupdated,valuestoupdate[i]);
 				valuestoupdate[i]= NULL;
 				del_count++;
@@ -1476,3 +1485,4 @@ valuearray_dn_normalize_value(Slapi_Value **vals)
 
 	return rc;
 }
+
